/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

import java.io.File;
import java.io.IOException;
import java.lang.invoke.CallSite;
import java.lang.invoke.MethodHandles;
import java.lang.invoke.MethodType;
import teajay.bytecode.ClassAccessFlag;
import teajay.bytecode.ClassFile;
import teajay.bytecode.attributes.bootstrap.BootstrapMethodEntry;
import teajay.bytecode.attributes.bootstrap.BootstrapMethods;
import teajay.bytecode.attributes.code.AConstNull;
import teajay.bytecode.attributes.code.InvokeDynamic;
import teajay.bytecode.attributes.code.InvokeStatic;
import teajay.bytecode.attributes.code.InvokeVirtual;
import teajay.bytecode.attributes.code.JVMInstructionList;
import teajay.bytecode.attributes.code.Return;
import teajay.bytecode.cpool.ClassInfo;
import teajay.bytecode.cpool.FieldDescriptor;
import teajay.bytecode.cpool.InvokeDynamicInfo;
import teajay.bytecode.cpool.MethodDescriptor;
import teajay.bytecode.cpool.MethodHandleInfo;
import teajay.bytecode.cpool.MethodRefInfo;
import teajay.bytecode.cpool.NameAndTypeInfo;
import teajay.bytecode.exceptions.ByteCodeException;
import teajay.bytecode.methods.MethodAccessFlag;
import teajay.bytecode.methods.MethodInfo;
import teajay.bytecode.methods.MethodKind;

/**
 * To run the Program use:
 * java -cp [...]/TeaJayRuntime/build/classes:.:[...]/TeaJayUtils/build/classes InvokeDynamic
 * @author Michael Kußmann <michi@outx.de>
 */
public class InvokeDynamicGenerator {

    public static void main(String[] args) throws ByteCodeException, IOException {
        ClassFile dyn = new ClassFile("InvokeDynamic", ClassAccessFlag.PUBLIC, ClassAccessFlag.SUPER);
        MethodInfo main = new MethodInfo("main",
                new MethodDescriptor(FieldDescriptor.getVoid(), FieldDescriptor.fromClass(String[].class)),
                dyn.getThis(), MethodAccessFlag.STATIC, MethodAccessFlag.PUBLIC);
        dyn.addMethod(main);
        JVMInstructionList lst = main.getCode().getInstructionList();

        BootstrapMethods boot = new BootstrapMethods();
        dyn.addAttribute(boot);
        MethodRefInfo ref = MethodRefInfo.make("teajay/runtime/Bootstraps", "bc",
                FieldDescriptor.fromClass(CallSite.class), FieldDescriptor.fromClass(MethodHandles.Lookup.class),
                FieldDescriptor.getString(), FieldDescriptor.fromClass(MethodType.class), FieldDescriptor.fromClass(Class.class));
        BootstrapMethodEntry bootstrapMethod = boot.addBootstrapMethod(
                new MethodHandleInfo<>(MethodKind.REF_invokeStatic, ref), new ClassInfo("teajay/lang/List"));
        lst.add(new AConstNull());
        FieldDescriptor listDescr = FieldDescriptor.fromString("teajay/lang/List");
        lst.add(new InvokeDynamic(new InvokeDynamicInfo(bootstrapMethod,
                new NameAndTypeInfo<>("anyName", new MethodDescriptor(listDescr, listDescr)))));

        lst.add(new InvokeVirtual(MethodRefInfo.make("java/lang/Object", "getClass", "java/lang/Class")));
        lst.add(new InvokeStatic(MethodRefInfo.make(new ClassInfo("teajay/lang/IO"),
                "println", FieldDescriptor.getVoid(), FieldDescriptor.getObject())));
        lst.add(new Return());
        dyn.write(new File("InvokeDynamic.class"));
    }
}
