/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package teajay.bytecode;

import java.io.DataOutputStream;
import java.io.IOException;
import teajay.bytecode.cpool.ConstantPool;
import teajay.bytecode.exceptions.ByteCodeException;

/**
 * signed byte
 * @author Michael Kußmann <michi@outx.de>
 */
public final class S1 implements Operand {

    public static final S1 ZERO = new S1();
    private final int value;
    
    private S1() {
        this.value = 0;
    }

    public S1(int value) throws ByteCodeException {
        if (value > Byte.MAX_VALUE || value < Byte.MIN_VALUE) {
            throw new ByteCodeException("signed short (S1) out of bounds: " + value);
        }
        this.value = value;
    }

    public S1(long value) throws ByteCodeException {
        if (value > Byte.MAX_VALUE || value < Byte.MIN_VALUE) {
            throw new ByteCodeException("signed short (S1) out of bounds: " + value);
        }
        this.value = (int) value;
    }

    public final int getValue() {
        return value;
    }
    
    @Override
    public long getAddress(ConstantPool cpool) {
        return value;
    }

    @Override
    public void write(DataOutputStream out, ConstantPool cpool) throws IOException {
        out.writeByte(value);
    }

    @Override
    public void writeIndex(DataOutputStream out, ConstantPool cpool) throws IOException {
        this.write(out, cpool);
    }

    @Override
    public OpCode.OpSize getDefaultLengthAsOperand() {
        return OpCode.OpSize.U1;
    }

    @Override
    public String toString() {
        return "S1: " + value;
    }
    
    @Override
    public boolean equals(Object obj) {
        if (obj instanceof S1) {
            S1 other = (S1) obj;
            return other.value == value;
        }
        return false;
    }

    @Override
    public int hashCode() {
        int hash = 7;
        hash = 67 * hash + this.value;
        return hash;
    }
}
