/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package teajay.bytecode;

import java.io.DataOutputStream;
import java.io.IOException;
import teajay.bytecode.OpCode.OpSize;
import teajay.bytecode.cpool.ConstantPool;
import teajay.bytecode.exceptions.ByteCodeException;

/**
 * signed short
 * @author Michael Kußmann <michi@outx.de>
 */
public final class S2 implements Operand {

    public static final S2 ZERO = new S2();
    private final int value;
    
    private S2() {
        this.value = 0;
    }

    public S2(int value) throws ByteCodeException {
        if (value > Short.MAX_VALUE || value < Short.MIN_VALUE) {
            throw new ByteCodeException("signed short (S2) out of bounds: " + value);
        }
        this.value = value;
    }

    public S2(long value) throws ByteCodeException {
        if (value > Short.MAX_VALUE || value < Short.MIN_VALUE) {
            throw new ByteCodeException("signed short (S2) out of bounds: " + value);
        }
        this.value = (int) value;
    }

    public final int getValue() {
        return value;
    }
    
    @Override
    public long getAddress(ConstantPool cpool) {
        return value;
    }

    @Override
    public void write(DataOutputStream out, ConstantPool cpool) throws IOException {
        out.writeShort(value);
    }

    @Override
    public void writeIndex(DataOutputStream out, ConstantPool cpool) throws IOException {
        this.write(out, cpool);
    }

    @Override
    public OpSize getDefaultLengthAsOperand() {
        return OpSize.U2;
    }

    @Override
    public String toString() {
        return "S2: " + value;
    }
    
    @Override
    public boolean equals(Object obj) {
        if (obj instanceof S2) {
            S2 other = (S2) obj;
            return other.value == value;
        }
        return false;
    }

    @Override
    public int hashCode() {
        int hash = 7;
        hash = 59 * hash + this.value;
        return hash;
    }
}
