/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package teajay.bytecode.attributes;

import java.io.DataInputStream;
import java.io.IOException;
import teajay.bytecode.U4;
import teajay.bytecode.cpool.ConstantPool;
import teajay.bytecode.cpool.Utf8Info;
import teajay.bytecode.exceptions.ByteCodeException;

/**
 * Implementation of the Deprecated attribute as stated in "The JavaTM Virtual
 * Machine Specification Version 7". Revision: 27.07.2012
 *
 * @author Michael Kußmann <michi@outx.de>
 */
public final class Deprecated extends AttributeInfo {
    public static final String NAME = "Deprecated";
    
    public Deprecated() {
        this(new Utf8Info(NAME));
    }
    
    private Deprecated(Utf8Info name) {
        super(name);
        checkName(NAME, name);
    }

    @Override
    public U4 getLength(ConstantPool cpool) throws ByteCodeException {
        return U4.ZERO;
    }
    
    public static Deprecated read(DataInputStream in, ConstantPool cpool, int major, int minor, int length, Utf8Info name) throws ByteCodeException, IOException {
        return new Deprecated(name);
    }
}
