/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package teajay.bytecode.attributes;

import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import teajay.bytecode.U4;
import teajay.bytecode.cpool.ConstantPool;
import teajay.bytecode.cpool.Utf8Info;
import teajay.bytecode.exceptions.ByteCodeException;
import static teajay.bytecode.util.Tools.ccpt;

/**
 * Implementation of the SourceFile attribute as stated in "The JavaTM Virtual
 * Machine Specification Version 7". Revision: 27.07.2012
 *
 * @author Michael Kußmann <michi@outx.de>
 */
public final class SourceFile extends AttributeInfo {
    public static final String NAME = "SourceFile";

    private final Utf8Info sourceFile;
    
    private SourceFile(Utf8Info name, Utf8Info sourceFile) {
        super(name);
        checkName(NAME, name);
        this.sourceFile = sourceFile;
    }

    public SourceFile(String sourceFile) {
        this(new Utf8Info(NAME), new Utf8Info(sourceFile));
    }

    @Override
    public U4 getLength(ConstantPool cpool) throws ByteCodeException {
        return new U4(2);
    }
    
    public String getSourceFile() {
        return this.sourceFile.getValue();
    }

    @Override
    public void write(DataOutputStream out, ConstantPool cpool) throws ByteCodeException, IOException {
        super.write(out, cpool);
        sourceFile.writeIndex(out, cpool);
    }

    @Override
    public void add(ConstantPool cpool) throws ByteCodeException {
        super.add(cpool);
        sourceFile.add(cpool);
    }
    
    public static SourceFile read(DataInputStream in, ConstantPool cpool, int major, int minor, int length, Utf8Info name) throws ByteCodeException, IOException {
        Utf8Info sourceFile = ccpt(cpool.get(in.readUnsignedShort()), Utf8Info.class, cpool);
        return new SourceFile(name, sourceFile);
    }
}
