/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package teajay.bytecode.attributes.annotations;

import java.io.DataInputStream;
import java.io.IOException;
import java.util.List;
import teajay.bytecode.cpool.ConstantPool;
import teajay.bytecode.cpool.Utf8Info;
import teajay.bytecode.exceptions.ByteCodeException;

/**
 *
 * @author Michael Kußmann <michi@outx.de>
 */
public class RuntimeInvisibleParameterAnnotations extends RuntimeParameterAnnotation {

    public static final String NAME = "RuntimeInvisibleParameterAnnotations";

    public RuntimeInvisibleParameterAnnotations() {
        super(new Utf8Info(NAME));
    }
       
    private RuntimeInvisibleParameterAnnotations(Utf8Info name, List<List<Annotation>> annotations) {
        super(name, annotations);
        checkName(NAME, name);
    }
    
    public static RuntimeInvisibleParameterAnnotations read(DataInputStream in, ConstantPool cpool, int major, int minor, int length, Utf8Info name) throws ByteCodeException, IOException {
        List<List<Annotation>> annotations = RuntimeParameterAnnotation.read(in, cpool, major, minor);
        return new RuntimeInvisibleParameterAnnotations(name, annotations);
    }
}
