/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package teajay.bytecode.attributes.code;

import teajay.bytecode.OpCode;
import teajay.bytecode.U1;
import teajay.bytecode.U2;
import teajay.bytecode.exceptions.ByteCodeException;

/**
 *
 * @author Michael Kußmann <michi@outx.de>
 */
public class IConst extends SimpleInstruction {

    public IConst(int n) throws ByteCodeException {
        switch (n) {
            case -1:
                this.setOpcode(OpCode.iconst_m1);
                break;
            case 0:
                this.setOpcode(OpCode.iconst_0);
                break;
            case 1:
                this.setOpcode(OpCode.iconst_1);
                break;
            case 2:
                this.setOpcode(OpCode.iconst_2);
                break;
            case 3:
                this.setOpcode(OpCode.iconst_3);
                break;
            case 4:
                this.setOpcode(OpCode.iconst_4);
                break;
            case 5:
                this.setOpcode(OpCode.iconst_5);
                break;
            default:
                if (n >= Byte.MIN_VALUE && n <= Byte.MAX_VALUE) {
                    this.setOpcode(OpCode.bipush);
                    this.setOperands(new U1(n));
                } else if (n >= Short.MIN_VALUE && n <= Short.MAX_VALUE) {
                    this.setOpcode(OpCode.sipush);
                    this.setOperands(new U2(n));
                } else {
                    throw new ByteCodeException("iconst: value out of bounds " + n);
                }
        }
    }

    @Override
    int getLocalModifier() {
        return 0;
    }
}
