/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package teajay.bytecode.attributes.code;

import teajay.bytecode.OpCode;
import teajay.bytecode.U1;
import teajay.bytecode.cpool.InterfaceMethodRefInfo;
import teajay.bytecode.exceptions.ByteCodeException;

/**
 *
 * @author Michael Kußmann <michi@outx.de>
 */
public class InvokeInterface extends InvokeInstruction {
    private final int pop;
    
    public InvokeInterface(InterfaceMethodRefInfo method) throws ByteCodeException {
        super(method, OpCode.invokeinterface);
        this.pop = super.getPop();
        if (pop > 0xFF) {
            throw new ByteCodeException("invokeinterface: count argument out of bounds: " + pop);
        }
        this.setOperands(method, new U1(pop), U1.ZERO);
    }
    
    @Override
    int getPop() {
        return pop;
    }

    @Override
    int getLocalModifier() {
        return 0;
    }
}
