/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package teajay.bytecode.attributes.code;

import java.io.DataOutputStream;
import java.io.IOException;
import java.util.List;
import teajay.bytecode.U1;
import teajay.bytecode.U4;
import teajay.bytecode.cpool.ConstantPool;
import teajay.bytecode.exceptions.ByteCodeException;

/**
 *
 * @author Michael Kußmann <michi@outx.de>
 */
final class SameLocalsOneStackItemExtendedFrame extends StackVerifierStackMapFrame {
    
    static final int FRAME_TYPE = 247;
    
    SameLocalsOneStackItemExtendedFrame(JVMInstruction start) {
        super(start, new VerificationTypeList(1));
    }
    
    SameLocalsOneStackItemExtendedFrame(JVMInstruction start, VerificationType type) throws ByteCodeException {
        super(start, new VerificationTypeList(1));
        this.getStackTypeList().add(type);
    }    

    @Override
    U1 getFrameType() throws ByteCodeException {
        return new U1(247);
    }

    @Override
    U4 getLength() throws ByteCodeException {
        return new U4(1 + 2 + this.getStackTypeList().getLength().getValue());
    }
    
    @Override
    public void write(DataOutputStream out, ConstantPool cpool) throws IOException, ByteCodeException {
        if (getStackTypeList().isEmpty()) {
            throw new ByteCodeException("Cannot have an empty verification type list.");
        }
        super.write(out, cpool);
        offsetDelta().write(out, cpool);
        getStackTypeList().write(out, cpool);
    }

    @Override
    List<VerificationType> getStack() {
        return getStackTypeList().getList();
    }

    @Override
    List<VerificationType> getLocals() throws ByteCodeException {
        return this.getPredecessor().getLocals();
    }
}
