/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package teajay.bytecode.cpool;

import java.io.DataOutputStream;
import java.io.IOException;
import java.util.Objects;
import teajay.bytecode.exceptions.ByteCodeException;

/**
 *
 * @author Michael Kußmann <michi@outx.de>
 * @param <T>
 */
public abstract class CPoolRef<T extends Utf8Info> extends CPoolEntry {
    private final ClassInfo className;
    private final NameAndTypeInfo<T> nameAndType;
    
    protected CPoolRef(CPoolTag tag, ClassInfo className, NameAndTypeInfo<T> nameAndType) {
        super(tag);
        switch (tag) {
            case FieldRef:
            case MethodRef:
            case InterfaceMethodRef:
                break;
            default:
                throw new IllegalArgumentException("invalid ref type: " + tag);
        }
        this.className = className;
        this.nameAndType = nameAndType;
    }
    
    public ClassInfo getRefClass() {
        return this.className;
    }
    
    public NameAndTypeInfo<T> getNameAndType() {
        return this.nameAndType;
    }
    
    @Override
    public void write(DataOutputStream out, ConstantPool cpool) throws IOException, ByteCodeException {
        super.write(out, cpool);
        className.getCIndex(cpool).write(out, cpool);
        nameAndType.getCIndex(cpool).write(out, cpool);
    }
    
    @Override
    public String toString() {
        return this.getClass().getSimpleName() + " (className = " + className + ", nameAndType = " + nameAndType + ")";
    }
    
    @Override
    public boolean equals(Object obj) {
        if ((obj instanceof CPoolRef) && super.equals(obj)) {
            CPoolRef<?> other = (CPoolRef<?>) obj;
            return other.getRefClass().equals(getRefClass()) && other.getNameAndType().equals(getNameAndType());
        } else {
            return false;
        }
    }

    @Override
    public int hashCode() {
        int hash = 23 * super.hashCode();
        hash = 17 * hash + Objects.hashCode(this.className);
        hash = 17 * hash + Objects.hashCode(this.nameAndType);
        return hash;
    }
    
    @Override
    public void add(ConstantPool pool) throws ByteCodeException {
        super.add(pool);
        className.add(pool);
        nameAndType.add(pool);
    }
}
