/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package teajay.bytecode.cpool;

import teajay.bytecode.exceptions.MalformedClassNameException;

/**
 *
 * @author Michael Kußmann <michi@outx.de>
 */
public final class InterfaceMethodRefInfo extends MethodRefInfo {
    
    public static InterfaceMethodRefInfo make(ClassInfo className, String name, FieldDescriptor returnType, FieldDescriptor... parameterTypes) {
        FieldDescriptor[] copy = new FieldDescriptor[parameterTypes.length];
        for (int i = 0; i < parameterTypes.length; i++) {
            copy[i] = parameterTypes[i];
        }
        return new InterfaceMethodRefInfo(className, new NameAndTypeInfo<>(name, new MethodDescriptor(returnType, copy)));
    }
    
    public static InterfaceMethodRefInfo make(String className, String name, FieldDescriptor returnType, FieldDescriptor... parameterTypes) throws MalformedClassNameException {
        FieldDescriptor[] copy = new FieldDescriptor[parameterTypes.length];
        for (int i = 0; i < parameterTypes.length; i++) {
            copy[i] = parameterTypes[i];
        }
        return new InterfaceMethodRefInfo(className, new NameAndTypeInfo<>(name, new MethodDescriptor(returnType, copy)));
    }

    public static InterfaceMethodRefInfo make(String className, String name, String returnType, String... parameterTypes) throws MalformedClassNameException {
        FieldDescriptor[] parameters = new FieldDescriptor[parameterTypes.length];
        for (int i = 0; i < parameters.length; i++) {
            parameters[i] = FieldDescriptor.fromString(parameterTypes[i]);
        }
        return new InterfaceMethodRefInfo(className, new NameAndTypeInfo<>(name, new MethodDescriptor(FieldDescriptor.fromString(returnType), parameters)));
    }

    public InterfaceMethodRefInfo(ClassInfo className, NameAndTypeInfo<MethodDescriptor> nameAndType) {
        super(CPoolTag.InterfaceMethodRef, className, nameAndType);
    }
    
    public InterfaceMethodRefInfo(String className, NameAndTypeInfo<MethodDescriptor> nameAndType) throws MalformedClassNameException {
        super(CPoolTag.InterfaceMethodRef, new ClassInfo(className), nameAndType);
    }
    
    @Override
    public boolean equals(Object obj) {
        return (obj instanceof InterfaceMethodRefInfo) && super.equals(obj);
    }

    @Override
    public int hashCode() {
        return 59 * super.hashCode();
    }
}
