/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package teajay.bytecode.cpool;

import java.io.DataOutputStream;
import java.io.IOException;
import java.util.Objects;
import teajay.bytecode.exceptions.ByteCodeException;

/**
 *
 * @author Michael Kußmann <michi@outx.de>
 */
abstract class U1U2Entry<T extends CPoolEntry> extends CPoolEntry {

    private final T reference;

    protected U1U2Entry(CPoolTag tag, T reference) {
        super(tag);
        this.reference = reference;
    }

    public final T getValue() {
        return reference;
    }

    @Override
    public void write(DataOutputStream out, ConstantPool cpool) throws IOException, ByteCodeException {
        super.write(out, cpool);
        reference.getCIndex(cpool).write(out, cpool);
    }

    @Override
    public String toString() {
        return this.getClass().getSimpleName() + ": references=" + getValue();
    }

    @Override
    public boolean equals(Object obj) {
        if ((obj instanceof U1U2Entry) && super.equals(obj)) {
            return ((U1U2Entry) obj).getValue().equals(getValue());
        } else {
            return false;
        }
    }

    @Override
    public int hashCode() {
        int hash = super.hashCode();
        hash = 23 * hash + Objects.hashCode(this.reference);
        return hash;
    }

    @Override
    public void add(ConstantPool pool) throws ByteCodeException {
        super.add(pool);
        reference.add(pool);
    }
}
