/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package teajay.bytecode.cpool;

import java.io.DataOutputStream;
import java.io.IOException;
import teajay.bytecode.exceptions.ByteCodeException;

/**
 *
 * @author Michael Kußmann <michi@outx.de>
 */
public abstract class U8Info extends CPoolEntry {
    private final long value;
    
    protected U8Info(CPoolTag tag, long value) {
        super(tag);
        this.value = value;
    }
    
    public long getValue() {
        return value;
    }

    @Override
    public void write(DataOutputStream out, ConstantPool cpool) throws IOException, ByteCodeException {
        super.write(out, cpool);
        out.writeLong(value);
    }
    
    @Override
    public boolean equals(Object obj) {
        if ( (obj instanceof U8Info) && super.equals(obj) ) {
            return ((U8Info) obj).getValue() == getValue();
        } else {
            return false;
        }
    }

    @Override
    public int hashCode() {
        int hash = 7 * super.hashCode();
        hash = 89 * hash + (int) (this.value ^ (this.value >>> 32));
        return hash;
    }
}
