/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package abstractsyntaxtree;

import codegeneration.Visitor;
import java.util.List;
import lexer.Symbol;
import exceptions.CompilerException;

/**
 *
 * @author Michael Burzan
 */
public class Expression3Node extends SyntaxNode {

    private Expression3Node exp3;
    private Expression1Node assignExp1;
    private Expression1RestNode exp1Rest;
    private Expression2RestNode exp2Rest;
    private Et1Node et1Expression;
    private Expression3Node exp;
    private PrefixOpNode prefixOp;
    private PrimaryNode primary;
    private AssignmentOperatorNode assignOp;
    private List<SelectorNode> lSelector;
    private List<PostfixOpNode> lPostfixOp;

    public boolean hasAssignment() {
        if (assignOp == null) {
            return false;
        }
        return true;
    }

    public boolean hasInfix() {
        if (exp2Rest == null) {
            return false;
        }
        return true;
    }

    public boolean isPrimary() {
        return this.primary == null ? false : true;
    }
    
    public AssignmentOperatorNode.Operators getAssignOp(){
        return this.assignOp.getOperator();
    }
    

    public PrimaryNode getPrimaryNode() {
        return this.primary;
    }

    public Expression3Node(Symbol s, PrefixOpNode prefixOp, Expression3Node exp, Expression2RestNode exp2Rest,
            Expression1RestNode exp1Rest, AssignmentOperatorNode assignOp,
            Expression1Node assignExp1, Et1Node et1Expression, Expression3Node exp3) {
        super(s);
        this.prefixOp = prefixOp;
        this.exp = exp;
        this.exp2Rest = exp2Rest;
        this.exp1Rest = exp1Rest;
        this.assignOp = assignOp;
        this.assignExp1 = assignExp1;
        this.et1Expression = et1Expression;
        this.exp3 = exp3;
    }

    /*
     * Primary {Selector} {PostfixOp} [expression2Rest] [expression1Rest] 
     *                  [AssignmentOperator expression1] ET1 expression3
     */
    public Expression3Node(Symbol s, PrimaryNode primary, List<SelectorNode> lSelector, List<PostfixOpNode> lPostfixOp,
            Expression2RestNode exp2, Expression1RestNode exp1, AssignmentOperatorNode assignOp,
            Expression1Node assignExp, Et1Node et1, Expression3Node exp3) {
        super(s);
        this.primary = primary;
        this.lSelector = lSelector;
        this.lPostfixOp = lPostfixOp;
        this.exp2Rest = exp2;
        this.exp1Rest = exp1;
        this.assignOp = assignOp;
        this.assignExp1 = assignExp;
        this.et1Expression = et1;
        this.exp3 = exp3;

    }

    @Override
    public void accept(Visitor visitor) throws CompilerException {
        visitor.begin(this);
        Visitor v = visitor.visit(this);
        checkNull(primary, v);
        checkNull(lSelector, v);
        checkNull(lPostfixOp, v);
        checkNull(prefixOp, v);
        checkNull(exp, v);
        checkNull(exp2Rest, v);
        checkNull(exp1Rest, v);
        checkNull(assignOp, v);
        checkNull(assignExp1, v);
        checkNull(et1Expression, v);
        checkNull(exp3, v);
        visitor.end(this);
    }
}
