/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package abstractsyntaxtree;

import codegeneration.Visitor;
import lexer.Symbol;
import exceptions.CompilerException;

/**
 *
 * @author Michael Burzan Statement class is divided in many helper classes.
 */
public class StatementNode extends SyntaxNode {

    private BlockNode block = null;
    private boolean hasSemicolon = false;
    private IdentDoublePointStatementNode identStatement = null;
    private StatementExpressionNode statementExpression = null;
    private IfStatementNode ifStatement = null;
    private AssertStatementNode assertStatement = null;
    private SwitchStatementNode switchStatement = null;
    private WhileStatementNode whileStatement = null;
    private DoStatementNode doStatement = null;
    private ForStatementNode forStatement = null;
    private BreakStatementNode breakStatement = null;
    private ContinueStatementNode continueStatement = null;
    private ReturnStatementNode returnStatement = null;
    private ThrowStatementNode throwStatement = null;
    private SynchronizedStatementNode synchronizedStatement = null;
    private TryStatementNode tryStatement = null;
    private TypeSwitchStatementNode typeSwitchStatement = null;

    public boolean hasSemicolon() {
        return this.hasSemicolon;
    }

    public StatementNode(Symbol s, BlockNode block) {
        super(s);
        this.block = block;
    }

    public StatementNode(Symbol s, boolean hasSemicolon) {
        super(s);
        this.hasSemicolon = hasSemicolon;
    }

    public StatementNode(Symbol s, IdentDoublePointStatementNode identStatement) {
        super(s);
        this.identStatement = identStatement;
    }

    public StatementNode(Symbol s, StatementExpressionNode statementExpression) {
        super(s);
        this.statementExpression = statementExpression;
    }

    public StatementNode(Symbol s, IfStatementNode ifStatement) {
        super(s);
        this.ifStatement = ifStatement;
    }

    public StatementNode(Symbol s, AssertStatementNode assertStatement) {
        super(s);
        this.assertStatement = assertStatement;
    }

    public StatementNode(Symbol s, SwitchStatementNode switchStatement) {
        super(s);
        this.switchStatement = switchStatement;
    }

    public StatementNode(Symbol s, WhileStatementNode whileStatement) {
        super(s);
        this.whileStatement = whileStatement;
    }

    public StatementNode(Symbol s, DoStatementNode doStatement) {
        super(s);
        this.doStatement = doStatement;
    }

    public StatementNode(Symbol s, ForStatementNode forStatement) {
        super(s);
        this.forStatement = forStatement;
    }

    public StatementNode(Symbol s, BreakStatementNode brakStatement) {
        super(s);
        this.breakStatement = brakStatement;
    }

    public StatementNode(Symbol s, ContinueStatementNode continueStatement) {
        super(s);
        this.continueStatement = continueStatement;
    }

    public StatementNode(Symbol s, ReturnStatementNode returnStatement) {
        super(s);
        this.returnStatement = returnStatement;
    }

    public StatementNode(Symbol s, ThrowStatementNode throwStatement) {
        super(s);
        this.throwStatement = throwStatement;
    }

    public StatementNode(Symbol s, SynchronizedStatementNode synchronizedStatement) {
        super(s);
        this.synchronizedStatement = synchronizedStatement;
    }

    public StatementNode(Symbol s, TryStatementNode tryStatement) {
        super(s);
        this.tryStatement = tryStatement;
    }

    public StatementNode(Symbol s, TypeSwitchStatementNode typeSwitchStatement) {
        super(s);
        this.typeSwitchStatement = typeSwitchStatement;
    }

    @Override
    public void accept(Visitor visitor) throws CompilerException {
        visitor.begin(this);
        Visitor v = visitor.visit(this);
        checkNull(this.assertStatement, v);
        checkNull(this.block, v);
        checkNull(this.breakStatement, v);
        checkNull(this.continueStatement, v);
        checkNull(this.doStatement, v);
        checkNull(this.forStatement, v);
        checkNull(this.identStatement, v);
        checkNull(this.ifStatement, v);
        checkNull(this.returnStatement, v);
        checkNull(this.statementExpression, v);
        checkNull(this.switchStatement, v);
        checkNull(this.synchronizedStatement, v);
        checkNull(this.throwStatement, v);
        checkNull(this.tryStatement, v);
        checkNull(this.whileStatement, v);
        checkNull(this.typeSwitchStatement, v);
        visitor.end(this);
    }
}
