/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package abstractsyntaxtree;

import codegeneration.Visitor;
import exceptions.CompilerException;
import java.util.List;
import lexer.LexerWrapper;
import lexer.Symbol;

/**
 *
 * @author Michael Burzan
 */
public abstract class SyntaxNode {

    public static LexerWrapper lexer;
    private int lineNr;
    private final Symbol symbol;

    public SyntaxNode() {
        this.symbol = null;
        this.lineNr = Integer.MIN_VALUE;
    }

    public SyntaxNode(Symbol s) {
        this.lineNr = s.line;
        this.symbol = s;
    }

    public int getLine() {
        return this.lineNr;
    }

    public Symbol getSymbol() {
        return this.symbol;
    }

    public abstract void accept(Visitor visitor) throws CompilerException;

    /*Is true if node null*/
    protected boolean checkNull(SyntaxNode n, Visitor v) throws CompilerException {
        if (v == null) {
            return true;
        }
        if (n != null) {
            n.accept(v);
            return false;
        }
        return true;
    }

    protected boolean checkNull(List<? extends SyntaxNode> list, Visitor v) throws CompilerException {
        if (v == null) {
            return true;
        }
        if (list != null) {
            for (SyntaxNode s : list) {
                s.accept(v);
            }
            return false;
        }
        return true;
    }
}
