/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package abstractsyntaxtree;

import codegeneration.Visitor;
import java.util.ArrayList;
import lexer.Symbol;
import exceptions.CompilerException;
import teajay.generics.parser.Type;

/**
 *
 * @author Michael Burzan
 */
public class TypeNode extends SyntaxNode {

    private BasicTypeNode basicType = null;
    private ReferenceTypeNode referenceType = null;
    private ArrayList<EdgeClampsOpenCloseHelperNode> edgeClampsOpenCloseList = null;
    private ClosureTypeDecNode closureDec = null;
    private Type type;

    public TypeNode(Symbol s, BasicTypeNode basicType, ReferenceTypeNode referenceType,
            ArrayList<EdgeClampsOpenCloseHelperNode> edgeClampsOpenCloseList) {
        super(s);
        this.basicType = basicType;
        this.referenceType = referenceType;
        this.edgeClampsOpenCloseList = edgeClampsOpenCloseList;

    }

    public TypeNode(Symbol s, ClosureTypeDecNode closureDec) {
        super(s);
        this.closureDec = closureDec;
    }

    //is not array dim is 0
    public int getArrayDim() {
        if (edgeClampsOpenCloseList != null) {
            return this.edgeClampsOpenCloseList.size();
        }
        return 0;
    }

    public boolean isReferenceType() {
        return referenceType == null ? false : true;
    }

    public boolean isClosureType() {
        return closureDec == null ? false : true;
    }

    @Override
    public void accept(Visitor visitor) throws CompilerException {
        visitor.begin(this);
        Visitor v = visitor.visit(this);
        checkNull(this.basicType, v);
        checkNull(this.referenceType, v);
        checkNull(this.closureDec, v);
        visitor.end(this);
    }

    public void setType(Type type) {
        this.type = type;
    }
    public Type getType(){
        return this.type;
    }
    
    public BasicTypeNode getBasicType() {
        return basicType;
    }

    public ReferenceTypeNode getReferenceType() {
        return referenceType;
    }

    public ArrayList<EdgeClampsOpenCloseHelperNode> getEdgeClampsOpenCloseList() {
        return edgeClampsOpenCloseList;
    }

    public ClosureTypeDecNode getClosureDec() {
        return closureDec;
    }
}
