/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package codegeneration.environment;

import abstractsyntaxtree.SyntaxNode;
import datastructs.Pair;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import minicompiler.WorkingNode;
import symbol.SymbolFacade;
import teajay.bytecode.ClassFile;
import teajay.util.QIDTools;

/**
 *
 * @author Michael Burzan
 * Environment to manage all Types and generics
 */
public class Stage1Environment implements Comparable<Stage1Environment> {

    private final Stage0Environment env;
    private HashSet<WorkingNode> workingNodeList = new HashSet<>();
    private HashSet<String> typeHash = new HashSet<>();
    private SymbolFacade symbolManager;
    private boolean isGenericMethodContext = false;
    private HashSet<String> genericMethodContextIdentifiers = new HashSet<>();

    public Stage1Environment(Stage0Environment env) {
        this.env = env;
    }

    public SymbolFacade getFacade() {
        return symbolManager;
    }
    /**
     * add typevariable to generic context
     * @param genericParamIdentifier name of the typevariable 
     */
    public void addGernericParameter(String genericParamIdentifier) {
        if (this.isGenericMethodContext) {
            if (this.typeHash.contains(genericParamIdentifier)) {
                return;
            } else {
                this.genericMethodContextIdentifiers.add(genericParamIdentifier);
            }

        }
        this.typeHash.add(genericParamIdentifier);
    }

    public void addWorkingNode(WorkingNode workingNode) {
        if (!this.workingNodeList.contains(workingNode)) {
            this.workingNodeList.add(workingNode);
        }
    }

    public void setFacade(SymbolFacade symbolFacade) {
        this.symbolManager = symbolFacade;
    }
    /**
     * check is ident in generic context
     * @param ident name of the typevariable
     * @return true is ident a typevariable
     */
    public boolean isGenericParameter(String ident) {
        return this.typeHash.contains(ident);
    }

    public void debug() {
        for (WorkingNode node : this.workingNodeList) {
            System.err.println("InputList:=" + node.getInputs().toString());
        }
    }
    /**
     * 
     * @return the package name
     */
    public String getPackageName() {
        String className = this.env.getClassFile().getThis().getClassName();
        return QIDTools.packageName(className);

    }
    /**
     * marked that current type is generic
     */
    public void createGenericMethodContext() {
        this.isGenericMethodContext = true;
    }
    /**
     * remove generic context
     */
    public void dropGenericMethodContext() {
        if (this.isGenericMethodContext) {
            this.isGenericMethodContext = false;
            this.typeHash.removeAll(this.genericMethodContextIdentifiers);
        }
    }
    /**
     * remove all typevariables from generic context 
     */
    public void dropGenerics() {
        this.typeHash.clear();
    }

    public HashSet<WorkingNode> getWorkingList() {
        return this.workingNodeList;
    }

    public Pair<String, SyntaxNode> getAst() {
        return this.env.getAst();
    }

    ClassFile getClassFile() {
        return this.env.getClassFile();
    }

    @Override
    public int compareTo(Stage1Environment o) {
        return this.env.compareTo(o.env);
    }

    public Stage0Environment getEnv() {
        return this.env;
    }

    @Override
    public String toString() {
        return "Stage1Environment{" + "class file = " + env.getClassFile().getThis().getClassName() + '}';
    }
    
    
}
