/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package codegeneration.javavisitor.compilationunit;

import codegeneration.abstractvisitors.AbstractQualifiedIdentifierVisitor;
import codegeneration.abstractvisitors.AbstractImportDeclarationVisitor;
import abstractsyntaxtree.QualifiedIdentifierNode;
import codegeneration.Visitor;
import codegeneration.environment.Stage0Environment;
import codegeneration.javavisitor.QualifiedIdentifierVisitor;
import java.io.IOException;
import java.util.Iterator;
import exceptions.CompilerException;
import java.util.Map;
import teajay.runtime.exceptions.ResolveException;
import symbol.SymbolFacade;
import teajay.bytecode.exceptions.ByteCodeException;
import teajay.util.io.Input;
import teajay.util.io.TJSourceFinder;

/**
 *
 * @author cefour
 */
public class ImportDeclarationVisitor extends AbstractImportDeclarationVisitor {

    private AbstractQualifiedIdentifierVisitor qVisitor;
    private final boolean isStatic;
    private final boolean hasWildcard;
    private final Stage0Environment stage0Env;

    public ImportDeclarationVisitor(boolean hasWildcard, boolean aStatic) throws CompilerException {
        this.hasWildcard = hasWildcard;
        this.isStatic = aStatic;
        this.stage0Env = null;
    }

    public ImportDeclarationVisitor(boolean hasWildcard, boolean aStatic, Stage0Environment stage0Env) throws CompilerException {
        this.hasWildcard = hasWildcard;
        this.isStatic = aStatic;
        this.stage0Env = stage0Env;
    }

    @Override
    public void begin(QualifiedIdentifierNode node) throws CompilerException {
        qVisitor = new QualifiedIdentifierVisitor();
    }

    @Override
    public Visitor visit(QualifiedIdentifierNode node) throws CompilerException {
        return this.qVisitor;
    }

    @Override
    public void end(QualifiedIdentifierNode node) throws CompilerException {
        StringBuilder builder = new StringBuilder();
        Iterator<String> iter = this.qVisitor.getQualifiedNameList().iterator();
        while (iter.hasNext()) {
            builder.append(iter.next());
            if (iter.hasNext()) {
                builder.append("/");
            }
        }

        String importName = builder.toString();
        SymbolFacade facade = stage0Env.getSymbolFacade();

        CompilerException ex = null;
        try {


            if (this.hasWildcard && this.isStatic) { // Resolve all Static attributes from import
                facade.addNameFromStaticImportWithWildcard(importName);
            } else if (this.hasWildcard && !this.isStatic) { //Resolve all non static
                facade.addNameFromImportWithWildcard(importName);
            } else if (!this.hasWildcard && this.isStatic) { //Resolve only static
                facade.addNameFromStaticImport(importName);
            } else {
                facade.addNameFromImport(importName);
            }

        } catch (ByteCodeException | IOException | ResolveException e) {
            throw new RuntimeException(e);
        } catch (CompilerException e) {
            TJSourceFinder finder = TJSourceFinder.instance();
            ex = e;
            if (this.hasWildcard && this.isStatic) { // Resolve all Static attributes from import
                Map<String, Input> findAll = finder.findAll(importName);
                if (findAll == null) {
                    throw new CompilerException(ex.getMessage(), node.getSymbol());
                }
                stage0Env.addStaticInputWildcardToWokingQueue(importName, findAll);
            } else if (this.hasWildcard && !this.isStatic) { //Resolve all non static
                Map<String, Input> findAll = finder.findAll(importName);
                if (findAll == null) {
                    throw new CompilerException(ex.getMessage(), node.getSymbol());
                }
                stage0Env.addInputWildcardToWorkinQueue(importName, findAll);
            } else if (!this.hasWildcard && this.isStatic) { //Resolve only static
                Input find = finder.find(importName);
                if (find == null) {
                    throw new CompilerException(ex.getMessage(), node.getSymbol());
                }
                stage0Env.addStaticInputToWorkingQueue(importName, find);
            } else {
                Input find = finder.find(importName);
                if (find == null) {
                    throw new CompilerException(ex.getMessage(), node.getSymbol());
                }
                stage0Env.addInputToWorkingQueue(importName, find);
            }
        }
    }
}
