/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package codegeneration.javavisitor.method;

import abstractsyntaxtree.BlockNode;
import abstractsyntaxtree.FormalParametersNode;
import abstractsyntaxtree.ModifierNode;
import static abstractsyntaxtree.ModifierNode.Modifier.ABSTRACT;
import static abstractsyntaxtree.ModifierNode.Modifier.FINAL;
import static abstractsyntaxtree.ModifierNode.Modifier.NATIVE;
import static abstractsyntaxtree.ModifierNode.Modifier.PRIVATE;
import static abstractsyntaxtree.ModifierNode.Modifier.PROTECTED;
import static abstractsyntaxtree.ModifierNode.Modifier.PUBLIC;
import static abstractsyntaxtree.ModifierNode.Modifier.STATIC;
import static abstractsyntaxtree.ModifierNode.Modifier.STRICTFP;
import static abstractsyntaxtree.ModifierNode.Modifier.SYNCHRONIZED;
import abstractsyntaxtree.QualifiedIdentifierListNode;
import codegeneration.Visitor;
import codegeneration.abstractvisitors.AbstractConstructorDeclaratorRestVisitor;
import codegeneration.abstractvisitors.AbstractFormalParametersVisitor;
import codegeneration.abstractvisitors.AbstractQualifiedIdentifierListVisitor;
import codegeneration.javavisitor.BlockVisitor;
import codegeneration.javavisitor.QualifiedIdentifierListVisitor;
import datastructs.Pair;
import exceptions.CompilerException;
import java.util.LinkedList;
import java.util.List;
import resolve.SymbolTable;
import symbol.SymbolFacade;
import teajay.bytecode.ClassFile;
import teajay.bytecode.attributes.Exceptions;
import teajay.bytecode.attributes.code.Return;
import teajay.bytecode.cpool.ClassInfo;
import teajay.bytecode.cpool.FieldDescriptor;
import teajay.bytecode.cpool.MethodDescriptor;
import teajay.bytecode.exceptions.ByteCodeException;
import teajay.bytecode.methods.MethodAccessFlag;
import teajay.bytecode.methods.MethodInfo;

/**
 *
 * @author cefour
 */
public class ConstructorDeclaratorRestVisitor extends AbstractConstructorDeclaratorRestVisitor {

    private MethodInfo info;
    private final List<ModifierNode> modis;
    private final ClassFile classFile;
    private AbstractFormalParametersVisitor<List<Pair<FieldDescriptor, String>>> formalParametersVisitor;
    private AbstractQualifiedIdentifierListVisitor qualifiedIdentifierListVisitor;

    ConstructorDeclaratorRestVisitor(ClassFile classFile, List<ModifierNode> modis) {
        this.classFile = classFile;
        this.modis = modis;
    }

    @Override
    public void begin(FormalParametersNode node) {
    }

    @Override
    public void begin(QualifiedIdentifierListNode node) {
    }

    @Override
    public void begin(BlockNode node) {
        System.out.println("Begin <init>");
    }

    @Override
    public Visitor visit(FormalParametersNode node) {
        return this.formalParametersVisitor = new FormalParametersVisitor();
    }

    @Override
    public Visitor visit(QualifiedIdentifierListNode node) {
        return this.qualifiedIdentifierListVisitor = new QualifiedIdentifierListVisitor();
    }

    @Override
    public Visitor visit(BlockNode node) {
        return new BlockVisitor(classFile, info);
    }

    @Override
    public void end(FormalParametersNode node) {
        try {
            MethodAccessFlag[] flags = this.getMethodAccessFlags();
            List<Pair<FieldDescriptor, String>> list = this.formalParametersVisitor.getFieldContainer();
            List<FieldDescriptor> fList = new LinkedList<>();
            for (Pair<FieldDescriptor, String> p : list) {
                fList.add(p.value1);
            }
            MethodDescriptor desc = new MethodDescriptor(FieldDescriptor.getVoid(), fList);


            this.info = new MethodInfo("<init>", desc, this.classFile.getThis(), flags);
            this.classFile.addMethod(this.info);

        } catch (ByteCodeException | CompilerException ex) {
            System.err.println(ex.getMessage());
            System.exit(-1);
        }
    }

    @Override
    public void end(QualifiedIdentifierListNode node) {
        List<String> qIdList = this.qualifiedIdentifierListVisitor.getQualifiedIdentifiers();
            Exceptions ex = new Exceptions();
            for (String q : qIdList) {

       //         ex.addException(new ClassInfo(SymbolFacade.instance().getFullQualifiedId(q)));

            }
            info.addAttribute(ex);
        
    }

    @Override
    public void end(BlockNode node) {
        try {
            this.info.getCode().getInstructionList().add(new Return());
        } catch (ByteCodeException ex) {
            System.err.println(ex.getMessage());
            System.exit(-1);
        }
        System.out.println("End <init>");
    }

    /**
     * Method checks the access flags of correctnes and uniquenes.
     *
     * @return Array of MethodAccessFlags
     * @throws CompilerException
     */
    private MethodAccessFlag[] getMethodAccessFlags() throws CompilerException {
        MethodAccessFlag[] flags = new MethodAccessFlag[modis.size()];
        boolean[] uniqueAccess = {false, false, false,
            false, false, false,
            false, false, false};
        int i = 0;
        for (ModifierNode mod : modis) {
            switch (mod.getModifier()) {
                case ABSTRACT:
                    if (!uniqueAccess[0]) {
                        flags[i] = MethodAccessFlag.ABSTRACT;
                        uniqueAccess[0] = true;
                    } else {
                        throw new CompilerException("Dublicate modifier at " + classFile.getThis().getSimpleName(), mod.getSymbol());
                    }
                    break;
                case FINAL:
                    if (!uniqueAccess[1]) {
                        flags[i] = MethodAccessFlag.FINAL;
                        uniqueAccess[1] = true;
                    } else {
                        throw new CompilerException("Dublicate modifier at " + classFile.getThis().getSimpleName(), mod.getSymbol());
                    }
                    break;
                case STATIC:
                    if (!uniqueAccess[2]) {
                        flags[i] = MethodAccessFlag.STATIC;
                        uniqueAccess[2] = true;
                    } else {
                        throw new CompilerException("Dublicate modifier at " + classFile.getThis().getSimpleName(), mod.getSymbol());
                    }
                    break;
                case SYNCHRONIZED:
                    if (!uniqueAccess[3]) {
                        flags[i] = MethodAccessFlag.SYNCHRONIZED;
                        uniqueAccess[3] = true;
                    } else {
                        throw new CompilerException("Dublicate modifier at " + classFile.getThis().getSimpleName(), mod.getSymbol());
                    }
                    break;
                case PUBLIC:
                    if (!uniqueAccess[4]) {
                        flags[i] = MethodAccessFlag.PUBLIC;
                        uniqueAccess[4] = true;
                    } else {
                        throw new CompilerException("Dublicate modifier at " + classFile.getThis().getSimpleName(), mod.getSymbol());
                    }
                    break;
                case PROTECTED:
                    if (!uniqueAccess[5]) {
                        flags[i] = MethodAccessFlag.PROTECTED;
                        uniqueAccess[5] = true;
                    } else {
                        throw new CompilerException("Dublicate modifier at " + classFile.getThis().getSimpleName(), mod.getSymbol());
                    }
                    break;
                case PRIVATE:
                    if (!uniqueAccess[6]) {
                        flags[i] = MethodAccessFlag.PRIVATE;
                        uniqueAccess[6] = true;
                    } else {
                        throw new CompilerException("Dublicate modifier at " + classFile.getThis().getSimpleName(), mod.getSymbol());
                    }
                    break;
                case NATIVE:
                    if (!uniqueAccess[7]) {
                        flags[i] = MethodAccessFlag.NATIVE;
                        uniqueAccess[7] = true;
                    } else {
                        throw new CompilerException("Dublicate modifier at " + classFile.getThis().getSimpleName(), mod.getSymbol());
                    }
                    break;
                case STRICTFP:
                    if (!uniqueAccess[8]) {
                        flags[i] = MethodAccessFlag.STRICT;
                        uniqueAccess[8] = true;
                    } else {
                        throw new CompilerException("Dublicate modifier at " + classFile.getThis().getSimpleName(), mod.getSymbol());
                    }
                    break;
                default:
            }
            i++;
        }
        return flags;
    }
}
