/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package codegeneration.javavisitor.method;

import abstractsyntaxtree.FormalParameterDeclsRestNode;
import abstractsyntaxtree.TypeNode;
import abstractsyntaxtree.VariableModifierNode;
import codegeneration.Visitor;
import codegeneration.abstractvisitors.AbstractFormalParameterDeclsRestVisitor;
import codegeneration.abstractvisitors.AbstractFormalParameterDeclsVisitor;
import codegeneration.abstractvisitors.AbstractTypeVisitor;
import codegeneration.abstractvisitors.AbstractVariableModifierVisitor;
import codegeneration.javavisitor.TypeVisitor;
import codegeneration.javavisitor.VariableModifierVisitor;
import datastructs.Pair;
import exceptions.CompilerException;
import java.util.List;
import teajay.bytecode.BaseType;
import teajay.bytecode.cpool.FieldDescriptor;
import teajay.bytecode.exceptions.ByteCodeException;
import teajay.bytecode.exceptions.MalformedClassNameException;

/**
 *
 * @author cefour
 */
public class FormalParameterDeclsVisitor extends AbstractFormalParameterDeclsVisitor {

    private int arrayDim;
    private AbstractTypeVisitor typeVisitor;
    private FormalParameterDeclsRestVisitor formalParameterDeclsRestVisitor;
    private boolean isReferenceType;
    private AbstractVariableModifierVisitor variableModifier;
    private boolean isParamFinal = false;
    private final List<Pair<FieldDescriptor, String>> paramList;
    private Pair<FieldDescriptor, String> pairItem;

    FormalParameterDeclsVisitor(List<Pair<FieldDescriptor, String>> paramList) {
        this.paramList = paramList;
    }

    @Override
    public void begin(VariableModifierNode node) throws CompilerException {
        this.variableModifier = new VariableModifierVisitor();

    }

    @Override
    public void begin(TypeNode node) throws CompilerException {
        this.arrayDim = node.getArrayDim();

    }

    @Override
    public void begin(FormalParameterDeclsRestNode node) throws CompilerException {

        String typeName = this.typeVisitor.getTypeName();
        this.pairItem = new Pair<>();
        if (this.arrayDim > 0) {
            if (isReferenceType) {
                pairItem.value1 = FieldDescriptor.
                        makeArray(FieldDescriptor.fromString(typeName), this.arrayDim);
                this.paramList.add(pairItem);
            } else {
                pairItem.value1 = FieldDescriptor.
                        makeArray(FieldDescriptor.fromBaseType(BaseType.valueOf(typeName)),
                        this.arrayDim);
                this.paramList.add(pairItem);
            }
        } else {
            if (isReferenceType) {
                pairItem.value1 = FieldDescriptor.fromString(typeName);
                this.paramList.add(pairItem);
            } else {
                pairItem.value1 = FieldDescriptor.
                        fromBaseType(BaseType.valueOf(typeName));
                this.paramList.add(pairItem);
            }
        }
    }

    @Override
    public Visitor visit(VariableModifierNode node) throws CompilerException {
        return this.variableModifier;
    }

    @Override
    public Visitor visit(TypeNode node) throws CompilerException {
        return this.typeVisitor = new TypeVisitor();
    }

    @Override
    public Visitor visit(FormalParameterDeclsRestNode node) throws CompilerException {
        return this.formalParameterDeclsRestVisitor = new FormalParameterDeclsRestVisitor(this.typeVisitor, this.paramList);
    }

    @Override
    public void end(VariableModifierNode node) throws CompilerException {
        this.isParamFinal = node.isFinal();
    }

    @Override
    public void end(TypeNode node) throws CompilerException {
        this.isReferenceType = node.isReferenceType();
    }

    @Override
    public void end(FormalParameterDeclsRestNode node) throws CompilerException {
        pairItem.value2 = this.formalParameterDeclsRestVisitor.getVarName();

    }
}
