/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package codegeneration.javavisitor.statement;

import abstractsyntaxtree.AssertStatementNode;
import abstractsyntaxtree.BlockNode;
import abstractsyntaxtree.BreakStatementNode;
import abstractsyntaxtree.ContinueStatementNode;
import abstractsyntaxtree.DoStatementNode;
import abstractsyntaxtree.ForStatementNode;
import abstractsyntaxtree.IdentDoublePointStatementNode;
import abstractsyntaxtree.IfStatementNode;
import abstractsyntaxtree.ReturnStatementNode;
import abstractsyntaxtree.StatementExpressionNode;
import abstractsyntaxtree.SwitchStatementNode;
import abstractsyntaxtree.SynchronizedStatementNode;
import abstractsyntaxtree.ThrowStatementNode;
import abstractsyntaxtree.TryStatementNode;
import abstractsyntaxtree.TypeSwitchStatementNode;
import abstractsyntaxtree.WhileStatementNode;
import codegeneration.Visitor;
import codegeneration.abstractvisitors.AbstractStatementVisitor;
import codegeneration.javavisitor.BlockVisitor;
import teajay.bytecode.ClassFile;
import teajay.bytecode.attributes.code.Label;
import teajay.bytecode.methods.MethodInfo;

/**
 *
 * @author cefour
 */
public class StatementVisitor extends AbstractStatementVisitor {

    private final ClassFile classFile;
    private final MethodInfo info;
    private IfStatementVisitor ifStatementVisitor;
    private StatementExpressionVisitor statementExpressionVisitor;
    private Label label;
    private WhileStatementVisitor whileStatementVisitor;

    public StatementVisitor(ClassFile classFile, MethodInfo info) {
        this.classFile = classFile;
        this.info = info;
    }

    public StatementVisitor(ClassFile classFile, MethodInfo info, Label label) {
        this.classFile = classFile;
        this.info = info;
        this.label = label;
    }

    @Override
    public void begin(BlockNode node) {
    }

    @Override
    public void begin(IdentDoublePointStatementNode node) {
    }

    @Override
    public void begin(StatementExpressionNode node) {
    }

    @Override
    public void begin(IfStatementNode node) {
    }

    @Override
    public void begin(AssertStatementNode node) {
    }

    @Override
    public void begin(SwitchStatementNode node) {
    }

    @Override
    public void begin(WhileStatementNode node) {
    }

    @Override
    public void begin(DoStatementNode node) {
    }

    @Override
    public void begin(ForStatementNode node) {
    }

    @Override
    public void begin(BreakStatementNode node) {
    }

    @Override
    public void begin(ContinueStatementNode node) {
    }

    @Override
    public void begin(ReturnStatementNode node) {
    }

    @Override
    public void begin(ThrowStatementNode node) {
    }

    @Override
    public void begin(SynchronizedStatementNode node) {
    }

    @Override
    public void begin(TryStatementNode node) {
    }

    @Override
    public void begin(TypeSwitchStatementNode node) {
    }

    @Override
    public Visitor visit(BlockNode node) {
        return new BlockVisitor(classFile, info);
    }

    @Override
    public Visitor visit(IdentDoublePointStatementNode node) {
        return null;
    }

    @Override
    public Visitor visit(StatementExpressionNode node) {
        return statementExpressionVisitor = new StatementExpressionVisitor(this.classFile, this.info, this.label);
    }

    @Override
    public Visitor visit(IfStatementNode node) {
        return ifStatementVisitor = new IfStatementVisitor(this.classFile, this.info);
    }

    @Override
    public Visitor visit(AssertStatementNode node) {
        return null;
    }

    @Override
    public Visitor visit(SwitchStatementNode node) {
        return null;
    }

    @Override
    public Visitor visit(WhileStatementNode node) {
        return whileStatementVisitor = new WhileStatementVisitor(this.classFile, this.info);
    }

    @Override
    public Visitor visit(DoStatementNode node) {
        return null;
    }

    @Override
    public Visitor visit(ForStatementNode node) {
        return null;
    }

    @Override
    public void visit(BreakStatementNode node) {
    }

    @Override
    public void visit(ContinueStatementNode node) {
    }

    @Override
    public Visitor visit(ReturnStatementNode node) {
        return null;
    }

    @Override
    public Visitor visit(ThrowStatementNode node) {
        return null;
    }

    @Override
    public Visitor visit(SynchronizedStatementNode node) {
        return null;
    }

    @Override
    public Visitor visit(TryStatementNode node) {
        return null;
    }

    @Override
    public Visitor visit(TypeSwitchStatementNode node) {
        return null;
    }

    @Override
    public void end(BlockNode node) {
    }

    @Override
    public void end(IdentDoublePointStatementNode node) {
    }

    @Override
    public void end(StatementExpressionNode node) {
        this.label = null;
    }

    @Override
    public void end(IfStatementNode node) {
        this.label = ifStatementVisitor.getLabel();
    }

    @Override
    public void end(AssertStatementNode node) {
    }

    @Override
    public void end(SwitchStatementNode node) {
    }

    @Override
    public void end(WhileStatementNode node) {
        this.label = whileStatementVisitor.getJmpOverWhile();
    }

    @Override
    public void end(DoStatementNode node) {
    }

    @Override
    public void end(ForStatementNode node) {
    }

    @Override
    public void end(BreakStatementNode node) {
    }

    @Override
    public void end(ContinueStatementNode node) {
    }

    @Override
    public void end(ReturnStatementNode node) {
    }

    @Override
    public void end(ThrowStatementNode node) {
    }

    @Override
    public void end(SynchronizedStatementNode node) {
    }

    @Override
    public void end(TryStatementNode node) {
    }

    @Override
    public void end(TypeSwitchStatementNode node) {
    }

    public Label getLabel() {
        return label;
    }
}
