/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package stage1;

import abstractsyntaxtree.DotIdentifierTypeArgumentsHelperNode;
import abstractsyntaxtree.TypeArgumentsNode;
import codegeneration.Visitor;
import codegeneration.abstractvisitors.AbstractReferenceTypeVisitor;
import codegeneration.environment.Stage1Environment;
import exceptions.CompilerException;
import java.util.ArrayList;
import java.util.List;

/**
 *
 * @author Michael Burzan
 */
class Stage1GenericReferenceTypeVisitor extends AbstractReferenceTypeVisitor {

    private final Stage1Environment stage1Env;
    private final List<String> identifierList;
    private boolean hasTypeArgument = false;
    private final List<String> typeArgumentIdentifierList = new ArrayList<>();

    public Stage1GenericReferenceTypeVisitor(Stage1Environment stage1Env, List<String> identifierList) {
        this.stage1Env = stage1Env;
        this.identifierList = identifierList;
    }

    @Override
    public void begin(TypeArgumentsNode node) throws CompilerException {
        this.hasTypeArgument = true;
    }

    @Override
    public Visitor visit(TypeArgumentsNode node) throws CompilerException {

        return new Stage1GenericTypeArgumentsVisitor(stage1Env, typeArgumentIdentifierList);
    }

    @Override
    public void end(TypeArgumentsNode node) throws CompilerException {
    }

    @Override
    public void begin(DotIdentifierTypeArgumentsHelperNode node) throws CompilerException {
        if (hasTypeArgument) {
            throw new CompilerException("Inner generic parameter is not allowed", node.getSymbol());
        }
        identifierList.add(node.getIdentifier());

    }

    @Override
    public Visitor visit(DotIdentifierTypeArgumentsHelperNode node) throws CompilerException {

        return new Stage1DotIdentifierTypeArgumentsHelperVisitor(stage1Env, this.identifierList);
    }

    @Override
    public void end(DotIdentifierTypeArgumentsHelperNode node) throws CompilerException {
    }
}
