/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package stage2;

import abstractsyntaxtree.ConstructorDeclaratorRestNode;
import abstractsyntaxtree.MethodeDeclaratorRestNode;
import abstractsyntaxtree.ModifierNode;
import abstractsyntaxtree.TypeNode;
import codegeneration.Visitor;
import codegeneration.abstractvisitors.AbstractGenericMethodeOrConstructorRestVisitor;
import codegeneration.environment.Stage2Environment;
import exceptions.CompilerException;
import java.util.List;
import stage2.utils.MethodHeadBuilder;
import teajay.bytecode.methods.MethodAccessFlag;
import teajay.generics.parser.Type;

/**
 *
 * @author Michael Burzan
 */
class Stage2GenericMethodOrConstructorRestVisitor extends AbstractGenericMethodeOrConstructorRestVisitor {
    private final Stage2Environment stage2Env;
    private final List<Type.TypeParamType> typeParams;
    private final List<ModifierNode> modis;
    private Stage2TypeVisitor typeVisitor;
    private Stage2MethodHeadVisitor methodDeclVisitor;
    private Stage2MethodHeadVisitor constructorDeclVisitor;
    private final String methodName;
    private Type returnType;


    Stage2GenericMethodOrConstructorRestVisitor(Stage2Environment stage2Env, List<Type.TypeParamType> typeParams, List<ModifierNode> modis,String methodName) {
        this.stage2Env = stage2Env;
        this.typeParams = typeParams;
        this.modis = modis;
        this.methodName = methodName;

    }

    

    @Override
    public void begin(TypeNode node) throws CompilerException {

    }

    @Override
    public void begin(MethodeDeclaratorRestNode node) throws CompilerException {

    }

    @Override
    public void begin(ConstructorDeclaratorRestNode node) throws CompilerException {

    }

    @Override
    public Visitor visit(TypeNode node) throws CompilerException {
        return typeVisitor = new Stage2TypeVisitor(stage2Env);
    }

    @Override
    public Visitor visit(MethodeDeclaratorRestNode node) throws CompilerException {
        return methodDeclVisitor = new Stage2MethodHeadVisitor(stage2Env);
    }

    @Override
    public Visitor visit(ConstructorDeclaratorRestNode node) throws CompilerException {
        this.returnType = new Type.SimpleType("void");
        return constructorDeclVisitor = new Stage2MethodHeadVisitor(stage2Env);
    }

    @Override
    public void end(TypeNode node) throws CompilerException {
        this.returnType = typeVisitor.getType();
    }

    @Override
    public void end(MethodeDeclaratorRestNode node) throws CompilerException {
      // generate the method head with generics
      MethodHeadBuilder.buildGenericMethodHeadClass(node, stage2Env, returnType, methodName, methodDeclVisitor.getMethodHead() , MethodHeadBuilder.getMethodAccessFlags(methodName, modis), typeParams);

    }

    @Override
    public void end(ConstructorDeclaratorRestNode node) throws CompilerException {
        // generate the constructor head
        MethodHeadBuilder.buildGenericMethodHeadClass(node, stage2Env, returnType, methodName, constructorDeclVisitor.getMethodHead() , MethodHeadBuilder.getMethodAccessFlags(methodName, modis), typeParams);
    }
    
}
