/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package stage2;

import abstractsyntaxtree.ConstructorDeclaratorRestNode;
import abstractsyntaxtree.MethodeDeclaratorRestNode;
import abstractsyntaxtree.ModifierNode;
import abstractsyntaxtree.SyntaxNode;
import abstractsyntaxtree.TypeNode;
import codegeneration.Visitor;
import codegeneration.abstractvisitors.AbstractGenericMethodeOrConstructorRestVisitor;
import codegeneration.environment.Stage2Environment;
import datastructs.Pair;
import exceptions.CompilerException;
import java.util.Collection;
import java.util.List;
import stage2.utils.MethodHeadBuilder;
import teajay.bytecode.methods.MethodInfo;
import teajay.generics.parser.Type;

/**
 *
 * @author Michael Burzan
 */
class Stage2TjImplGenericMethodOrConstructorRestVisitor extends AbstractGenericMethodeOrConstructorRestVisitor {

    private final Stage2Environment stage2Env;
    private final List<ModifierNode.Modifier> flagList;
    private final Collection<List<MethodInfo>> methods;
    private final List<Type.TypeParamType> typeParams;
    private final String identifier;
    private final boolean aVoid;
    private Stage2MethodHeadVisitor stage2MethodDecl;
    private Stage2TypeVisitor stage2Type;
    private Stage2MethodHeadVisitor constructorDecl;
    private Type type;
    private String methodName;
    private Pair<List<Pair<Type, String>>, List<String>> methodHead;

    public Stage2TjImplGenericMethodOrConstructorRestVisitor(Stage2Environment stage2Env, List<ModifierNode.Modifier> flagList,
            Collection<List<MethodInfo>> methods, List<Type.TypeParamType> typeParams, String identifier, boolean aVoid) {
        this.stage2Env = stage2Env;
        this.flagList = flagList;
        this.methods = methods;
        this.typeParams = typeParams;
        this.identifier = identifier;
        this.aVoid = aVoid;
        if (aVoid) {
            this.type = new Type.SimpleType("void");
        }
        this.methodName = identifier;
    }

    @Override
    public void begin(TypeNode node) throws CompilerException {
    }

    @Override
    public void begin(MethodeDeclaratorRestNode node) throws CompilerException {
    }

    @Override
    public void begin(ConstructorDeclaratorRestNode node) throws CompilerException {
        this.type = new Type.SimpleType("void");
        if(!methodName.equals(stage2Env.getClassFile().getThis().getSimpleName())){
            throw new CompilerException("Invalid constructor name "+ methodName,node.getSymbol());
        }
        methodName = "<init>";
    }

    @Override
    public Visitor visit(TypeNode node) throws CompilerException {
        return stage2Type = new Stage2TypeVisitor(stage2Env);
    }

    @Override
    public Visitor visit(MethodeDeclaratorRestNode node) throws CompilerException {
        return stage2MethodDecl = new Stage2MethodHeadVisitor(stage2Env);
    }

    @Override
    public Visitor visit(ConstructorDeclaratorRestNode node) throws CompilerException {

        return constructorDecl = new Stage2MethodHeadVisitor(stage2Env);
    }

    @Override
    public void end(TypeNode node) throws CompilerException {
        if (!aVoid) {
            this.type = stage2Type.getType();
        }
        node.setType(type);
    }

    @Override
    public void end(ConstructorDeclaratorRestNode node) throws CompilerException {
        this.methodHead = this.constructorDecl.getMethodHead();
        createMethod(node);
    }

    @Override
    public void end(MethodeDeclaratorRestNode node) throws CompilerException {
        this.methodHead = this.stage2MethodDecl.getMethodHead();
        createMethod(node);
    }

    private void createMethod(SyntaxNode node)throws CompilerException{
        MethodHeadBuilder.buildGenericMethodHead(node, stage2Env, type, methodName, this.methodHead, flagList,this.typeParams);
    }
    
    
    
}
