/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package stage2;

import abstractsyntaxtree.ModifierNode;
import abstractsyntaxtree.TjTypeDefBodyNode;
import abstractsyntaxtree.TypeListNode;
import abstractsyntaxtree.TypeNode;
import abstractsyntaxtree.TypeParametersNode;
import codegeneration.Visitor;
import codegeneration.abstractvisitors.AbstractTjTypeDefVisitor;
import teajay.util.tjattribute.TeaJayAttribute;
import codegeneration.environment.Stage2Environment;
import codegeneration.environment.Stage3Environment;
import exceptions.CompilerException;
import java.io.IOException;
import java.util.Collections;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;
import teajay.bytecode.ClassFile;
import teajay.bytecode.attributes.AttributeInfo;
import teajay.bytecode.attributes.code.ALoad;
import teajay.bytecode.attributes.code.InvokeSpecial;
import teajay.bytecode.attributes.code.Return;
import teajay.bytecode.attributes.signature.Signature;
import teajay.bytecode.cpool.FieldDescriptor;
import teajay.bytecode.cpool.MethodDescriptor;
import teajay.bytecode.cpool.MethodRefInfo;
import teajay.bytecode.exceptions.ByteCodeException;
import teajay.bytecode.methods.MethodAccessFlag;
import teajay.bytecode.methods.MethodInfo;
import teajay.generics.check.TypeUtils;
import teajay.generics.parser.Type;
import teajay.generics.writer.SignatureEncoder;
import teajay.runtime.exceptions.ResolveException;
import teajay.util.io.ClassFinder;

/**
 *
 * @author Michael (Burzan | Kußmann)
 */
class Stage2TjTypeDefVisitor extends AbstractTjTypeDefVisitor {

    private final Stage2Environment stage2Env;
    private Stage2TypeParametersVisitor stage2TypeParameters;
    private Stage2TypeVisitor stage2Type;
    private Stage2TypeListVisitor stage2TypeList;

    public Stage2TjTypeDefVisitor(Stage2Environment stage2Env) {
        this.stage2Env = stage2Env;
        this.stage2Env.setTypeUtils(new TypeUtils());
    }

    @Override
    public void begin(TypeParametersNode node) throws CompilerException {
        stage2Env.getTypeUtils().addFrame();
    }

    @Override
    public void begin(TypeNode node) throws CompilerException {
    }

    @Override
    public void begin(TypeListNode node) throws CompilerException {
    }

    @Override
    public void begin(TjTypeDefBodyNode node) throws CompilerException {
        Type extendsType;
        List<Type.TypeParamType> generics;
        List<Type> implementsList;
        //typedef foo<T extends Test>
        if (stage2TypeParameters == null) {
            generics = null;
        } else {
            generics = stage2TypeParameters.getTypeParams();
        }
        
        if (stage2Type == null) {
            extendsType = new Type.ClassType(null, "teajay/lang/TJObject", null);
        } else {
            extendsType = stage2Type.getType();
            checkIsExtendsTypeDef(extendsType, node);
        }
        
        //the interfaces list
        if (stage2TypeList == null) {
            implementsList = null;
        } else {
            implementsList = stage2TypeList.getImplementsList();
        }
        // create signature
        Type.ClassSigType sigType = new Type.ClassSigType(generics, extendsType, implementsList);
        ClassFile classFile = stage2Env.getClassFile();
        try {
            //add signature to class file
            classFile.addAttribute(new Signature(SignatureEncoder.encode(sigType)));
            // add super class
            classFile.setSuper(((Type.ClassType) extendsType).name);
            // add interfaces
            if (implementsList != null) {
                for (Type classType : implementsList) {
                    Type.ClassType c = (Type.ClassType) classType;
                    classFile.addInterface(ClassFinder.instance().find(c.name).get().getThis());
                }
            }
            //save the class signature
            stage2Env.setClassSigType(sigType);
        } catch (ByteCodeException | IOException | ResolveException ex) {
            ex.printStackTrace(System.err);
            System.exit(-1);
        }


    }

    @Override
    public Visitor visit(TypeParametersNode node) throws CompilerException { //Generics
        stage2Env.addGenericFrame();
        return stage2TypeParameters = new Stage2TypeParametersVisitor(stage2Env);
    }

    @Override
    public Visitor visit(TypeNode node) throws CompilerException { //extends
        return stage2Type = new Stage2TypeVisitor(stage2Env);
    }

    @Override
    public Visitor visit(TypeListNode node) throws CompilerException { //Implements
        return stage2TypeList = new Stage2TypeListVisitor(stage2Env);
    }

    @Override
    public Visitor visit(TjTypeDefBodyNode node) throws CompilerException { //Body
        return new Stage2TjTypeDefBodyVisitor(stage2Env);
    }

    @Override
    public void end(TypeParametersNode node) throws CompilerException {
        node.setGeneric(this.stage2TypeParameters.getTypeParams());
    }

    @Override
    public void end(TypeNode node) throws CompilerException {
        node.setType(this.stage2Type.getType());
    }

    @Override
    public void end(TypeListNode node) throws CompilerException {
        node.setTypeList(this.stage2TypeList.getImplementsList());
    }
    
    @Override
    public void end(TjTypeDefBodyNode node) throws CompilerException {
        try {
            List<MethodInfo> methods = this.stage2Env.getClassFile().getMethods("<init>");
            if (methods == null) {
                createProtectedConstructor(this.stage2Env.getClassFile());
                return;
            }
            MethodInfo cons;
            cons = new MethodInfo("<init>", new MethodDescriptor(FieldDescriptor.getVoid()),stage2Env.getClassFile().getThis(), MethodAccessFlag.PUBLIC);
            for(MethodInfo info : methods){
                if(info.equals(cons)){
                    return;
                }
            }
            createProtectedConstructor(this.stage2Env.getClassFile());
                                    
        } catch (ByteCodeException ex) {
            ex.printStackTrace(System.err);
            System.exit(-1);
        }

    }
    /**
     * Checks that typedef extends a type 
     * @param extendsType
     * @param node
     * @throws CompilerException 
     */
    private void checkIsExtendsTypeDef(Type extendsType, TjTypeDefBodyNode node) throws CompilerException {
        if (extendsType instanceof Type.ClassType) {
            Type.ClassType classType = (Type.ClassType) extendsType;
            String className = stage2Env.getSymbolFacade().getFullQualifiedId( classType.name );
            try {
                AttributeInfo attribute = ClassFinder.instance().find(className).get().getAttribute(TeaJayAttribute.NAME);
                if (attribute == null) {
                    throw new CompilerException("Can only extend typedefs", node.getSymbol());
                }
                if (((TeaJayAttribute) attribute).getType() != TeaJayAttribute.Type.Typedef
                        && ((TeaJayAttribute) attribute).getType() != TeaJayAttribute.Type.TJBuiltinType) {
                    throw new CompilerException("Can only extend typedefs", node.getSymbol());
                }else if(((TeaJayAttribute) attribute).getAccessType() == TeaJayAttribute.AccessType.Final){
                     throw new CompilerException("Can not extend final typedef", node.getSymbol());
                }
            } catch (ByteCodeException | IOException | ResolveException ex) {
                ex.printStackTrace(System.err);
                System.exit(-1);
            }
        } else {
            throw new CompilerException("Can not extend from type variable " + extendsType, node.getSymbol());
        }

    }
    /**
     * Build a protected constructor if not exist in typedef
     * Because the constructor of the typeimpl must call a super constructor
     * @param classFile
     * @throws CompilerException 
     */
    private void createProtectedConstructor(ClassFile classFile) throws CompilerException {
        Type.MethodType methodType = new Type.MethodType(Collections.EMPTY_LIST, new Type.SimpleType("void"));
        MethodDescriptor makeMethodDescriptor = stage2Env.getTypeUtils().makeMethodDescriptor(methodType);
        try {
            MethodInfo minfo = new MethodInfo("<init>", makeMethodDescriptor,
                    classFile.getThis(), MethodAccessFlag.PROTECTED);
            MethodRefInfo refInfo = MethodRefInfo.make(classFile.getSuper(), "<init>", FieldDescriptor.getVoid());
            if (refInfo == null) {
                throw new Error("Fatal error:" + classFile.getSuper() + "has no constructor");
            }
            minfo.getCode().getInstructionList().add(new ALoad(0));
            minfo.getCode().getInstructionList().add(new InvokeSpecial(refInfo));
            minfo.getCode().getInstructionList().add(new Return());
            classFile.addMethod(minfo);

        } catch (ByteCodeException ex) {
            ex.printStackTrace(System.err);
            System.exit(-1);
        }
    }

    @Override
    public void begin(ModifierNode node) throws CompilerException {
    }

    @Override
    public Visitor visit(ModifierNode node) throws CompilerException {
        return null;
    }

    @Override
    public void end(ModifierNode node) throws CompilerException {
    }
}