/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package stage2;

import abstractsyntaxtree.TypeParameterNode;
import codegeneration.Visitor;
import codegeneration.abstractvisitors.AbstractTypeParametersVisitor;
import codegeneration.environment.Stage2Environment;
import exceptions.CompilerException;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;
import teajay.bytecode.ClassFile;
import teajay.bytecode.exceptions.ByteCodeException;
import teajay.generics.parser.Type;
import teajay.runtime.exceptions.ResolveException;
import teajay.util.io.ClassFileRef;
import teajay.util.io.ClassFinder;

/**
 *
 * @author Michael Burzan
 */
class Stage2TypeParametersVisitor extends AbstractTypeParametersVisitor {

    private final Stage2Environment stage2Env;
    private Stage2TypeParameterVisitor stage2TypeParameters;
    private final static Type.ClassType typeObject = new Type.ClassType(null, "java/lang/Object", null);
    private List<Type.TypeParamType> typeParamType = new ArrayList<>();

    public Stage2TypeParametersVisitor(Stage2Environment stage2Env) {
        this.stage2Env = stage2Env;
    }

    @Override
    public void begin(TypeParameterNode node) throws CompilerException {
        stage2Env.addTypeVariable(node.getIdentifier());
    }

    @Override
    public Visitor visit(TypeParameterNode node) throws CompilerException {
        return this.stage2TypeParameters = new Stage2TypeParameterVisitor(stage2Env);
    }

    @Override
    public void end(TypeParameterNode node) throws CompilerException {
        List<Type> bounds = stage2TypeParameters.getBounds();
        Type.TypeParamType type =null;
        if (bounds.isEmpty()) {
            type = new Type.TypeParamType(node.getIdentifier(), typeObject, null);
            this.typeParamType.add(type);
        } else {
            // get the first bound
            Type firstElement = bounds.get(0);
            if (firstElement instanceof Type.ClassType) {
                //get full qualified name 
                String qId = ((Type.ClassType) firstElement).name;

                try {
                    ClassFile find = ClassFinder.instance().find(qId).get();
                    //check is type a interface
                    if (find.isInterface()) {
                        type = new Type.TypeParamType(node.getIdentifier(), null, (List<Type>) (Object) bounds);
                        this.typeParamType.add(type);
                    } else {
                        type = new Type.TypeParamType(node.getIdentifier(), firstElement, (List<Type>) (Object) bounds.subList(1, bounds.size()));
                        this.typeParamType.add(type);
                    }
                } catch (ByteCodeException | IOException | ResolveException ex) {
                    ex.printStackTrace(System.err);
                    System.exit(-1);
                }
            } else if (firstElement instanceof Type.SimpleType) {
                //if type a normal type like Number
                type = new Type.TypeParamType(node.getIdentifier(), firstElement, (List<Type>) (Object) bounds.subList(1, bounds.size()));
                this.typeParamType.add(type);
            }


        }
        if(type == null){
            throw new Error("(CMP_ERROR)Type parameter is not valid: "+ node.getSymbol());
        }
        stage2Env.getTypeUtils().addTypeToContext(type);      
    }

    public List<Type.TypeParamType> getTypeParams() {
        return typeParamType;
    }
}
