/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package stage2.utils;

import codegeneration.environment.Stage2Environment;
import java.io.IOException;
import java.util.List;
import symbol.LocalTable;
import teajay.bytecode.ClassFile;
import teajay.bytecode.attributes.code.ALoad;
import teajay.bytecode.attributes.code.Dup;
import teajay.bytecode.attributes.code.Goto;
import teajay.bytecode.attributes.code.IfNull;
import teajay.bytecode.attributes.code.InvokeSpecial;
import teajay.bytecode.attributes.code.InvokeVirtual;
import teajay.bytecode.attributes.code.JVMInstructionList;
import teajay.bytecode.attributes.code.Label;
import teajay.bytecode.attributes.code.PutField;
import teajay.bytecode.attributes.code.Return;
import teajay.bytecode.attributes.signature.Signature;
import teajay.bytecode.cpool.FieldDescriptor;
import teajay.bytecode.cpool.MethodRefInfo;
import teajay.bytecode.exceptions.ByteCodeException;
import teajay.bytecode.fields.FieldAccessFlag;
import teajay.bytecode.fields.FieldInfo;
import teajay.bytecode.methods.MethodInfo;
import teajay.generics.parser.Type;
import teajay.generics.writer.SignatureEncoder;
import teajay.runtime.exceptions.ResolveException;
import teajay.util.io.ClassFinder;

/**
 *
 * @author Michael Burzan
 */
public class ConstructorBuilder {
    /**
     * Build the constructor <init>(Type t, Parameters) from a typeimpl
     * @param constructorInfo
     * @param stage2Env 
     */
    public static void build(MethodInfo constructorInfo, Stage2Environment stage2Env) {
        try {
            ClassFile typeImpl = stage2Env.getClassFile();
            ClassFile typeDefClass = ClassFinder.instance().find(typeImpl.getSuper().getClassName()).get();
            Label label = new Label("1");
            Label jmpOver = new Label("2");
            //get this field from typeimpl
            FieldInfo info = typeImpl.getField("this");
            //count of parameter from constructor
            int size = constructorInfo.getDescriptor().getParameters().size();

            //Get constructor parameters without the first parameter
            List<FieldDescriptor> subList = constructorInfo.getDescriptor().getParameters().subList(1, size);
            
            //if this-field not set in class file
            //set this field to class file
            if (info == null) {
                info = new FieldInfo("this", FieldDescriptor.fromClassFile(typeDefClass), FieldAccessFlag.PRIVATE, FieldAccessFlag.SYNTHETIC);
                typeImpl.addField(info);
                info.addAttribute(new Signature(SignatureEncoder.encode(stage2Env.getTypeUtils().createThis(typeDefClass.getThis().getClassName()))));
            }
            //this helps to write correct bytecode
            LocalTable table = new LocalTable(constructorInfo);
            table.addFrame();
            
            //add all parameters to local table
            table.addEntry("this", new Type.ClassType(null, typeImpl.getThis().getClassName(), null));
            for (int i = 0; i < constructorInfo.getDescriptor().getParameters().size(); i++) {
                table.addEntry(String.valueOf(i), new Type.ClassType(null, constructorInfo.getDescriptor().getParameters().get(i).getClassInfo().getClassName(), null));
            }
            //load this from register 0
            JVMInstructionList instructionList = constructorInfo.getCode().getInstructionList();
            instructionList.add(new ALoad(0));

            //call the typedef constructor 
            MethodRefInfo ref = MethodRefInfo.make(stage2Env.getClassFile().getSuper(), "<init>", FieldDescriptor.getVoid());
            instructionList.add(new InvokeSpecial(ref));
            //Load first parameter
            instructionList.add(new ALoad(1));
            //check is first parameter null
            instructionList.add(new IfNull(label));
            //if the first parameter not null
            //load this from register 0
            instructionList.add(new ALoad(0));
            //load first parameter from register 1
            instructionList.add(new ALoad(1));
            //set this-field to value of register 1
            instructionList.add(new PutField(info.getFieldRefInfo(typeImpl.getThis())));
            //jump to end
            instructionList.add(new Goto(jmpOver));
            //if the first parameter is null
            //load this from register 0 and bind to this instructon a stack map frame
            table.addStackMapFrame(instructionList.add(label, new ALoad(0)));
            //duplicate the this reference
            instructionList.add(new Dup());
            //set this-field to this-reference
            instructionList.add(new PutField(info.getFieldRefInfo(typeImpl.getThis())));

            //bind jmpOver to load 0 instruction
            table.addStackMapFrame(instructionList.add(jmpOver, new ALoad(0)));


            //load all other parameters from register 
            for (int i = 1; i < constructorInfo.getDescriptor().getParameterCount(); i++) {
                instructionList.add(new ALoad(i + 1));
            }


            //call the $init$-method
            MethodRefInfo init = MethodRefInfo.make(typeImpl.getThis(), "$init$", FieldDescriptor.getVoid(), subList.toArray(new FieldDescriptor[0]));
            instructionList.add(new InvokeVirtual(init));
            //return
            instructionList.add(new Return());

        } catch (ByteCodeException | IOException | ResolveException ex) {
            ex.printStackTrace(System.err);
            System.exit(-1);
        }
    }
}
