/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package stage3;

import abstractsyntaxtree.ExpressionNode;
import abstractsyntaxtree.InfixOpNode;
import codegeneration.Visitor;
import codegeneration.abstractvisitors.AbstractArgumentsVisitor;
import codegeneration.environment.ExpressionEnvironment;
import codegeneration.environment.ShortCircuitEnvironment;
import codegeneration.environment.Stage3Environment;
import codegeneration.instruction.InstructionCreator;
import exceptions.CompilerException;
import java.util.ArrayList;
import java.util.Deque;
import java.util.LinkedList;
import java.util.List;
import stage3.states.BlockState;
import teajay.bytecode.attributes.code.Label;
import teajay.bytecode.attributes.code.NOP;
import teajay.generics.parser.Type;

/**
 *
 * @author Michael Burzan
 */
class Stage3ArgumentsVisitor extends AbstractArgumentsVisitor {

    private final Stage3Environment stage3Env;
    private Deque<InfixOpNode.InfixOps> stack;
    private List<Type> argumentList = new LinkedList<>();
    private ShortCircuitEnvironment oldCircEnv;

    public Stage3ArgumentsVisitor(Stage3Environment stage3Env) {
        this.stage3Env = stage3Env;
    }

    @Override
    public void begin(ExpressionNode node) throws CompilerException {
        /*
         * Save old expression environment and short circuit environment
         * then visit the expression nodes with fresh environments
         */
        stack = new LinkedList<>();
        oldCircEnv = stage3Env.getExpressionEnvironment().getShortCircuit();
        stage3Env.createNewExpressionEnvironment();
        stage3Env.getExpressionEnvironment().setShortCircuit(null);
        ExpressionEnvironment env = stage3Env.getExpressionEnvironment();

        for (Type t : argumentList) {
            env.setResultType(t);
        }
    }

    @Override
    public Visitor visit(ExpressionNode node) throws CompilerException {
        return new Stage3ExpressionVisitor(stage3Env, this.stack, true);
    }

    @Override
    public void end(ExpressionNode node) throws CompilerException {
        /*
         * Get environtments
         * restore the old expression environment
         * write operators to instruction list
         * 
         */
        ExpressionEnvironment argEnv = stage3Env.getExpressionEnvironment();
        ShortCircuitEnvironment shortCircuit = argEnv.getShortCircuit();

        stage3Env.restoreOldExpressionEnvironment();
        ExpressionEnvironment env = stage3Env.getExpressionEnvironment();
        InstructionCreator.PutOperatorInstructions(argEnv, stack, node);
        // has expression short circuit  
        if (shortCircuit != null ) {
            BlockState hasState = stage3Env.hasState(BlockState.class);
            Label label = new Label("ParExpression short circuit gotoLabel");
            //create correct operand stack
            List<Type> merged = new ArrayList<>();
            merged.addAll(env.getOperandStack());
            merged.addAll(argumentList);
             
            shortCircuit.write(stage3Env, label, merged.toArray(new Type[0]));

            hasState.addLabel(label);
            //argumentList.add(new Type.ClassType(null, "java/lang/Boolean", null));
            merged.add(new Type.ClassType(null, "java/lang/Boolean", null));
            stage3Env.getExpressionEnvironment().insertCodeWithVerificationStack(new NOP(),merged.toArray(new Type[0]));
            
            argEnv.setResultType(new Type.ClassType(null, "java/lang/Boolean", null));
        }
        
        /*
         *  was call parameter like this expression ( a = 2 )
         *  then the generated type is a LValue 
         */
        if (argEnv.isLvalue()) {
            env.pushCallParameter(argEnv.getLValueResultType());
            argumentList.add(argEnv.getLValueResultType());
        } else {
            env.pushCallParameter(argEnv.typeOfResult());
            argumentList.add(argEnv.typeOfResult());
        }
        //Restore old environment
        env.setShortCircuit(oldCircEnv);

    }
}
