/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package stage3;

import abstractsyntaxtree.ArrayCreatorRestNode;
import abstractsyntaxtree.ClassCreatorRestNode;
import abstractsyntaxtree.CreatedNameNode;
import abstractsyntaxtree.NonWildcardTypeArgumentsNode;
import abstractsyntaxtree.SyntaxNode;
import codegeneration.Visitor;
import codegeneration.abstractvisitors.AbstractCreatorVisitor;
import codegeneration.environment.ExpressionEnvironment;
import codegeneration.environment.Stage3Environment;
import exceptions.CompilerException;
import java.io.IOException;
import stage3.utils.InformationCenter;
import teajay.bytecode.ClassFile;
import teajay.bytecode.attributes.code.AConstNull;
import teajay.bytecode.attributes.code.Dup;
import teajay.bytecode.attributes.code.New;
import teajay.bytecode.cpool.ClassInfo;
import teajay.bytecode.exceptions.ByteCodeException;
import teajay.generics.parser.Type;
import teajay.runtime.exceptions.ResolveException;
import teajay.util.io.ClassFinder;
import teajay.util.tjattribute.TeaJayAttribute;
import static teajay.util.tjattribute.TeaJayAttribute.Type.Class;
import static teajay.util.tjattribute.TeaJayAttribute.Type.Closure;
import static teajay.util.tjattribute.TeaJayAttribute.Type.Enum;
import static teajay.util.tjattribute.TeaJayAttribute.Type.Hidden;
import static teajay.util.tjattribute.TeaJayAttribute.Type.Impl;
import static teajay.util.tjattribute.TeaJayAttribute.Type.Interface;
import static teajay.util.tjattribute.TeaJayAttribute.Type.TJBuiltinType;
import static teajay.util.tjattribute.TeaJayAttribute.Type.Typedef;

/**
 *
 * @author Michael Burzan
 */
class Stage3CreatorVisitor extends AbstractCreatorVisitor {

    private final Stage3Environment stage3Env;
    private boolean invokeDynamic;
    private Type typeOfTypeDef;
    private Stage3CreatedNameVisitor createdNameVisitor;
    private Stage3NonWildcardTypeArgumentsVisitor nonWildcardTypeArguments;
    private Type resultType;
    private boolean hasNull = false;

    public Stage3CreatorVisitor(Stage3Environment stage3Env) {
        this.stage3Env = stage3Env;
    }

    @Override
    public void begin(NonWildcardTypeArgumentsNode node) throws CompilerException {
    }

    @Override
    public void begin(CreatedNameNode node) throws CompilerException {
    }

    @Override
    public void begin(ClassCreatorRestNode node) throws CompilerException {
    }

    @Override
    public void begin(ArrayCreatorRestNode node) throws CompilerException {
    }

    @Override
    public Visitor visit(NonWildcardTypeArgumentsNode node) throws CompilerException {
        return nonWildcardTypeArguments = new Stage3NonWildcardTypeArgumentsVisitor(stage3Env);
    }

    @Override
    public Visitor visit(CreatedNameNode node) throws CompilerException {
        return this.createdNameVisitor = new Stage3CreatedNameVisitor(stage3Env);
    }

    @Override
    public Visitor visit(ClassCreatorRestNode node) throws CompilerException {
        
        if (invokeDynamic) {
            //generate new with invoke dynamic
            return new Stage3ClassCreatorRestVisitor(stage3Env, typeOfTypeDef, true, hasNull);
        } else {
            //generate new with <init> call
            return new Stage3ClassCreatorRestVisitor(stage3Env, resultType, false, hasNull);
        }
    }

    @Override
    public Visitor visit(ArrayCreatorRestNode node) throws CompilerException {
        throw new CompilerException("Array creator not supported in teajay", node.getSymbol());
    }

    @Override
    public void end(NonWildcardTypeArgumentsNode node) throws CompilerException {
        throw new CompilerException("new with Typearguments not supported in teajay");
    }

    @Override
    public void end(CreatedNameNode node) throws CompilerException {
        String ident = node.getIdentifier();
        ident = stage3Env.getSymbolFacade().getFullQualifiedId(ident);
        resultType = typeInterference(ident, node);
        ExpressionEnvironment env = stage3Env.getExpressionEnvironment();
        try {
            if (ident == null) {
                throw new CompilerException("new requires reference type, found: "+node.getIdentifier(), node.getSymbol());
            }

            ClassFile typeDef = ClassFinder.instance().find(ident).get();
            //Check flags
            TeaJayAttribute.AccessType teaJayAccessType = InformationCenter.getTeaJayAccessType(typeDef);
            if (teaJayAccessType == TeaJayAttribute.AccessType.Abstract) {
                throw new CompilerException("cannot instantiate abstract types");
            } else if (InformationCenter.getTeaJayAccessType(typeDef) == TeaJayAttribute.AccessType.Hidden) {
                throw new CompilerException("type has hidden access");
            }
            //check typedef attribute
            switch (InformationCenter.getTeaJayAttributeType(typeDef)) {
                case Closure:
                    throw new CompilerException("cannot instantiate closures", node.getSymbol());
                case Impl:
                    hasNull = true;
                    if (this.stage3Env.getClassFile().getThis().getClassName().equals(ident)) {
                        /*env.insertCode(new New(new ClassInfo(ident)));
                        env.insertCode(new Dup());*/
                        env.insertCode(new AConstNull());
                        //env.setResultType(resultType);
                        //env.setResultType(resultType);
                        env.setResultType(new Type.ClassType(null, "null", null));
                        env.pushCallParameter(resultType);
                    } else {
                        throw new CompilerException("cannot instantiate TeaJay implementations", node.getSymbol());
                    }
                    break;
                case Typedef:
                    hasNull = true;
                    env.insertCode(new AConstNull());
                    env.setResultType(new Type.ClassType(null, "null", null));
                    this.invokeDynamic = true;
                    typeOfTypeDef = typeInterference(ident, node);
                    break;
                case Enum:
                    throw new CompilerException("cannot instantiate enums", node.getSymbol());
                case Interface:
                    throw new CompilerException("cannot instantiate interfaces", node.getSymbol());
                case TJBuiltinType:
                case Class:
                    /*env.insertCode(new New(new ClassInfo(ident)));
                    env.insertCode(new Dup());
                    env.setResultType(resultType);
                    env.setResultType(resultType);*/
                    break;
                case Hidden:
                    throw new CompilerException("Direct create of hidden class is not allowed", node.getSymbol());
                default:
                    throw new AssertionError("No such case: " + InformationCenter.getTeaJayAttributeType(typeDef));
            }
        } catch (ByteCodeException | IOException | ResolveException ex) {
            ex.printStackTrace(System.err);
            System.exit(-1);
        }
    }

    @Override
    public void end(ClassCreatorRestNode node) throws CompilerException {
    }

    @Override
    public void end(ArrayCreatorRestNode node) throws CompilerException {
    }


    /*
     * This code is only a experiment
     */
    public Type typeInterference(String ident,SyntaxNode node) throws CompilerException{
        if (createdNameVisitor != null) {
            if (createdNameVisitor.isDiamond()) {
                Type expectedType = stage3Env.tryToTypeInference(node);
                if (!(expectedType instanceof Type.ClassType)) {
                    throw new CompilerException("Type inference is not possible", node.getSymbol());
                } else {
                    Type.ClassType cType = (Type.ClassType) expectedType;
                    typeOfTypeDef = new Type.ClassType(null, ident, cType.typeArgs);
                }
            } else {
                typeOfTypeDef = new Type.ClassType(null, ident, createdNameVisitor.getTypeParams());
            }
        } else {
            typeOfTypeDef = new Type.ClassType(null, ident, null);
        }
        return typeOfTypeDef;
    }
}
