/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package stage3;

import abstractsyntaxtree.Expression3Node;
import abstractsyntaxtree.InfixOpNode;
import abstractsyntaxtree.InfixOpNode.InfixOps;
import abstractsyntaxtree.TypeNode;
import codegeneration.Visitor;
import codegeneration.abstractvisitors.AbstractExpression2RestVisitor;
import codegeneration.environment.ExpressionEnvironment;
import codegeneration.environment.ShortCircuitEnvironment;
import codegeneration.environment.Stage3Environment;
import codegeneration.instruction.InstructionCreator;
import exceptions.CompilerException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Deque;
import java.util.Iterator;
import java.util.List;
import stage3.utils.TypeChecker;
import teajay.bytecode.attributes.code.IMode;
import teajay.bytecode.attributes.code.If0;
import teajay.bytecode.attributes.code.InvokeVirtual;
import teajay.bytecode.attributes.code.Label;
import teajay.bytecode.attributes.code.NOP;
import teajay.bytecode.attributes.code.PlaceholderInstruction;
import teajay.bytecode.attributes.code.VerificationType;
import teajay.bytecode.cpool.FieldDescriptor;
import teajay.bytecode.cpool.MethodRefInfo;
import teajay.bytecode.exceptions.ByteCodeException;
import teajay.generics.check.TypeException;
import teajay.generics.check.TypeUtils;
import teajay.generics.parser.Type;

/**
 *
 * @author Michael Burzan
 */
class Stage3Expression2RestVisitor extends AbstractExpression2RestVisitor {

    static int i = 0;
    private final Stage3Environment stage3Env;
    private final Deque<InfixOpNode.InfixOps> stack;
    private Stage3TypeVisitor stage3TypeVisitor;
    private ExpressionEnvironment tempExpEnv;
    private ExpressionEnvironment argEnv;
    private int opPriority;
    private Label falseLabel;
    private Label trueLabel;

    Stage3Expression2RestVisitor(Stage3Environment stage3Env, Deque<InfixOpNode.InfixOps> stack) {
        this.stage3Env = stage3Env;
        this.stack = stack;

        falseLabel = new Label("FalseLabel:");
        trueLabel = new Label("trueLabel:");
    }

    @Override
    public void begin(Expression3Node node) throws CompilerException {
    }

    @Override
    public void begin(InfixOpNode node) throws CompilerException {
    }

    @Override
    public void begin(TypeNode node) throws CompilerException {
    }

    @Override
    public Visitor visit(Expression3Node node) throws CompilerException {
        this.stage3Env.getExpressionEnvironment().setRValue();
        return new Stage3Expression3Visitor(this.stage3Env, this.stack, false);

    }

    @Override
    public void visit(InfixOpNode node) throws CompilerException {
        ExpressionEnvironment env = stage3Env.getExpressionEnvironment();

        env.writePrefix(node);
        if (env.isLvalue()) {
            throw new CompilerException("Not a statement", node.getSymbol());
        }
        InfixOpNode.InfixOps operator = node.getOperator();

        if (!this.stack.isEmpty()) {
            int stackPriority = this.stack.peek().getPriority();
            this.opPriority = operator.getPriority();
            if (stackPriority >= opPriority) {               
                    InstructionCreator.PutOperatorInstructionsByPriority(env, stack, opPriority, node);
            }
        }
        Type typeOfResult = null;
        try {
            //Short circut expression
            if (operator == InfixOps.AND) {

                typeOfResult = stage3Env.getExpressionEnvironment().popType();
                stage3Env.getTypeUtils().checkAssignability(new Type.ClassType(null, "java/lang/Boolean", null), typeOfResult);
                MethodRefInfo ref = MethodRefInfo.make("java/lang/Boolean", "booleanValue", "boolean");
                stage3Env.getExpressionEnvironment().insertCode(new InvokeVirtual(ref));

                PlaceholderInstruction andPlaceholder = new PlaceholderInstruction();
                if (!stage3Env.getExpressionEnvironment().isShortCircuit()) {
                    stage3Env.getExpressionEnvironment().setShortCircuit(new ShortCircuitEnvironment(trueLabel, falseLabel, operator));
                    stage3Env.getExpressionEnvironment().insertCode(andPlaceholder);
                    stage3Env.getExpressionEnvironment().getShortCircuit().addBeginAnd(andPlaceholder);
                } else {
                    NOP nop = new NOP();
                    stage3Env.getExpressionEnvironment().insertCode(andPlaceholder);

                    List<VerificationType> verficationTypes = new ArrayList<>();
                    for (Type type : stage3Env.getExpressionEnvironment().getOperandStack()) {
                        FieldDescriptor descr = stage3Env.getTypeUtils().makeFieldDescriptor(type);
                        verficationTypes.add(VerificationType.valueOf(descr));
                    }
                    stage3Env.getLocalTable().addStackMapFrame(
                            stage3Env.getMethodInfo().getCode().getInstructionList().add(nop), verficationTypes);
                    stage3Env.getExpressionEnvironment().getShortCircuit().addNextAnd(andPlaceholder, nop);
                }


            } else if (operator == InfixOps.OR) {
                typeOfResult = stage3Env.getExpressionEnvironment().popType();
                stage3Env.getTypeUtils().checkAssignability(new Type.ClassType(null, "java/lang/Boolean", null), typeOfResult);
                MethodRefInfo ref = MethodRefInfo.make("java/lang/Boolean", "booleanValue", "boolean");
                stage3Env.getExpressionEnvironment().insertCode(new InvokeVirtual(ref));
                PlaceholderInstruction orPlaceholder = new PlaceholderInstruction();
                if (!stage3Env.getExpressionEnvironment().isShortCircuit()) {
                    stage3Env.getExpressionEnvironment().setShortCircuit(new ShortCircuitEnvironment(trueLabel, falseLabel, operator));
                    stage3Env.getExpressionEnvironment().insertCode(orPlaceholder);
                    stage3Env.getExpressionEnvironment().getShortCircuit().addBeginOr(orPlaceholder);
                } else {
                    NOP nop = new NOP();
                    stage3Env.getExpressionEnvironment().insertCode(orPlaceholder);

                    List<VerificationType> verficationTypes = new ArrayList<>();
                    for (Type type : stage3Env.getExpressionEnvironment().getOperandStack()) {
                        FieldDescriptor descr = stage3Env.getTypeUtils().makeFieldDescriptor(type);
                        verficationTypes.add(VerificationType.valueOf(descr));
                    }
                    stage3Env.getLocalTable().addStackMapFrame(
                            stage3Env.getMethodInfo().getCode().getInstructionList().add(nop), verficationTypes);
                    stage3Env.getExpressionEnvironment().getShortCircuit().addNextOr(orPlaceholder, nop);
                }
            } else {
                stack.push(operator);
            }

        } catch (ByteCodeException ex) {
            ex.printStackTrace(System.err);
            System.exit(-1);
        } catch (TypeException ex) {
            throw new CompilerException("Requiered: java/lang/Boolean Found: " + typeOfResult, node.getSymbol());
        }
    }

    @Override
    public Visitor visit(TypeNode node) throws CompilerException {
        return stage3TypeVisitor = new Stage3TypeVisitor(stage3Env);
    }

    @Override
    public void end(Expression3Node node) throws CompilerException {
        Type typeOfResult = null;
        stage3Env.getExpressionEnvironment().writePrefix(node);
        try {
            ShortCircuitEnvironment shortCircuit = stage3Env.getExpressionEnvironment().getShortCircuit();

            if (shortCircuit != null && !shortCircuit.released()) {
                switch (shortCircuit.getOperator()) {
                    case AND:
                        InstructionCreator.PutOperatorInstructions(stage3Env.getExpressionEnvironment(), stack, node);
                        typeOfResult = stage3Env.getExpressionEnvironment().popType();
                        //stage3Env.getExpressionEnvironment().popType();
                        stage3Env.getTypeUtils().checkAssignability(new Type.ClassType(null, "java/lang/Boolean", null), typeOfResult);
                        MethodRefInfo ref = MethodRefInfo.make("java/lang/Boolean", "booleanValue", "boolean");
                        stage3Env.getExpressionEnvironment().insertCode(new InvokeVirtual(ref));
                        stage3Env.getExpressionEnvironment().insertCode(new If0(IMode.eq, shortCircuit.getFalseLabel())); //GOTO FALSE
                        stage3Env.getExpressionEnvironment().getShortCircuit().release();
                        stage3Env.getExpressionEnvironment().setResultType(typeOfResult);
                        break;
                    case OR:
                        InstructionCreator.PutOperatorInstructions(stage3Env.getExpressionEnvironment(), stack, node);
                        typeOfResult = stage3Env.getExpressionEnvironment().popType();
                        //stage3Env.getExpressionEnvironment().popType();
                        stage3Env.getTypeUtils().checkAssignability(new Type.ClassType(null, "java/lang/Boolean", null), typeOfResult);
                        ref = MethodRefInfo.make("java/lang/Boolean", "booleanValue", "boolean");
                        stage3Env.getExpressionEnvironment().insertCode(new InvokeVirtual(ref));
                        stage3Env.getExpressionEnvironment().insertCode(new If0(IMode.eq, shortCircuit.getFalseLabel())); //GOTO FALSE
                        stage3Env.getExpressionEnvironment().getShortCircuit().release();
                        stage3Env.getExpressionEnvironment().setResultType(typeOfResult);
                        break;

                    default:
                        throw new AssertionError(shortCircuit.getOperator().name());

                }
            }
        } catch (ByteCodeException ex) {
            ex.printStackTrace(System.err);
            System.exit(-1);
        } catch (TypeException ex) {
            throw new CompilerException("Requiered: java/lang/Boolean Found: " + typeOfResult, node.getSymbol());
        }

    }

    @Override
    public void end(InfixOpNode node) throws CompilerException {
    }

    @Override
    public void end(TypeNode node) throws CompilerException {
    }
}




/*
     if (opPriority <= 1) { // operator is <=, < , == , > ,>= , ||  or && 
                    //This code simulate that the left expression is a paranthesis expression
    } else {
                    InstructionCreator.PutOperatorInstructionsByPriority(env, stack, opPriority, node);
                    
                     Please don't delete this commented code. Because it helps me!
                    if (opPriority == 3) {
                        InstructionCreator.PutOperatorInstructions(env, stack, node);
                    } else {
                        System.err.println("Stack2:" + stack);
                        InfixOpNode.InfixOps firstOperator = this.stack.pop();
                        InstructionCreator.PutOperatorInstruction(env, operator, node);

                    }

                }*/