/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package stage3;

import abstractsyntaxtree.ArgumentsNode;
import abstractsyntaxtree.AssignmentOperatorNode;
import abstractsyntaxtree.ExplicitGenericInvocationNode;
import abstractsyntaxtree.ExpressionNode;
import abstractsyntaxtree.InfixOpNode;
import abstractsyntaxtree.InnerCreatorNode;
import abstractsyntaxtree.NonWildcardTypeArgumentsNode;
import codegeneration.Visitor;
import codegeneration.abstractvisitors.AbstractIdentifierSuffixVisitor;
import codegeneration.environment.ExpressionEnvironment;
import codegeneration.environment.ShortCircuitEnvironment;
import codegeneration.environment.Stage3Environment;
import codegeneration.instruction.InstructionCreator;
import codegeneration.instruction.MethodInvokeCreator;
import exceptions.CompilerException;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Deque;
import java.util.LinkedList;
import java.util.List;
import stage3.states.BlockState;
import teajay.bytecode.attributes.code.ALoad;
import teajay.bytecode.attributes.code.AStore;
import teajay.bytecode.attributes.code.Dup;
import teajay.bytecode.attributes.code.Dup2;
import teajay.bytecode.attributes.code.Label;
import teajay.bytecode.attributes.code.NOP;
import teajay.bytecode.attributes.code.Swap;
import teajay.bytecode.exceptions.ByteCodeException;
import teajay.generics.parser.Type;
import teajay.runtime.exceptions.ResolveException;

/**
 *
 * @author Michael Burzan
 */
class Stage3IdentifierSuffixVisitor extends AbstractIdentifierSuffixVisitor {

    private final Stage3Environment stage3Env;
    private Deque<InfixOpNode.InfixOps> stack;
    private final int dim;
    private int size;
    private Stage3ExplicitGenericInvocationVisitor explicitGenericInvocationVisitor;

    Stage3IdentifierSuffixVisitor(Stage3Environment stage3Env, int dim) {
        this.stage3Env = stage3Env;
        this.dim = dim;
        this.size = dim;
    }

    @Override
    public void begin(ExplicitGenericInvocationNode node) throws CompilerException {
    }

    @Override
    public void begin(NonWildcardTypeArgumentsNode node) throws CompilerException {
    }

    @Override
    public void begin(InnerCreatorNode node) throws CompilerException {
    }

    @Override
    public void begin(ExpressionNode node) throws CompilerException {
        /*
         * Visit [ exp ]
         */

        stack = new LinkedList<>();
        ExpressionEnvironment env = stage3Env.getExpressionEnvironment();
        if (env.isLvalue()) {
            try {
                env.writeLValueInstruction();

            } catch (ByteCodeException ex) {
                ex.printStackTrace(System.err);
                System.exit(-1);
            }
        }
        stage3Env.createNewExpressionEnvironment();

    }

    @Override
    public void begin(ArgumentsNode node) throws CompilerException {
    }

    @Override
    public Visitor visit(ExplicitGenericInvocationNode node) throws CompilerException {
        return explicitGenericInvocationVisitor = new Stage3ExplicitGenericInvocationVisitor(stage3Env);
    }

    @Override
    public Visitor visit(NonWildcardTypeArgumentsNode node) throws CompilerException {
        return new Stage3NonWildcardTypeArgumentsVisitor(stage3Env);
    }

    @Override
    public Visitor visit(InnerCreatorNode node) throws CompilerException {
        return new Stage3InnerCreatorVisitor(stage3Env);
    }

    /*This method is for edge calls*/
    @Override
    public Visitor visit(ExpressionNode node) throws CompilerException {
        return new Stage3ExpressionVisitor(stage3Env, this.stack, true);
    }

    @Override
    public Visitor visit(ArgumentsNode node) throws CompilerException {
        return new Stage3ArgumentsVisitor(stage3Env);
    }

    @Override
    public void end(ExplicitGenericInvocationNode node) throws CompilerException {
    }

    @Override
    public void end(NonWildcardTypeArgumentsNode node) throws CompilerException {
    }

    @Override
    public void end(InnerCreatorNode node) throws CompilerException {
    }

    @Override
    public void end(ExpressionNode node) throws CompilerException {
        try {

            ExpressionEnvironment argEnv = stage3Env.getExpressionEnvironment();
            InstructionCreator.PutOperatorInstructions(argEnv, stack, node);

            stage3Env.restoreOldExpressionEnvironment();
            ExpressionEnvironment env = stage3Env.getExpressionEnvironment();
            

            // exp := exp1 = exp2[exp3]
            if (env.isRvalue()) {
                Type t = argEnv.typeOfResult();
                env.pushCallParameter(t);
                MethodInvokeCreator.CallContext callContext = new MethodInvokeCreator.CallContext(stage3Env, env.typeOfResult(), "operator_get", 1);
                MethodInvokeCreator.createMethodCall(callContext, env, "operator_get", null, node);
            }
            // exp := exp1[exp*][exp*]... = exp3
            if (env.isLvalue() && env.getLValueOperator() == AssignmentOperatorNode.Operators.SET) {
                if (size > 1) {
                    Type t = argEnv.typeOfResult();
                    env.pushCallParameter(t);
                    MethodInvokeCreator.CallContext callContext = new MethodInvokeCreator.CallContext(stage3Env, env.typeOfResult(), "operator_get", 1);
                    MethodInvokeCreator.createMethodCall(callContext, env, "operator_get", null, node);
                    size--;
                } else {
                    //the last [ exp ] is a operator_set call
                    Type t = argEnv.typeOfResult();
                    env.setLValueCall(env.typeOfResult(), t);
                }

            } else if (env.isLvalue()) {
                //exp := exp1[exp*][exp*]... op= exp3
                if (size > 1) {
                    Type t = argEnv.typeOfResult();
                    env.pushCallParameter(t);
                    MethodInvokeCreator.CallContext callContext = new MethodInvokeCreator.CallContext(stage3Env, env.typeOfResult(), "operator_get", 1);
                    MethodInvokeCreator.createMethodCall(callContext, env, "operator_get", null, node);
                    size--;
                } else {
                    env.insertCode(new Dup2());

                    Type t = argEnv.typeOfResult();
                    env.setLValueCall(null, t);
                    env.pushCallParameter(t);
                    Type z = env.typeOfResult();
                    env.setResultType(z);
                    MethodInvokeCreator.CallContext callContext = new MethodInvokeCreator.CallContext(stage3Env, env.typeOfResult(), "operator_get", 1);
                    MethodInvokeCreator.createMethodCall(callContext, env, "operator_get", null, node);
                }
            }

        } catch (ByteCodeException | IOException | ResolveException ex) {
            ex.printStackTrace(System.err);
            System.exit(-1);
        }

    }

    @Override
    public void end(ArgumentsNode node) throws CompilerException {
    }

    /**
     * @description: returns the method name of a static generic call
     * @return String
     */
    public String getGenericMethodName() {
        if (explicitGenericInvocationVisitor != null) {
            return explicitGenericInvocationVisitor.getMethodName();
        } else {
            return null;
        }
    }

    /**
     * @description: returns the generic type arguments of a static generic call
     * @return List<Type>
     */
    public List<Type> getGenericArguments() {
        if (explicitGenericInvocationVisitor != null) {
            return explicitGenericInvocationVisitor.getGenericArgs();
        } else {
            return null;
        }
    }
}
