/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package stage3;

import abstractsyntaxtree.BlockNode;
import abstractsyntaxtree.FormalParametersNode;
import abstractsyntaxtree.QualifiedIdentifierListNode;
import codegeneration.Visitor;
import codegeneration.abstractvisitors.AbstractMethodeDeclaratorRestVisitor;
import codegeneration.environment.ExpressionEnvironment;
import codegeneration.environment.Stage3Environment;
import datastructs.Pair;
import exceptions.CompilerException;
import java.util.Arrays;
import java.util.Iterator;
import java.util.List;
import stage3.states.BlockState;
import stage3.states.EndState;
import stage3.states.UnreachableCodeState;
import stage3.utils.TypeChecker;
import symbol.LocalTable;
import teajay.bytecode.attributes.code.ALoad;
import teajay.bytecode.attributes.code.AStore;
import teajay.bytecode.attributes.code.InvokeStatic;
import teajay.bytecode.attributes.code.JVMInstructionList;
import teajay.bytecode.attributes.code.Return;
import teajay.bytecode.cpool.MethodRefInfo;
import teajay.bytecode.exceptions.ByteCodeException;
import teajay.bytecode.methods.MethodInfo;
import teajay.generics.parser.Type;

/**
 *
 * @author Michael (Burzan | Kußmann)
 */
class Stage3MethodDeclaratorRestVisitor extends AbstractMethodeDeclaratorRestVisitor {

    private enum MethodKind {

        Constructor,
        StaticConstructor,
        Method;
    }
    private final Stage3Environment stage3Env;
    private String methodName;
    private final Type returnType;
    private final MethodKind kind;
    private List<Pair<Type, String>> formalParameters;
    private List<String> throwsList;
    private Stage3BlockVisitor methodBlock;

    public Stage3MethodDeclaratorRestVisitor(Stage3Environment stage3Env, String methodName, Type returnType) {
        this.stage3Env = stage3Env;
        this.methodName = methodName;
        this.returnType = returnType;
        switch (methodName) {
            case "<init>":
                this.methodName = "$init$";
                this.kind = MethodKind.Constructor;
                break;
            case "<clinit>":
                this.kind = MethodKind.StaticConstructor;
                break;
            default:
                this.kind = MethodKind.Method;
                break;
        }
    }

    @Override
    public void begin(FormalParametersNode node) throws CompilerException {
        this.formalParameters = node.getFormalParameters();
        for (Pair<Type, String> param : formalParameters) {
            TypeChecker.check(param.value1, node, stage3Env);
        }

    }

    @Override
    public void begin(QualifiedIdentifierListNode node) throws CompilerException {
        this.throwsList = node.getQualifiedIdentifierList();
    }

    @Override
    public void begin(BlockNode node) throws CompilerException {
//        try {
        stage3Env.setExpressionEnvironment(new ExpressionEnvironment(stage3Env));
//            stage3Env.getMethodInfo().getCode().getInstructionList().add(new ALoad(0));
//            stage3Env.getMethodInfo().getCode().getInstructionList().add(
//                    new GetField(FieldRefInfo.make(stage3Env.getClassFile().getThis(), "this", FieldDescriptor.fromClassInfo(stage3Env.getClassFile().getThis()))));
//            stage3Env.getMethodInfo().getCode().getInstructionList().add(new AStore(0));

        stage3Env.changeState(new BlockState(), node);
//        } catch (ByteCodeException ex) {
//            ex.printStackTrace(System.err);
//            System.exit(-1);
//        }
    }

    @Override
    public Visitor visit(FormalParametersNode node) throws CompilerException {
        return null;
    }

    @Override
    public Visitor visit(QualifiedIdentifierListNode node) throws CompilerException {
        return null;
    }

    @Override
    public Visitor visit(BlockNode node) throws CompilerException {
        MethodInfo methodInfo = stage3Env.getMethodInfo();
        LocalTable table = new LocalTable(methodInfo, stage3Env.getTypeUtils());
        table.addFrame();
        if (!methodInfo.isStatic()) {
            table.addEntry("this", stage3Env.getThis());
        }
        if (formalParameters != null) {
            createTableEntrysFromFormalParameters(methodInfo, table);
        }


        stage3Env.setLocalTable(table);
        //Make jvm array to TeaJay List
        if (methodInfo.isVarArgs()) {
            Pair<Type, String> varArg = formalParameters.get(formalParameters.size() - 1);
            JVMInstructionList instructionList = methodInfo.getCode().getInstructionList();
            try {
                int index = table.getEntry(varArg.value2).getIndex();
                instructionList.add(new ALoad(index));
                instructionList.add(new InvokeStatic(MethodRefInfo.make("teajay/util/ArrayUtils", "toList", "teajay/lang/List", "[Ljava/lang/Object;")));
                instructionList.add(new AStore(index));
            } catch (ByteCodeException ex) {
                ex.printStackTrace(System.err);
                System.exit(-1);
            }

        }


        if (kind == MethodKind.Constructor && this.stage3Env.isTeaJayImpl()) {
            //This code call the uncallable super constructor from user code 


            return this.methodBlock = new Stage3BlockVisitor(stage3Env);

        } else if (methodInfo.isStatic()) {
            return this.methodBlock = new Stage3BlockVisitor(stage3Env);
        } else {
            return this.methodBlock = new Stage3BlockVisitor(stage3Env);
        }


    }

    @Override
    public void end(FormalParametersNode node) throws CompilerException {
    }

    @Override
    public void end(QualifiedIdentifierListNode node) throws CompilerException {
    }

    @Override
    public void end(BlockNode node) throws CompilerException {
        JVMInstructionList instructionList = this.stage3Env.getMethodInfo().getCode().getInstructionList();
        
        if (kind == MethodKind.Constructor && methodBlock.isEmptyBlock()) {
            stage3Env.writeSuperConstructorCall(node);

        }
        //last statement in non void method must be return or throw
        if (stage3Env.hasState(UnreachableCodeState.class) == null) {
            if (stage3Env.hasState(EndState.class) == null && !this.stage3Env.getMethodInfo().getDescriptor().getReturnType().isVoid()) {
                throw new CompilerException("Missing return statement ", node.getSymbol());
            }
        }
        //if void method then return must be the last statement
        if (this.stage3Env.getMethodInfo().getDescriptor().getReturnType().isVoid()) {
            if (instructionList.size() > 0) {

                BlockState block = stage3Env.hasState(BlockState.class);
                //if not return or throw statement or block is empty set return statement
                if (!block.getLabels().isEmpty() && stage3Env.hasState(UnreachableCodeState.class) == null && stage3Env.hasState(EndState.class) == null) {
                    stage3Env.getExpressionEnvironment().insertCode(new Return());
                } else {
                    if (stage3Env.hasState(EndState.class) == null && stage3Env.hasState(UnreachableCodeState.class) == null ) {
                        stage3Env.getExpressionEnvironment().insertCode(new Return());
                    }
                }
            } else {
                stage3Env.getExpressionEnvironment().insertCode(new Return());
            }
        }
        stage3Env.prevState();
        stage3Env.removeMethodInfo();
    }
    /**
     * Initialize LocalTable with formal parameters
     */
    private void createTableEntrysFromFormalParameters(MethodInfo methodInfo, LocalTable table) {
        for (Iterator<Pair<Type, String>> it = formalParameters.iterator(); it.hasNext();) {
            Pair<Type, String> param = it.next();
            if (!methodInfo.isVarArgs() || it.hasNext()) {
                table.addEntry(param.value2, param.value1);
            } else {
                Type.ArrayType aType = (Type.ArrayType) param.value1;
                table.addEntry(param.value2, new Type.ClassType(null, "teajay/lang/List", Arrays.asList((Type) aType.elemType)));
            }
        }
    }
}
