/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package stage3;

import abstractsyntaxtree.ExpressionNode;
import abstractsyntaxtree.InfixOpNode;
import codegeneration.Visitor;
import codegeneration.abstractvisitors.AbstractParExpressionVisitor;
import codegeneration.environment.ExpressionEnvironment;
import codegeneration.environment.ShortCircuitEnvironment;
import codegeneration.environment.Stage3Environment;
import codegeneration.instruction.InstructionCreator;
import exceptions.CompilerException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Deque;
import java.util.LinkedList;
import java.util.List;
import stage3.states.BlockState;
import teajay.bytecode.attributes.code.Label;
import teajay.bytecode.attributes.code.NOP;
import teajay.generics.parser.Type;

/**
 *
 * @author Michael Burzan
 */
class Stage3ParExpressionVisitor extends AbstractParExpressionVisitor {

    private final Stage3Environment stage3Env;
    private final Deque<InfixOpNode.InfixOps> stack;
    private ShortCircuitEnvironment oldShortCircuitEnv;

    public Stage3ParExpressionVisitor(Stage3Environment stage3Env) {
        this.stage3Env = stage3Env;
        this.stack = new LinkedList<>();
    }

    @Override
    public void begin(ExpressionNode node) throws CompilerException {
        /* save environments
         * make fresh environments
         */
        oldShortCircuitEnv = stage3Env.getExpressionEnvironment().getShortCircuit();
        stage3Env.createNewExpressionEnvironment();
        stage3Env.getExpressionEnvironment().setShortCircuit(null);
    }

    @Override
    public Visitor visit(ExpressionNode node) throws CompilerException {
        return new Stage3ExpressionVisitor(stage3Env, stack, true);
    }

    @Override
    public void end(ExpressionNode node) throws CompilerException {
        //Restore old expression environment
        ExpressionEnvironment argEnv = stage3Env.getExpressionEnvironment();
        stage3Env.restoreOldExpressionEnvironment();
        ExpressionEnvironment env = stage3Env.getExpressionEnvironment();
        
        InstructionCreator.PutOperatorInstructions(argEnv, stack, node);
        //restore old short circuit environment: exp:= exp && ( exp || exp)
        env.setShortCircuit(oldShortCircuitEnv);
        ShortCircuitEnvironment shortCircuit = argEnv.getShortCircuit();
        //has  ( exp ) short circuit. Build semantic
        if (shortCircuit != null) {
            BlockState hasState = stage3Env.hasState(BlockState.class);
            Label label = new Label("ParExpression short circuit gotoLabel");
            shortCircuit.write(stage3Env, label, env.getOperandStack().toArray(new Type[0]));
            hasState.addLabel(label);
            //result type of  parenthisis expression with short circuit is java/lang/Boolean  
            env.setResultType(new Type.ClassType(null, "java/lang/Boolean", null));
            stage3Env.getExpressionEnvironment().insertCodeWithVerificationStack(new NOP(), env.getOperandStack());
        }else{
            //store restlt type of paraenthisis expression
            env.setResultType(argEnv.typeOfResult());
        }
    }
}
