/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package stage3;

import abstractsyntaxtree.ArgumentsNode;
import abstractsyntaxtree.BasicTypeNode;
import abstractsyntaxtree.BasicTypeNode.BasicType;
import abstractsyntaxtree.ClosureCreatorNode;
import abstractsyntaxtree.CreatorNode;
import abstractsyntaxtree.ExplicitGenericInvocationSuffixNode;
import abstractsyntaxtree.ExpressionNode;
import abstractsyntaxtree.IdentifierSuffixNode;
import static abstractsyntaxtree.IdentifierSuffixNode.SuffixType.ARRAY_EXPRESSION;
import static abstractsyntaxtree.IdentifierSuffixNode.SuffixType.ARRAY_SUFFIX;
import static abstractsyntaxtree.IdentifierSuffixNode.SuffixType.CALL;
import static abstractsyntaxtree.IdentifierSuffixNode.SuffixType.EXPLICIT_GENERIC_INVOCATION;
import static abstractsyntaxtree.IdentifierSuffixNode.SuffixType.INNER_CREATOR;
import static abstractsyntaxtree.IdentifierSuffixNode.SuffixType.NON_WILDCARD_TYPE_ARGUMENTS;
import static abstractsyntaxtree.IdentifierSuffixNode.SuffixType.SUFFIX;
import static abstractsyntaxtree.IdentifierSuffixNode.SuffixType.SUFFIX_CALL;
import abstractsyntaxtree.InfixOpNode.InfixOps;
import abstractsyntaxtree.LiteralNode;
import static abstractsyntaxtree.LiteralNode.LiteralType.BOOLEAN;
import static abstractsyntaxtree.LiteralNode.LiteralType.CHAR;
import static abstractsyntaxtree.LiteralNode.LiteralType.DOUBLE;
import static abstractsyntaxtree.LiteralNode.LiteralType.FLOAT;
import static abstractsyntaxtree.LiteralNode.LiteralType.INT;
import static abstractsyntaxtree.LiteralNode.LiteralType.LONG;
import static abstractsyntaxtree.LiteralNode.LiteralType.NULL;
import static abstractsyntaxtree.LiteralNode.LiteralType.STRING;
import abstractsyntaxtree.NonWildcardTypeArgumentsNode;
import abstractsyntaxtree.ParExpressionNode;
import abstractsyntaxtree.SuperSuffixNode;
import abstractsyntaxtree.TypeNode;
import codegeneration.Visitor;
import codegeneration.abstractvisitors.AbstractPrimaryVisitor;
import codegeneration.environment.ExpressionEnvironment;
import codegeneration.environment.Stage3Environment;
import codegeneration.instruction.MethodInvokeCreator;
import codegeneration.instruction.VariableInstructionCreator;
import exceptions.CompilerException;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.Deque;
import java.util.List;
import stage3.states.ConstructorState;
import stage3.states.MethodState;
import symbol.FieldTypeResolver;
import symbol.LocalEntry;
import teajay.bytecode.BaseType;
import teajay.bytecode.attributes.code.AConstNull;
import teajay.bytecode.attributes.code.IConst;
import teajay.bytecode.attributes.code.InvokeStatic;
import teajay.bytecode.attributes.code.JVMInstruction;
import teajay.bytecode.attributes.code.Ldc;
import teajay.bytecode.attributes.code.StackMapFrame;
import teajay.bytecode.cpool.ClassInfo;
import teajay.bytecode.cpool.FieldDescriptor;
import teajay.bytecode.cpool.MethodRefInfo;
import teajay.bytecode.exceptions.ByteCodeException;
import teajay.bytecode.fields.FieldInfo;
import teajay.generics.check.TypeUtils;
import teajay.generics.parser.Type;
import teajay.runtime.exceptions.ResolveException;
import teajay.util.io.ClassFinder;

/**
 *
 * @author Michael (Burzan | Kußmann)
 */
class Stage3PrimaryVisitor extends AbstractPrimaryVisitor {

    private final Stage3Environment stage3Env;
    private Stage3TypeVisitor typeVisitor;
    private MethodInvokeCreator.CallContext context;
    private StackMapFrame frame;
    private JVMInstruction thisCall;
    private Type castType;
    private int castArrayDim;
    private Stage3ExplicitGenericInvocationSuffixVisitor genericInvocationVisitor;
    private ExpressionEnvironment rValueEnv;
    private MethodInvokeCreator.CallContext thisCallContext;
    private Stage3IdentifierSuffixVisitor identSuffixVisitor;
    private final Deque<InfixOps> stack;
    private String thisCallIdent;

    public Stage3PrimaryVisitor(Stage3Environment stage3Env, Deque<InfixOps> stack) {
        this.stage3Env = stage3Env;
        this.stack = stack;
    }

    @Override
    public void begin(LiteralNode node) throws CompilerException {
        stage3Env.writeSuperConstructorCall(node);
        LiteralNode.LiteralType literalType = node.getLiteralType();
        // JVMInstructionList instructionList = this.env.info.getCode().getInstructionList();
        String val = node.getLiteralValue();
        ExpressionEnvironment env = this.stage3Env.getExpressionEnvironment();
        try {
            switch (literalType) {
                case BOOLEAN:
                    Type.ClassType booleanType = new Type.ClassType(null, "java/lang/Boolean", null);
                    env.setResultType(booleanType);
                    Boolean valueOf = Boolean.valueOf(val);
                    //JVM has not boolean type
                    if (valueOf == true) {
                        env.insertCode(new IConst(1));
                    } else {
                        env.insertCode(new IConst(0));
                    }
                    env.insertCode(new InvokeStatic(MethodRefInfo.make("java/lang/Boolean",
                            "valueOf",
                            FieldDescriptor.fromClass(Boolean.class), FieldDescriptor.fromBaseType(BaseType.Boolean))));
                    break;
                case INT:
                case LONG:
                case FLOAT:
                case DOUBLE:
                case CHAR:
                    Type.ClassType numberType = new Type.ClassType(null, "teajay/lang/Number", null);
                    env.setResultType(numberType);
                    //env.instructionTypes.add(this.env.resultType);
                    MethodRefInfo ref = MethodRefInfo.make(new ClassInfo("teajay/lang/Number"), "$valueOf", FieldDescriptor.fromString("teajay/lang/Number"), FieldDescriptor.getString());
                    //Set constant to instruction list as string
                    env.insertCode(new Ldc(val));
                    //call convert method valueOf
                    env.insertCode(new InvokeStatic(ref));
                    break;
                case STRING:
                    env.setResultType(new Type.ClassType(null, "java/lang/String", null));
                    env.insertCode(new Ldc(val));
                    break;
                case NULL:
                    //new Type.ClassType(null,"null",null);
                    env.setResultType(new Type.ClassType(null, "null", null));
                    env.insertCode(new AConstNull());
                    break;
                default:
                    throw new AssertionError(literalType.name());

            }
        } catch (ByteCodeException ex) {
            ex.printStackTrace(System.err);
        }
    }

    @Override
    public void begin(TypeNode node) throws CompilerException {
        stage3Env.writeSuperConstructorCall(node);
    }

    @Override
    public void begin(ParExpressionNode node) throws CompilerException {
        stage3Env.writeSuperConstructorCall(node);
    }

    @Override
    public void begin(ArgumentsNode node) throws CompilerException {
        MethodState mstate = stage3Env.hasState(MethodState.class);
        if (mstate == null) {
            ConstructorState cState = stage3Env.hasState(ConstructorState.class);
            if (cState == null) {
                throw new CompilerException("this(...) call outside constructor or closure");
            }
            if (cState.hasSuper()) {
                throw new CompilerException("this(...) call must be the first statement in a constructor");
            }
            try {
                thisCallContext = MethodInvokeCreator.createMethodAccess(stage3Env, stage3Env.getExpressionEnvironment(), "$init$", Collections.EMPTY_LIST, false, node);
                thisCallIdent = "$init$";
            } catch (IOException | ByteCodeException | ResolveException ex) {
                ex.printStackTrace(System.err);
                System.exit(-1);
            }
            cState.setHasSuper();
        } else {
            if (mstate.isClosure()) {
                try {
                    thisCallContext = MethodInvokeCreator.createMethodAccess(stage3Env, stage3Env.getExpressionEnvironment(), "@1", Collections.EMPTY_LIST, true, mstate.getClosureType(), node);
                    thisCallContext.overrideOwnerclass(mstate.getClosureType());
                    thisCallIdent = "@1";
                } catch (IOException | ByteCodeException | ResolveException ex) {
                    ex.printStackTrace(System.err);
                    System.exit(-1);
                }
            } else {
                throw new CompilerException("this(...) call outside constructor or closure");
            }
        }
    }

    @Override
    public void begin(SuperSuffixNode node) throws CompilerException {
    }

    @Override
    public void begin(CreatorNode node) throws CompilerException {
        stage3Env.writeSuperConstructorCall(node);
    }

    @Override
    public void begin(NonWildcardTypeArgumentsNode node) throws CompilerException {
        throw new CompilerException("bogus generic arguments", node.getSymbol());
        //stage3Env.writeSuperConstructorCall(node);
    }

    @Override
    public void begin(ExplicitGenericInvocationSuffixNode node) throws CompilerException {
        stage3Env.writeSuperConstructorCall(node);

    }

    /**
     * create: identifier ({.identifier} | . <Type> identifier )( [ exp ] | "("
     * Arguments ")")
     *
     * @param node
     * @throws CompilerException
     */
    @Override
    public void begin(IdentifierSuffixNode node) throws CompilerException {
        try {
            stage3Env.writeSuperConstructorCall(node);
            switch (node.getSuffixType()) {
                case SUFFIX_CALL:
                    break;
                case CALL:
                    //call
                    this.context = MethodInvokeCreator.
                            createMethodAccess(stage3Env, stage3Env.getExpressionEnvironment(), node.getIdentifier(), node.getIdentifierList(), node);
                    this.context.setNonPublic();
                    break;
                case SUFFIX:
                case ARRAY_SUFFIX: {
                    ExpressionEnvironment env = stage3Env.getExpressionEnvironment();
                    int dimension = node.getDim();
                    if (node.getSuffixes() == IdentifierSuffixNode.Suffixes.CLASS) {
                        String typeName = node.getIdentifier();
                        String fullQualifiedId;
                        if (node.getIdentifierList() == null || node.getIdentifierList().isEmpty()) {
                            fullQualifiedId = stage3Env.getSymbolFacade().getFullQualifiedId(typeName);
                        } else {
                            StringBuilder b = new StringBuilder();
                            b.append(node.getIdentifier());
                            for (String str : node.getIdentifierList()) {
                                b.append('/').append(str);
                            }
                            fullQualifiedId = b.toString();
                            if (ClassFinder.instance().find(fullQualifiedId) == null) {
                                throw new CompilerException("Type " + fullQualifiedId + " not found", node.getSymbol());
                            }
                        }
                        if (fullQualifiedId == null) {
                            throw new CompilerException("Type " + typeName + " not found", node.getSymbol());
                        }
                        Type result = new Type.ClassType(null, fullQualifiedId, null);
                        
                        TypeUtils.checkTypes(result, env.getStage3Env().getThis(), node.getSymbol());
                            
                        
                        for (int i = 0; i < dimension; i++) {
                            result = new Type.ArrayType(result);
                        }
                        env.setResultType(result, false);
                    }
                }
                break;
                case INNER_CREATOR:
                    break;
                case EXPLICIT_GENERIC_INVOCATION:
                    //static generic call
                    List<String> identList = new ArrayList<>();
                    identList.addAll(node.getIdentifierList());
                    identList.add("§");
                    this.context = MethodInvokeCreator.createMethodAccess(stage3Env, stage3Env.getExpressionEnvironment(), node.getIdentifier(), identList, node);
                    break;
                case NON_WILDCARD_TYPE_ARGUMENTS:

                    break;
                case ARRAY_EXPRESSION: {
                    // exp1[exp2] = exp | exp = exp1[exp2] | exp2[exp]
                    ExpressionEnvironment env = stage3Env.getExpressionEnvironment();
                    String ident = node.getIdentifier();
                    VariableInstructionCreator varInstructionCreator = env.getVariableInstructionCreator();
                    if (ident != null) {
                        LocalEntry entry = stage3Env.getLocalTable().getEntry(ident);
                        if (entry != null) { //Is ident a local variable
                            varInstructionCreator.createLocalVariableInstruction(entry, ident, node.getIdentifierList(), node, ARRAY_EXPRESSION);
                        } else {
                            FieldInfo field = FieldTypeResolver.find(env.getFile(), ident);
                            if (field != null) {
                                varInstructionCreator.createFieldInstruction(ident, node.getIdentifierList(), true, node);
                            } else {
                                if (node.getIdentifierList() == null && node.getIdentifierList().isEmpty()) {
                                    throw new CompilerException("Local or field variable " + ident + " not found", node.getSymbol());
                                }
                                varInstructionCreator.createClassInstruction(ident, node.getIdentifierList(), node);
                            }
                        }

                    }
                }

                break;

                default:
                    throw new AssertionError(node.getSuffixType().name());

            }
        } catch (ByteCodeException | IOException | ResolveException ex) {
            ex.printStackTrace(System.err);
            System.exit(-1);
        }
    }

    @Override
    public void begin(BasicTypeNode node) throws CompilerException {
        stage3Env.writeSuperConstructorCall(node);
    }

    @Override
    public void begin(ClosureCreatorNode node) throws CompilerException {
        stage3Env.writeSuperConstructorCall(node);
    }

    @Override
    public void begin(ExpressionNode node) throws CompilerException {
        stage3Env.writeSuperConstructorCall(node);

    }

    @Override
    public void visit(LiteralNode node) throws CompilerException {
    }

    @Override
    public Visitor visit(TypeNode node) throws CompilerException {
        return typeVisitor = new Stage3TypeVisitor(stage3Env, true);
    }

    @Override
    public Visitor visit(ParExpressionNode node) throws CompilerException {
        return new Stage3ParExpressionVisitor(stage3Env);
    }

    @Override
    public Visitor visit(ArgumentsNode node) throws CompilerException {
        return new Stage3ArgumentsVisitor(stage3Env);
    }

    @Override
    public Visitor visit(SuperSuffixNode node) throws CompilerException {
        return new Stage3SuperSuffixVisitor(stage3Env, node.getIdentifier());
    }

    @Override
    public Visitor visit(CreatorNode node) throws CompilerException {
        return new Stage3CreatorVisitor(stage3Env);
    }

    @Override
    public Visitor visit(NonWildcardTypeArgumentsNode node) throws CompilerException {
        return new Stage3NonWildcardTypeArgumentsVisitor(stage3Env);
    }

    @Override
    public Visitor visit(ExplicitGenericInvocationSuffixNode node) throws CompilerException {
        return genericInvocationVisitor = new Stage3ExplicitGenericInvocationSuffixVisitor(stage3Env);
    }

    @Override
    public Visitor visit(IdentifierSuffixNode node) throws CompilerException {
        return identSuffixVisitor = new Stage3IdentifierSuffixVisitor(stage3Env, node.getDim());
    }

    @Override
    public void visit(BasicTypeNode node) throws CompilerException {
    }

    @Override
    public Visitor visit(ClosureCreatorNode node) throws CompilerException {
        return new Stage3ClosureCreatorVisitor(stage3Env);
    }

    @Override
    public Visitor visit(ExpressionNode node) throws CompilerException {
        //return null;
        return new Stage3ExpressionVisitor(stage3Env, stack, false);
    }

    @Override
    public void end(LiteralNode node) throws CompilerException {
    }

    @Override
    public void end(TypeNode node) throws CompilerException {
        /*
         * cast expression
         */
        castType = typeVisitor.getType();
        castArrayDim = node.getArrayDim();
        if (castArrayDim <= 0) {
            if (castType instanceof Type.SimpleType) {
                Type.SimpleType simpleType = (Type.SimpleType) castType;
                if (simpleType.isPrimitiveType()) {
                    throw new CompilerException("cannot cast to java primitive types", node.getSymbol());
                }
            }
        }
        for (int i = 0; i < castArrayDim; i++) {
            castType = new Type.ArrayType(castType);
        }
        stage3Env.getExpressionEnvironment().pushCastPrefix(castType);
    }

    @Override
    public void end(ParExpressionNode node) throws CompilerException {
        stage3Env.getExpressionEnvironment().writePrefix(node);
    }

    @Override
    public void end(ArgumentsNode node) throws CompilerException {
        try {
            LocalEntry entry = stage3Env.getLocalTable().getEntry(thisCallIdent);
            if (thisCallIdent.equals("@1")) {
                MethodState mstate = stage3Env.hasState(MethodState.class);
                entry.setTypeSwitchType(mstate.getClosureType());
            }
            MethodInvokeCreator.createMethodCall(thisCallContext, stage3Env.getExpressionEnvironment(), thisCallIdent, Collections.EMPTY_LIST, node);
            if (thisCallIdent.equals("@1")) {
                entry.resetTypeSwitchType();
            }
        } catch (ByteCodeException | IOException | ResolveException ex) {
            ex.printStackTrace(System.err);
            System.exit(-1);
        }

        //stage3Env.getExpressionEnvironment().printResultTypeList();
        if (!thisCallIdent.equals("@1")) {
            stage3Env.getExpressionEnvironment().cleanOperandStack();
        }
    }

    @Override
    public void end(SuperSuffixNode node) throws CompilerException {
    }

    @Override
    public void end(CreatorNode node) throws CompilerException {
    }

    @Override
    public void end(NonWildcardTypeArgumentsNode node) throws CompilerException {
    }

    @Override
    public void end(ExplicitGenericInvocationSuffixNode node) throws CompilerException {
    }

    @Override
    public void end(IdentifierSuffixNode node) throws CompilerException {
        try {
            ExpressionEnvironment env = stage3Env.getExpressionEnvironment();
            switch (node.getSuffixType()) {
                case SUFFIX_CALL:
                    break;
                case CALL:
                    //invoke private method
                    MethodInvokeCreator.createMethodCall(this.context, this.stage3Env.getExpressionEnvironment(), node.getIdentifier(), node.getIdentifierList(), node);
                    env.resetCallParameters();
                    break;
                case SUFFIX:
                case ARRAY_SUFFIX:
                    // exp.class
                    Type typeOfResult = env.typeOfResult();
                    if (node.getSuffixes() == IdentifierSuffixNode.Suffixes.CLASS) {
                        FieldDescriptor descr = stage3Env.getTypeUtils().makeFieldDescriptor(typeOfResult);
                        env.insertCode(new Ldc(descr.getClassInfo()));
                        if (typeOfResult instanceof Type.ArrayType) {
                            env.setResultType(new Type.ClassType(null, "java/lang/Class", Arrays.<Type>asList(new Type.WildcardType())));
                        } else {
                            env.setResultType(new Type.ClassType(null, "java/lang/Class", Arrays.asList(typeOfResult)));
                        }
                    }
                    break;
                case INNER_CREATOR:
                    break;
                case EXPLICIT_GENERIC_INVOCATION:
                    // invoke static generic method
                    List<String> identList = new ArrayList<>();
                    identList.addAll(node.getIdentifierList());
                    identList.add(identSuffixVisitor.getGenericMethodName());
                    context.setMethodName(identSuffixVisitor.getGenericMethodName());
                    MethodInvokeCreator.createMethodCall(this.context, this.stage3Env.getExpressionEnvironment(), node.getIdentifier(), identList, identSuffixVisitor.getGenericArguments(), node);
                    env.resetCallParameters();
                    break;
                case NON_WILDCARD_TYPE_ARGUMENTS:

                    break;
                case ARRAY_EXPRESSION: {
                }
                break;
                default:
                    throw new AssertionError(node.getSuffixType().name());

            }

        } catch (ByteCodeException | ResolveException | IOException ex) {
            ex.printStackTrace(System.err);
            System.exit(-1);
        }
    }

    @Override
    public void end(BasicTypeNode node) throws CompilerException {       
    }

    @Override
    public void end(ClosureCreatorNode node) throws CompilerException {
    }

    @Override
    public void end(ExpressionNode node) throws CompilerException {
        stage3Env.getExpressionEnvironment().writePrefix(node);
    }
}
