/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package stage3;

import abstractsyntaxtree.ExpressionNode;
import abstractsyntaxtree.InfixOpNode;
import codegeneration.Visitor;
import codegeneration.abstractvisitors.AbstractReturnStatementVisitor;
import codegeneration.environment.ExpressionEnvironment;
import codegeneration.environment.ShortCircuitEnvironment;
import codegeneration.environment.Stage3Environment;
import codegeneration.instruction.InstructionCreator;
import exceptions.CompilerException;
import java.util.Deque;
import java.util.LinkedList;
import stage3.states.BlockState;
import stage3.states.MethodState;
import stage3.states.VoidMethodState;
import teajay.bytecode.attributes.code.Label;
import teajay.bytecode.attributes.code.NOP;
import teajay.bytecode.attributes.code.Return;
import teajay.bytecode.attributes.code.ReturnType;
import teajay.bytecode.exceptions.ByteCodeException;
import teajay.generics.check.TypeException;
import teajay.generics.parser.Type;

/**
 *
 * @author Michael Burzan
 */
public class Stage3ReturnStatementVisitor extends AbstractReturnStatementVisitor {

    private final Stage3Environment stage3Env;
    private final Deque<InfixOpNode.InfixOps> stack;

    public Stage3ReturnStatementVisitor(Stage3Environment stage3Env) {
        this.stage3Env = stage3Env;
        this.stack = new LinkedList<>();
        this.stage3Env.createNewExpressionEnvironment();
        //this.stage3Env.setExpressionEnvironment(new ExpressionEnvironment(stage3Env));
    }

    @Override
    public void begin(ExpressionNode node) throws CompilerException {
        //check is retrun exp ; possible 
        VoidMethodState voidState = stage3Env.hasState(VoidMethodState.class);
        if (voidState != null) {
            throw new CompilerException("Cannot return a value from method whose return type is void", node.getSymbol());
        }
        MethodState cState = stage3Env.hasState(MethodState.class);
        if(cState != null && cState.isClosure() && cState.isVoidClosure()){
            throw new CompilerException("Cannot return a value from method whose return type is void", node.getSymbol());
        }



    }

    @Override
    public Visitor visit(ExpressionNode node) throws CompilerException {
        return new Stage3ExpressionVisitor(stage3Env, this.stack, true);
    }

    @Override
    public void end(ExpressionNode node) throws CompilerException {
        ExpressionEnvironment env = stage3Env.getExpressionEnvironment();
        stage3Env.restoreOldExpressionEnvironment();
        ExpressionEnvironment oldExp = stage3Env.getExpressionEnvironment();
        //write operators
        InstructionCreator.PutOperatorInstructions(env, stack, node);
        try {
            // Make short circuit environment
            ShortCircuitEnvironment shortCircuit = env.getShortCircuit();
            if (shortCircuit != null) {
                BlockState hasState = stage3Env.hasState(BlockState.class);
                Label label = new Label("return short circuit gotoLabel");
                shortCircuit.write(stage3Env, label, oldExp.getOperandStack().toArray(new Type[0]));
                hasState.addLabel(label);
                oldExp.setResultType(new Type.ClassType(null, "java/lang/Boolean", null));
                stage3Env.getExpressionEnvironment().insertCodeWithVerificationStack(new NOP(), oldExp.getOperandStack());
                
            }else{
                oldExp.setResultType(env.typeOfResult());
            }
            oldExp.setShortCircuit(null);
            stage3Env.getTypeUtils().checkAssignability(stage3Env.getCurrentMethodExpectedType(), oldExp.typeOfResult());
        } catch (TypeException ex) {
            //ex.printStackTrace(System.err);
            throw new CompilerException("Requiered:" + stage3Env.getCurrentMethodExpectedType() + " Found:" + env.typeOfResult(), node.getSymbol());
        }
    }
}
