/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package stage3;

import abstractsyntaxtree.BlockNode;
import abstractsyntaxtree.ParExpressionNode;
import codegeneration.Visitor;
import codegeneration.abstractvisitors.AbstractSynchronizedStatementVisitor;
import codegeneration.environment.Stage3Environment;
import codegeneration.javavisitor.expression.expressiontransform.ParExpressioVisitor;
import exceptions.CompilerException;
import java.util.ArrayList;
import java.util.List;
import stage3.states.BlockState;
import stage3.states.EndState;
import teajay.bytecode.attributes.code.ALoad;
import teajay.bytecode.attributes.code.AStore;
import teajay.bytecode.attributes.code.AThrow;
import teajay.bytecode.attributes.code.Dup;
import teajay.bytecode.attributes.code.Goto;
import teajay.bytecode.attributes.code.JVMInstruction;
import teajay.bytecode.attributes.code.Label;
import teajay.bytecode.attributes.code.MonitorEnter;
import teajay.bytecode.attributes.code.MonitorExit;
import teajay.bytecode.attributes.code.VerificationType;
import teajay.bytecode.cpool.ClassInfo;
import teajay.bytecode.exceptions.ByteCodeException;
import teajay.generics.parser.Type;

/**
 *
 * @author Michael Burzan
 */
public class Stage3SynchronizedStatementVisitor extends AbstractSynchronizedStatementVisitor {

    private final Stage3Environment stage3Env;
    private String syncName;
    private Label label = new Label("Synchronized jmpLabel");
    private Label startBlockLabel = new Label("SynchronizedBlockLabel");

    public Stage3SynchronizedStatementVisitor(Stage3Environment stage3Env) {
        this.stage3Env = stage3Env;
    }

    @Override
    public void begin(ParExpressionNode node) throws CompilerException {
        throw new CompilerException("synchronized statement not supported yet",node.getSymbol());
    }

    @Override
    public void begin(BlockNode node) throws CompilerException {
        BlockState block = new BlockState();
        stage3Env.changeState(block, node);
        stage3Env.getLocalTable().addFrame();
        stage3Env.getExpressionEnvironment().insertCode(new MonitorEnter());
        block.addLabel(startBlockLabel);

    }

    @Override
    public Visitor visit(ParExpressionNode node) throws CompilerException {
        return new Stage3ParExpressionVisitor(stage3Env);
    }

    @Override
    public Visitor visit(BlockNode node) throws CompilerException {
        return new Stage3BlockVisitor(stage3Env);
    }

    @Override
    public void end(ParExpressionNode node) throws CompilerException {
        syncName = stage3Env.getSynchronizedVarName();
        stage3Env.getLocalTable().addEntry(syncName, stage3Env.getExpressionEnvironment().typeOfResult());
        try {
            stage3Env.getExpressionEnvironment().insertCode(new Dup());
            stage3Env.getExpressionEnvironment().insertCode(new AStore(stage3Env.getLocalTable().getEntry(syncName).getIndex()));
        } catch (ByteCodeException ex) {
            ex.printStackTrace(System.err);
            System.exit(-1);
        }
    }

    @Override
    public void end(BlockNode node) throws CompilerException {
        EndState hasState = stage3Env.hasState(EndState.class);
        if(hasState != null){
            stage3Env.prevState();
        }
        try {
            int syncVarIndex = stage3Env.getLocalTable().getEntry(syncName).getIndex();
            Label endLabel = new Label("Sync end label");
            Label target = new Label("target label");
            stage3Env.getExpressionEnvironment().insertCode(new ALoad(syncVarIndex));
            stage3Env.getExpressionEnvironment().insertCode(new MonitorExit());
            setLabel(endLabel);
            stage3Env.getExpressionEnvironment().insertCode(new Goto(label));
            stage3Env.getMethodInfo().getCode().addExceptionHandler(startBlockLabel, endLabel, target, ClassInfo.fromClass(Throwable.class));
         
            
            /*WHAT A FUCKING HACK */
            List<Label> startAndTargetLabelList = new ArrayList<>();
            startAndTargetLabelList.add(target);
            startAndTargetLabelList.add(startBlockLabel = new Label("exception sync begin label"));
            stage3Env.getLocalTable().addStackMapFrame(this.stage3Env.getMethodInfo().
                    getCode().getInstructionList().
                    add(startAndTargetLabelList, new AStore(stage3Env.getLocalTable().getNextId())),
                    VerificationType.valueOf(ClassInfo.fromClass(Throwable.class)));
            //stage3Env.getExpressionEnvironment().insertCode(new AStore(stage3Env.getLocalTable().getNextId()));
            stage3Env.getExpressionEnvironment().insertCode(new ALoad(syncVarIndex));
            stage3Env.getExpressionEnvironment().insertCode(new MonitorExit());
            setLabel(endLabel = new Label("Sync expression end label"));
            stage3Env.getExpressionEnvironment().insertCode(new ALoad(stage3Env.getLocalTable().getNextId()));
            
            stage3Env.getMethodInfo().getCode().addExceptionHandler(startBlockLabel, endLabel, target, ClassInfo.fromClass(Throwable.class));
            stage3Env.getExpressionEnvironment().insertCode(new AThrow());
            stage3Env.getLocalTable().removeFrame();
            stage3Env.prevState();
        } catch (ByteCodeException ex) {
            ex.printStackTrace(System.err);
            System.exit(-1);
        }
    }
    
    
    private void setLabel(Label l){
        BlockState bState = stage3Env.hasState(BlockState.class);
        bState.addLabel(l);
    }
    
    public Label getLabel(){
        return this.label;
    }
}
