/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package stage3;

import abstractsyntaxtree.ClassBodyNode;
import abstractsyntaxtree.TypeNode;
import abstractsyntaxtree.TypeParametersNode;
import codegeneration.Visitor;
import codegeneration.abstractvisitors.AbstractTjImplDeclarationVisitor;
import codegeneration.environment.Stage3Environment;
import exceptions.CompilerException;
import java.io.IOException;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;
import teajay.bytecode.ClassFile;
import teajay.bytecode.attributes.signature.Signature;
import teajay.bytecode.cpool.FieldDescriptor;
import teajay.bytecode.exceptions.ByteCodeException;
import teajay.bytecode.fields.FieldAccessFlag;
import teajay.bytecode.fields.FieldInfo;
import teajay.generics.check.TypeUtils;
import teajay.generics.parser.Type;
import teajay.generics.writer.SignatureEncoder;
import teajay.runtime.exceptions.ResolveException;
import teajay.util.io.ClassFinder;

/**
 *
 * @author Michael Burzan
 */
class Stage3TjImplDeclarationVisitor extends AbstractTjImplDeclarationVisitor {

    private final Stage3Environment stage3Env;

    public Stage3TjImplDeclarationVisitor(Stage3Environment stage3Env) {
        this.stage3Env = stage3Env;
    }

    @Override
    public void begin(TypeNode node) throws CompilerException {
    }

    @Override
    public void begin(TypeParametersNode node) throws CompilerException {
        //make generic context and add generic parameter
        stage3Env.addGenericFrame();
        List<Type.TypeParamType> generic = node.getGeneric();
        for (Type.TypeParamType p : generic) {
            stage3Env.addTypeVariable(p.name);
        }
    }

    @Override
    public void begin(ClassBodyNode node) throws CompilerException {
    }

    @Override
    public Visitor visit(TypeNode node) throws CompilerException {
        return null;
    }

    @Override
    public Visitor visit(TypeParametersNode node) throws CompilerException {

        return null;
    }

    @Override
    public Visitor visit(ClassBodyNode node) throws CompilerException {
        return new Stage3ClassBodyVisitor(stage3Env);
    }

    @Override
    public void end(TypeNode node) throws CompilerException {
    }

    @Override
    public void end(TypeParametersNode node) throws CompilerException {
    }

    @Override
    public void end(ClassBodyNode node) throws CompilerException {
        // add super field to class fiel if not exist
        if (stage3Env.getClassFile().getField("super") == null) {
            try {
                ClassFile typeDef = ClassFinder.instance().find(stage3Env.getClassFile().getSuper().getClassName()).get();
                FieldInfo superField = new FieldInfo("super", FieldDescriptor.fromClassInfo(typeDef.getSuper()), FieldAccessFlag.SYNTHETIC, FieldAccessFlag.PRIVATE);
                Type.ClassSigType classSignature = TypeUtils.getClassSignature(typeDef.getThis().getClassName());
                superField.addAttribute(new Signature(SignatureEncoder.encode(classSignature.superclassType)));
                stage3Env.getClassFile().addField(superField);
            } catch (ByteCodeException | IOException | ResolveException ex) {
                ex.printStackTrace(System.err);
                System.exit(-1);
            }
        }
    }
}
