/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package stage3;

import abstractsyntaxtree.TypeNode;
import codegeneration.Visitor;
import codegeneration.abstractvisitors.AbstractTypeListVisitor;
import codegeneration.environment.Stage3Environment;
import exceptions.CompilerException;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import teajay.bytecode.exceptions.ByteCodeException;
import teajay.generics.parser.Type;
import teajay.runtime.exceptions.ResolveException;
import teajay.util.io.ClassFinder;

/**
 *
 * @author Michael Burzan
 */
class Stage3TypeListVisitor extends AbstractTypeListVisitor {

    private Stage3TypeVisitor stage3Type;
    private final Stage3Environment stage3Env;
    private List<Type> list = new ArrayList<>();
    private final boolean isNonWildcardTypeArguments;

    public Stage3TypeListVisitor(Stage3Environment stage3Env) {
        this.stage3Env = stage3Env;
        this.isNonWildcardTypeArguments = false;
    }

    public Stage3TypeListVisitor(Stage3Environment stage3Env, boolean isNonWildcardTypeArguments) {
        this.stage3Env = stage3Env;
        this.isNonWildcardTypeArguments = isNonWildcardTypeArguments;
    }

    public List<Type> getImplementsList() {
        return list;
    }

    @Override
    public void begin(TypeNode node) throws CompilerException {
    }

    @Override
    public Visitor visit(TypeNode node) throws CompilerException {
        return stage3Type = new Stage3TypeVisitor(stage3Env);
    }

    @Override
    public void end(TypeNode node) throws CompilerException {
        //check is type a reference type
        Type t = stage3Type.getType();
        try {
            if (t instanceof Type.ClassType) {
                if (!isNonWildcardTypeArguments) {
                    Type.ClassType classType = (Type.ClassType) t;

                    if (ClassFinder.instance().find(classType.name).get().isInterface()) {
                        list.add(t);
                    } else {
                        throw new CompilerException("Expecting interface ", node.getSymbol());
                    }
                } else {
                    list.add(t);
                }
            }else if(!(t instanceof Type.SimpleType) || !isNonWildcardTypeArguments){
                throw new CompilerException("Expecting interface ", node.getSymbol());  
            }
            else {
                list.add(t);
            }

        } catch (ByteCodeException | IOException | ResolveException ex) {
            ex.printStackTrace(System.err);
            System.exit(-1);
        }

    }

    @Override
    public Object getContainer() throws CompilerException {
        return null;
    }
}
