/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package stage3;

import abstractsyntaxtree.BlockNode;
import abstractsyntaxtree.TypeSwitchLabelNode;
import codegeneration.Visitor;
import codegeneration.abstractvisitors.AbstractTypeSwitchBlockVisitor;
import codegeneration.environment.Stage3Environment;
import datastructs.Pair;
import datastructs.PairOfTypeAndBlock;
import exceptions.CompilerException;
import java.io.IOException;
import java.util.Collection;
import java.util.Collections;
import java.util.Comparator;
import java.util.LinkedList;
import java.util.List;
import stage3.utils.InformationCenter;
import symbol.DerivationTree;
import teajay.bytecode.ClassFile;
import teajay.bytecode.exceptions.ByteCodeException;
import teajay.generics.parser.Type;
import teajay.runtime.exceptions.ResolveException;
import teajay.util.io.ClassFinder;
import static teajay.util.tjattribute.TeaJayAttribute.Type.Class;
import static teajay.util.tjattribute.TeaJayAttribute.Type.Closure;
import static teajay.util.tjattribute.TeaJayAttribute.Type.Enum;
import static teajay.util.tjattribute.TeaJayAttribute.Type.Hidden;
import static teajay.util.tjattribute.TeaJayAttribute.Type.Impl;
import static teajay.util.tjattribute.TeaJayAttribute.Type.Interface;
import static teajay.util.tjattribute.TeaJayAttribute.Type.TJBuiltinType;
import static teajay.util.tjattribute.TeaJayAttribute.Type.Typedef;

/**
 *
 * @author Michael Burzan | Kußmann
 */
class Stage3TypeSwitchBlockVisitor extends AbstractTypeSwitchBlockVisitor {

    private final Stage3Environment stage3Env;
    private final String origType;
    private List<Pair<Type.ClassType, BlockNode>> caseList = new LinkedList<>();
    private Stage3TypeSwitchLabelVisitor typeSwitchLabel;
    private static final CaseComparator comparator = new CaseComparator();

    public Stage3TypeSwitchBlockVisitor(Stage3Environment stage3Env, String origType) {
        this.stage3Env = stage3Env;
        this.origType = origType;
    }

    @Override
    public void begin(TypeSwitchLabelNode node) throws CompilerException {
    }

    @Override
    public void begin(BlockNode node) throws CompilerException {
    }

    @Override
    public Visitor visit(TypeSwitchLabelNode node) throws CompilerException {
        return typeSwitchLabel = new Stage3TypeSwitchLabelVisitor(stage3Env, origType);
    }

    @Override
    public Visitor visit(BlockNode node) throws CompilerException {
        return null;
    }

    @Override
    public void end(TypeSwitchLabelNode node) throws CompilerException {
    }

    @Override
    public void end(BlockNode node) throws CompilerException {
        // check case types of duplicates
        if (typeSwitchLabel != null) {
            Pair<Type.ClassType, BlockNode> pair = new PairOfTypeAndBlock(typeSwitchLabel.getType(), node);
            if (this.caseList.contains(pair)) {
                throw new CompilerException("Duplicate case label in typeswitch", node.getSymbol());
            }
            this.caseList.add(pair);
        } else {
            Pair<Type.ClassType, BlockNode> pair = new PairOfTypeAndBlock(null, node);
            if (this.caseList.contains(pair)) {
                throw new CompilerException("Duplicate case label in typeswitch", node.getSymbol());
            }
            this.caseList.add(pair);
        }
        typeSwitchLabel = null;
    }
    /**
     * @return Sorted Collection of type and blocks
     */
    public Collection<Pair<Type.ClassType, BlockNode>> getCaseList() {
        Collections.sort(this.caseList, comparator);
        return this.caseList;
    }

    static private class CaseComparator implements Comparator<Pair<Type.ClassType, BlockNode>> {

        @Override
        public int compare(Pair<Type.ClassType, BlockNode> o1, Pair<Type.ClassType, BlockNode> o2) {
            try {
                //Types
                if (o1.value1 == null) {
                    return 1;
                }
                if (o2.value1 == null) {
                    return -1;
                }
                if (o1.value1 == o2.value1) {
                    return 0;
                }
                if (isTypeDef(o1.value1)) {
                    if (!isTypeDef(o2.value1)) { // Typedef < Interface
                        return -1;
                    }
                    int depth1 = DerivationTree.instance().getDepth(ClassFinder.instance().find(o1.value1.name));
                    int depth2 = DerivationTree.instance().getDepth(ClassFinder.instance().find(o2.value1.name));
                    return Integer.compare(depth2, depth1);


                } else if (isTypeDef(o2.value1)) { // Interface > Typedef
                    return 1;
                } else { // Interface <-> Interface
                    int depth1 = DerivationTree.instance().getDepth(ClassFinder.instance().find(o1.value1.name));
                    int depth2 = DerivationTree.instance().getDepth(ClassFinder.instance().find(o2.value1.name));
                    return Integer.compare(depth2, depth1);
                }

            } catch (ByteCodeException | ResolveException | IOException ex) {
                ex.printStackTrace(System.err);
                System.exit(-1);

            }

            throw new Error("WIe bist du hier hin gekommen");

        }

        private boolean isTypeDef(Type.ClassType type) {
            try {
                ClassFile file = ClassFinder.instance().find(type.name).get();
                switch (InformationCenter.getTeaJayAttributeType(file)) {
                    case Closure:
                        throw new Error("no closures allowed in typeswitch");
                    case Impl:
                        throw new Error("no implementations allowed in typeswitch");
                    case Class:
                    case Typedef:
                        return true;
                    case Enum:
                        throw new Error("no enums allowed in typeswitch");
                    case Interface:
                        return false;
                    case TJBuiltinType:
                    case Hidden:
                        return true;
                    default:
                        throw new AssertionError(InformationCenter.getTeaJayAttributeType(file).name());
                }
            } catch (ByteCodeException | IOException | ResolveException ex) {
                ex.printStackTrace(System.err);
                System.exit(-1);
            }
            return false;
        }
    }
}
