/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package stage3;

import abstractsyntaxtree.BlockNode;
import abstractsyntaxtree.TypeSwitchBlocksNode;
import codegeneration.Visitor;
import codegeneration.abstractvisitors.AbstractTypeSwitchStatementVisitor;
import codegeneration.environment.ExpressionEnvironment;
import codegeneration.environment.Stage3Environment;
import datastructs.Pair;
import exceptions.CompilerException;
import java.util.Collection;
import java.util.Iterator;
import stage3.adaptedvisitor.AdaptedTypeSwitchBlockVisitor;
import stage3.states.BlockState;
import stage3.states.EndState;
import symbol.LocalEntry;
import teajay.bytecode.attributes.code.ALoad;
import teajay.bytecode.attributes.code.AStore;
import teajay.bytecode.attributes.code.CheckCast;
import teajay.bytecode.attributes.code.Goto;
import teajay.bytecode.attributes.code.IMode;
import teajay.bytecode.attributes.code.If0;
import teajay.bytecode.attributes.code.InstanceOf;
import teajay.bytecode.attributes.code.JVMInstructionList;
import teajay.bytecode.attributes.code.Label;
import teajay.bytecode.attributes.code.NOP;
import teajay.bytecode.cpool.ClassInfo;
import teajay.bytecode.exceptions.ByteCodeException;
import teajay.generics.parser.Type;

/**
 *
 * @author Michael Burzan 
 */
public class Stage3TypeSwitchStatementVisitor extends AbstractTypeSwitchStatementVisitor {

    private final Stage3Environment stage3Env;
    private Label jmpLabel = new Label("typeSwitchJmpLabel");
    private Stage3TypeSwitchBlockVisitor typeSwitchBlock;
    private final String identifier;
    private LocalEntry entry;
    private String origType;
    private boolean isUnreachableCode = false;

    public Stage3TypeSwitchStatementVisitor(Stage3Environment stage3Env, String identifier) {
        this.stage3Env = stage3Env;
        this.identifier = identifier;

    }

    @Override
    public void begin(TypeSwitchBlocksNode node) throws CompilerException {
        entry = stage3Env.getLocalTable().getEntry(this.identifier);
        if (entry == null) {
            throw new CompilerException("Local variable " + identifier + " not found.", node.getSymbol());
        }
        this.origType = entry.getQID();
    }

    @Override
    public Visitor visit(TypeSwitchBlocksNode node) throws CompilerException {
        return typeSwitchBlock = new Stage3TypeSwitchBlockVisitor(stage3Env, origType);
    }

    @Override
    /*
     * Transform:
     * 
     * typeswitch(a){
     *  case CaseType1 {}
     *  case CaseType2 {}
     *  default { }
     * }
     * 
     * TO:
     * 
     * if( ! identifier instance of CaseType1 ){
     *      goto NextCase: 
     * }
     * Case1Statements
     * goto end
     * NextCase: if( ! identifier instance of CaseType2){
     *      goto NextCase:
     * }
     * Case2Statements
     * goto end;
     * NextCase: default statements
     * ...
     * end: ...
     * 
     */
    public void end(TypeSwitchBlocksNode node) throws CompilerException {
        try {
            //counter for blocks that ended with EndState
            int cntNoJmpOver = 0;
            ExpressionEnvironment env = stage3Env.getExpressionEnvironment();
            //Get ordered cases
            Collection<Pair<Type.ClassType, BlockNode>> caseList = this.typeSwitchBlock.getCaseList();
            Iterator<Pair<Type.ClassType, BlockNode>> iterator = caseList.iterator();
            Label nextCase = new Label("nextCase");
            boolean hasDefaultCase = false;
            //check has list a cases
            if (iterator.hasNext()) {
                //get block of case type
                Pair<Type.ClassType, BlockNode> p = iterator.next();
                //change state to block state
                stage3Env.changeState(new BlockState(), node);
                //check is case type default case
                if (p.value1 != null) {
                    //load local variable from register
                    env.insertCode(new ALoad(entry.getIndex()));
                    ClassInfo newClass = new ClassInfo(p.value1.name);
                    //var instaneof caseType
                    env.insertCode(new InstanceOf(newClass));
                    Label target;
                    //check has list one more item
                    if (iterator.hasNext()) {
                        target = nextCase;
                    } else {
                        target = jmpLabel;
                    }
                    //if ((var instanceof ) == false) => goto nextCase
                    env.insertCode(new If0(IMode.eq, target));
                    // cast local variable to case type
                    env.insertCode(new ALoad(entry.getIndex()));
                    env.insertCode(new CheckCast(newClass));
                    // store result of cast in register
                    env.insertCode(new AStore(entry.getIndex()));
                    // add ? extends case type to local variable table
                    entry.setTypeSwitchType(new Type.WildcardType(Type.WildcardType.Kind.EXTENDS, p.value1));
                    //new frame for the block
                    stage3Env.getLocalTable().addFrame();
                    //generate code for case block
                    p.value2.accept(new AdaptedTypeSwitchBlockVisitor(stage3Env, new ClassInfo(entry.getPreviousQID()), entry.getIndex()));
                    //remove the block frame form local table
                    stage3Env.getLocalTable().removeFrame();
                    EndState endState = stage3Env.hasState(EndState.class);
                    if (endState != null) {
                        stage3Env.prevState();
                        cntNoJmpOver++;
                    } else {
                        //Satisfy the verifier
                        newClass = new ClassInfo(entry.getPreviousQID());
                        env.insertCode(new ALoad(entry.getIndex()));
                        env.insertCode(new CheckCast(newClass));
                        env.insertCode(new AStore(entry.getIndex()));
                        //

                        stage3Env.getExpressionEnvironment().insertCode(new Goto(jmpLabel));
                    }
                    entry.resetTypeSwitchType();
                    stage3Env.prevState();
                } else { // DEFAULT CASE
                    hasDefaultCase = true;
                    JVMInstructionList ins = env.getMethod().getCode().getInstructionList();
                    stage3Env.getLocalTable().addStackMapFrame(ins.add(nextCase, new NOP()));
                    nextCase = new Label("nextCase");
                    stage3Env.getLocalTable().addFrame();
                    p.value2.accept(new Stage3BlockVisitor(stage3Env));
                    stage3Env.getLocalTable().removeFrame();

                    EndState endState = stage3Env.hasState(EndState.class);
                    if (endState != null) {
                        stage3Env.prevState();
                        cntNoJmpOver++;
                    }
                    stage3Env.prevState();
                }
            }

            while (iterator.hasNext()) {
                Pair<Type.ClassType, BlockNode> p = iterator.next();
                stage3Env.changeState(new BlockState(), node);
                if (p.value1 != null) {
                    JVMInstructionList ins = env.getMethod().getCode().getInstructionList();
                    stage3Env.getLocalTable().addStackMapFrame(ins.add(nextCase, new ALoad(entry.getIndex())));
                    nextCase = new Label("nextCase");
                    Label target;
                    if (iterator.hasNext()) {
                        target = nextCase;
                    } else {
                        target = jmpLabel;
                    }
                    ClassInfo newClass = new ClassInfo(p.value1.name);
                    env.insertCode(new InstanceOf(newClass));
                    env.insertCode(new If0(IMode.eq, target));
                    env.insertCode(new ALoad(entry.getIndex()));
                    env.insertCode(new CheckCast(newClass));
                    env.insertCode(new AStore(entry.getIndex()));
                    entry.setTypeSwitchType(new Type.WildcardType(Type.WildcardType.Kind.EXTENDS, p.value1));
                    stage3Env.getLocalTable().addFrame();
                    p.value2.accept(new AdaptedTypeSwitchBlockVisitor(stage3Env, new ClassInfo(entry.getPreviousQID()), entry.getIndex()));
                    stage3Env.getLocalTable().removeFrame();
                    EndState endState = stage3Env.hasState(EndState.class);
                    if (endState != null) {
                        stage3Env.prevState();
                        cntNoJmpOver++;
                    } else {
                        //Satisfy the verifier
                        newClass = new ClassInfo(entry.getPreviousQID());
                        env.insertCode(new ALoad(entry.getIndex()));
                        env.insertCode(new CheckCast(newClass));
                        env.insertCode(new AStore(entry.getIndex()));
                        //

                        stage3Env.getExpressionEnvironment().insertCode(new Goto(jmpLabel));
                    }
                    entry.resetTypeSwitchType();
                    stage3Env.prevState();
                } else { // DEFAULT CASE
                    hasDefaultCase = true;
                    JVMInstructionList ins = env.getMethod().getCode().getInstructionList();
                    stage3Env.getLocalTable().addStackMapFrame(ins.add(nextCase, new NOP()));
                    nextCase = new Label("nextCase");
                    stage3Env.getLocalTable().addFrame();
                    p.value2.accept(new Stage3BlockVisitor(stage3Env));
                    stage3Env.getLocalTable().removeFrame();

                    EndState endState = stage3Env.hasState(EndState.class);
                    if (endState != null) {
                        stage3Env.prevState();
                        cntNoJmpOver++;
                    }
                    stage3Env.prevState();
                }

            }


            if (caseList.size() == cntNoJmpOver && hasDefaultCase) {
                stage3Env.prevState();
                isUnreachableCode = true;
            }
        } catch (ByteCodeException ex) {
            ex.printStackTrace(System.err);
            System.exit(-1);
        }
    }

    public Label getLabel() {
        return this.jmpLabel;
    }

    public boolean isNextStatementUnreachable() {
        return this.isUnreachableCode;
    }
}
