/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package stage3.adaptedvisitor;

import abstractsyntaxtree.BreakStatementNode;
import abstractsyntaxtree.ContinueStatementNode;
import abstractsyntaxtree.ReturnStatementNode;
import abstractsyntaxtree.ThrowStatementNode;
import codegeneration.environment.Stage3Environment;
import exceptions.CompilerException;
import stage3.Stage3StatementVisitor;
import teajay.bytecode.attributes.code.ALoad;
import teajay.bytecode.attributes.code.AStore;
import teajay.bytecode.attributes.code.CheckCast;
import teajay.bytecode.cpool.ClassInfo;
import teajay.bytecode.exceptions.ByteCodeException;

/**
 *
 * @author Michael Burzan
 */
public class AdaptedTypeSwitchStatementVisitor extends Stage3StatementVisitor {

    final Stage3Environment stage3Env;
    private final ClassInfo oldInfo;
    private final int indexId;

    public AdaptedTypeSwitchStatementVisitor(Stage3Environment stage3Env, ClassInfo classInfo, int index) {
        super(stage3Env);
        this.stage3Env = stage3Env;
        this.oldInfo = classInfo;
        this.indexId = index;

    }

    @Override
    public void end(ReturnStatementNode node) throws CompilerException {
        try {
            stage3Env.getExpressionEnvironment().insertCode(new ALoad(indexId));
            stage3Env.getExpressionEnvironment().insertCode(new CheckCast(oldInfo));
            stage3Env.getExpressionEnvironment().insertCode(new AStore(indexId));
            super.end(node);
        } catch (ByteCodeException ex) {
            ex.printStackTrace(System.err);
            System.exit(-1);
        }
    }

    @Override
    public void end(ThrowStatementNode node) throws CompilerException {
        try {
            stage3Env.getExpressionEnvironment().insertCode(new ALoad(indexId));
            stage3Env.getExpressionEnvironment().insertCode(new CheckCast(oldInfo));
            stage3Env.getExpressionEnvironment().insertCode(new AStore(indexId));
            super.end(node);
        } catch (ByteCodeException ex) {
            ex.printStackTrace(System.err);
            System.exit(-1);
        }
    }
    @Override
    public void begin(BreakStatementNode node) throws CompilerException {
        try {
            stage3Env.getExpressionEnvironment().insertCode(new ALoad(indexId));
            stage3Env.getExpressionEnvironment().insertCode(new CheckCast(oldInfo));
            stage3Env.getExpressionEnvironment().insertCode(new AStore(indexId));
            super.begin(node);
        } catch (ByteCodeException ex) {
            ex.printStackTrace(System.err);
            System.exit(-1);
        }
    }
    
    @Override
    public void begin(ContinueStatementNode node) throws CompilerException {
        try {
            stage3Env.getExpressionEnvironment().insertCode(new ALoad(indexId));
            stage3Env.getExpressionEnvironment().insertCode(new CheckCast(oldInfo));
            stage3Env.getExpressionEnvironment().insertCode(new AStore(indexId));
            super.begin(node);
        } catch (ByteCodeException ex) {
            ex.printStackTrace(System.err);
            System.exit(-1);
        }
    }
}
