/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package stage3.bridgeimplementation;

import abstractsyntaxtree.AssertStatementNode;
import abstractsyntaxtree.BlockNode;
import abstractsyntaxtree.BreakStatementNode;
import abstractsyntaxtree.ContinueStatementNode;
import abstractsyntaxtree.DoStatementNode;
import abstractsyntaxtree.ForStatementNode;
import abstractsyntaxtree.IdentDoublePointStatementNode;
import abstractsyntaxtree.IfStatementNode;
import abstractsyntaxtree.ReturnStatementNode;
import abstractsyntaxtree.StatementExpressionNode;
import abstractsyntaxtree.SwitchStatementNode;
import abstractsyntaxtree.SynchronizedStatementNode;
import abstractsyntaxtree.ThrowStatementNode;
import abstractsyntaxtree.TryStatementNode;
import abstractsyntaxtree.TypeSwitchStatementNode;
import abstractsyntaxtree.WhileStatementNode;
import codegeneration.Visitor;
import codegeneration.environment.Stage3Environment;
import exceptions.CompilerException;
import java.util.List;
import stage3.Stage3AssertStatementVisitor;
import stage3.Stage3BlockVisitor;
import stage3.Stage3DoStatementVisitor;
import stage3.Stage3ForStatementVisitor;
import stage3.Stage3IdentDoublePointStatementVisitor;
import stage3.Stage3IfStatementVisitor;
import stage3.Stage3ReturnStatementVisitor;
import stage3.Stage3StatementExpressionVistor;
import stage3.Stage3SwitchStatementVisitor;
import stage3.Stage3SynchronizedStatementVisitor;
import stage3.Stage3ThrowStatementVisitor;
import stage3.Stage3TryStatementVisitor;
import stage3.Stage3TypeSwitchStatementVisitor;
import stage3.Stage3WhileStatementVisitor;
import stage3.states.BlockState;
import stage3.states.BreakState;
import stage3.states.ContinueState;
import stage3.states.DoState;
import stage3.states.EndState;
import stage3.states.ForState;
import stage3.states.IfState;
import stage3.states.LoopState;
import stage3.states.MethodState;
import stage3.states.ReturnState;
import stage3.states.StatementExpressionState;
import stage3.states.SynchronizedState;
import stage3.states.ThrowState;
import stage3.states.TryState;
import stage3.states.TypeSwitchState;
import stage3.states.UnreachableCodeState;
import stage3.states.VoidMethodState;
import stage3.states.WhileState;
import teajay.bytecode.attributes.code.AConstNull;
import teajay.bytecode.attributes.code.AThrow;
import teajay.bytecode.attributes.code.Goto;
import teajay.bytecode.attributes.code.Label;
import teajay.bytecode.attributes.code.NOP;
import teajay.bytecode.attributes.code.Return;
import teajay.bytecode.attributes.code.ReturnType;
import teajay.bytecode.exceptions.ByteCodeException;

/**
 *
 * @author Michael Burzan
 */
public class BaseStatementVisitorImpl implements StatementVisitorImpl {

    private final Stage3Environment stage3Env;
    private Stage3IfStatementVisitor ifStatementVistor;
    private Stage3WhileStatementVisitor whileStatementVisitor;
    private Stage3DoStatementVisitor doStatement;
    private Stage3ForStatementVisitor forStatement;
    private Stage3AssertStatementVisitor assertStatementVisitor;
    private Stage3TryStatementVisitor tryVisitor;
    private Stage3TypeSwitchStatementVisitor typeSwitchStatement;
    private Stage3SynchronizedStatementVisitor synchronizedVisitor;

    public BaseStatementVisitorImpl(Stage3Environment stage3Env) {
        this.stage3Env = stage3Env;

    }

    @Override
    public void beginImpl(BlockNode node) throws CompilerException {
        stage3Env.writeSuperConstructorCall(node);
        BlockState oldBlock = stage3Env.hasState(BlockState.class);
        if (oldBlock == null) {
            throw new Error("Fatal error: Missing surrounding block before block state");
        }
        BlockState newState = new BlockState();
        newState.addLabels(oldBlock.getLabels());
        oldBlock.resetLabelList();

        stage3Env.changeState(newState, node);
        stage3Env.getLocalTable().addFrame();
    }

    @Override
    public void beginImpl(IdentDoublePointStatementNode node) throws CompilerException {
        //Not supported
        stage3Env.writeSuperConstructorCall(node);
    }

    @Override
    public void beginImpl(StatementExpressionNode node) throws CompilerException {
        stage3Env.changeState(new StatementExpressionState(), node);
    }

    @Override
    public void beginImpl(IfStatementNode node) throws CompilerException {
        stage3Env.writeSuperConstructorCall(node);
        stage3Env.changeState(new IfState(), node);
    }

    @Override
    public void beginImpl(AssertStatementNode node) throws CompilerException {
        stage3Env.writeSuperConstructorCall(node);
    }

    @Override
    public void beginImpl(SwitchStatementNode node) throws CompilerException {
        stage3Env.writeSuperConstructorCall(node);
    }

    @Override
    public void beginImpl(WhileStatementNode node) throws CompilerException {
        stage3Env.writeSuperConstructorCall(node);
        stage3Env.changeState(new WhileState(), node);
    }

    @Override
    public void beginImpl(DoStatementNode node) throws CompilerException {
        stage3Env.writeSuperConstructorCall(node);
        stage3Env.changeState(new DoState(), node);
    }

    @Override
    public void beginImpl(ForStatementNode node) throws CompilerException {
        stage3Env.writeSuperConstructorCall(node);
        stage3Env.changeState(new ForState(), node);
    }

    @Override
    public void beginImpl(BreakStatementNode node) throws CompilerException {
        stage3Env.changeState(new BreakState(), node);
    }

    @Override
    public void beginImpl(ContinueStatementNode node) throws CompilerException {
        stage3Env.changeState(new ContinueState(), node);
    }

    @Override
    public void beginImpl(ReturnStatementNode node) throws CompilerException {
        stage3Env.writeSuperConstructorCall(node);
        stage3Env.changeState(new ReturnState(), node);
    }

    @Override
    public void beginImpl(ThrowStatementNode node) throws CompilerException {
        stage3Env.writeSuperConstructorCall(node);
        stage3Env.changeState(new ThrowState(), node);
    }

    @Override
    public void beginImpl(SynchronizedStatementNode node) throws CompilerException {
        throw new CompilerException("synchronized not supported yet", node.getSymbol());
//        stage3Env.writeSuperConstructorCall(node);
//        stage3Env.changeState(new SynchronizedState(), node);
    }

    @Override
    public void beginImpl(TryStatementNode node) throws CompilerException {
        stage3Env.writeSuperConstructorCall(node);
        stage3Env.changeState(new TryState(), node);
    }

    @Override
    public void beginImpl(TypeSwitchStatementNode node) throws CompilerException {
        stage3Env.writeSuperConstructorCall(node);
        stage3Env.getExpressionEnvironment().insertCode(new NOP()); //flush labels
        stage3Env.changeState(new TypeSwitchState(), node);
    }

    @Override
    public Visitor visitImpl(BlockNode node) throws CompilerException {
        return new Stage3BlockVisitor(stage3Env);
    }

    @Override
    public Visitor visitImpl(IdentDoublePointStatementNode node) throws CompilerException {
        return new Stage3IdentDoublePointStatementVisitor(stage3Env);
    }

    @Override
    public Visitor visitImpl(StatementExpressionNode node) throws CompilerException {
        return new Stage3StatementExpressionVistor(stage3Env);
    }

    @Override
    public Visitor visitImpl(IfStatementNode node) throws CompilerException {
        return ifStatementVistor = new Stage3IfStatementVisitor(stage3Env);
    }

    @Override
    public Visitor visitImpl(AssertStatementNode node) throws CompilerException {
        return assertStatementVisitor = new Stage3AssertStatementVisitor(stage3Env, node.hasColon());
    }

    @Override
    public Visitor visitImpl(SwitchStatementNode node) throws CompilerException {
        return new Stage3SwitchStatementVisitor(stage3Env);
    }

    @Override
    public Visitor visitImpl(WhileStatementNode node) throws CompilerException {
        return this.whileStatementVisitor = new Stage3WhileStatementVisitor(stage3Env);
    }

    @Override
    public Visitor visitImpl(DoStatementNode node) throws CompilerException {
        return doStatement = new Stage3DoStatementVisitor(stage3Env);
    }

    @Override
    public Visitor visitImpl(ForStatementNode node) throws CompilerException {
        return forStatement = new Stage3ForStatementVisitor(stage3Env);
    }

    @Override
    public void visitImpl(BreakStatementNode node) throws CompilerException {
        LoopState loopState = stage3Env.hasState(LoopState.class);
        if (loopState != null) {
            if (loopState.getBreakLabel() == null) {
                throw new Error("loop state without break label");
            }
            try {
                stage3Env.getMethodInfo().getCode().getInstructionList().add(new Goto(loopState.getBreakLabel()));

            } catch (ByteCodeException ex) {
                ex.printStackTrace(System.err);
                System.exit(-1);
            }
        }
    }

    @Override
    public void visitImpl(ContinueStatementNode node) throws CompilerException {
        LoopState loopState = stage3Env.hasState(LoopState.class);
        if (loopState != null) {
            if (loopState.getLoopLabel() == null) {
                throw new Error("loop state without loop label");
            }
            try {
                stage3Env.getMethodInfo().getCode().getInstructionList().add(new Goto(loopState.getLoopLabel()));
            } catch (ByteCodeException ex) {
                ex.printStackTrace(System.err);
                System.exit(-1);
            }
        }
    }

    @Override
    public Visitor visitImpl(ReturnStatementNode node) throws CompilerException {

        return new Stage3ReturnStatementVisitor(stage3Env);
    }

    @Override
    public Visitor visitImpl(ThrowStatementNode node) throws CompilerException {
        return new Stage3ThrowStatementVisitor(stage3Env);
    }

    @Override
    public Visitor visitImpl(SynchronizedStatementNode node) throws CompilerException {
        return synchronizedVisitor = new Stage3SynchronizedStatementVisitor(stage3Env);
    }

    @Override
    public Visitor visitImpl(TryStatementNode node) throws CompilerException {
        return tryVisitor = new Stage3TryStatementVisitor(stage3Env);
    }

    @Override
    public Visitor visitImpl(TypeSwitchStatementNode node) throws CompilerException {
        return typeSwitchStatement = new Stage3TypeSwitchStatementVisitor(stage3Env, node.getIdentifier());
    }

    @Override
    public void endImpl(BlockNode node) throws CompilerException {
        BlockState state = stage3Env.hasState(BlockState.class);

        EndState endState = stage3Env.hasState(EndState.class);
        stage3Env.prevState();
        if (endState != null) {
            stage3Env.prevState();
            stage3Env.changeState(new UnreachableCodeState(), node);
        } else {
            List<Label> labels = state.getLabels();
            if (!labels.isEmpty()) {
                state = stage3Env.hasState(BlockState.class);
                state.addLabels(labels);
            }
        }
        stage3Env.getLocalTable().removeFrame();
    }

    @Override
    public void endImpl(IdentDoublePointStatementNode node) throws CompilerException {
    }

    @Override
    public void endImpl(StatementExpressionNode node) throws CompilerException {
        stage3Env.prevState();
    }

    @Override
    public void endImpl(IfStatementNode node) throws CompilerException {
        stage3Env.prevState();
        List<Label> labels = ifStatementVistor.getLabel();

        if (labels != null) {
            BlockState block = stage3Env.hasState(BlockState.class);
            block.addLabels(labels);
        }
        if (ifStatementVistor.isNextStatementUnreachable()) {
            stage3Env.changeState(new UnreachableCodeState(), node);
        } else {
            stage3Env.getExpressionEnvironment().insertCode(new NOP());
        }

    }

    @Override
    public void endImpl(AssertStatementNode node) throws CompilerException {
        Label label = assertStatementVisitor.getLabel();
        if (label != null) {
            BlockState block = stage3Env.hasState(BlockState.class);
            block.addLabel(label);
        }
    }

    @Override
    public void endImpl(SwitchStatementNode node) throws CompilerException {
    }

    @Override
    public void endImpl(WhileStatementNode node) throws CompilerException {
        stage3Env.prevState();
        Label label = whileStatementVisitor.getLabel();
        if (label != null) {
            BlockState block = stage3Env.hasState(BlockState.class);
            block.addLabel(label);
        }
        stage3Env.getExpressionEnvironment().insertCode(new NOP());
    }

    @Override
    public void endImpl(DoStatementNode node) throws CompilerException {
        stage3Env.prevState();
    }

    @Override
    public void endImpl(ForStatementNode node) throws CompilerException {
        stage3Env.prevState();
        Label label = forStatement.getLabel();
        if (label != null) {
            BlockState block = stage3Env.hasState(BlockState.class);
            block.addLabel(label);
        }
        stage3Env.getExpressionEnvironment().insertCode(new NOP());
        stage3Env.getLocalTable().removeFrame();
    }

    @Override
    public void endImpl(BreakStatementNode node) throws CompilerException {
    }

    @Override
    public void endImpl(ContinueStatementNode node) throws CompilerException {
    }

    @Override
    public void endImpl(ReturnStatementNode node) throws CompilerException {
        MethodState methodState = stage3Env.hasState(MethodState.class);
        if (methodState == null) {
            throw new CompilerException("return outside method?!", node.getSymbol());
        }
        //check method is a void method
        if (node.isVoidReturn()) {
            if (methodState.isClosure() && methodState.isVoidClosure()) {
                stage3Env.getExpressionEnvironment().insertCode(new AConstNull());
                stage3Env.getExpressionEnvironment().insertCode(new Return(ReturnType.Reference));
            } else {
                VoidMethodState voidState = stage3Env.hasState(VoidMethodState.class); // check is method void for return statement
                if (voidState == null) {
                    throw new CompilerException("Missing return value", node.getSymbol());
                }
                stage3Env.getExpressionEnvironment().insertCode(new Return());
            }
        } else {
            if (methodState.isClosure() && methodState.isVoidClosure()) {
                throw new CompilerException("Cannot return a value from a closure whose return type is void.", node.getSymbol());
            }
            stage3Env.getExpressionEnvironment().insertCode(new Return(ReturnType.Reference));
        }
    }

    @Override
    public void endImpl(ThrowStatementNode node) throws CompilerException {
        stage3Env.getExpressionEnvironment().insertCode(new AThrow());
    }

    @Override
    public void endImpl(SynchronizedStatementNode node) throws CompilerException {
        stage3Env.prevState();
        BlockState block = stage3Env.hasState(BlockState.class);
        Label label = synchronizedVisitor.getLabel();
        if (label != null) {
            block.addLabel(label);
        }
    }

    @Override
    public void endImpl(TryStatementNode node) throws CompilerException {
        TryState tryState = stage3Env.hasState(TryState.class);
        if (tryState != null) {
//            stage3Env.getMethodInfo().getCode().addExceptionHandler(tryState.getStartPC(),
//                    tryState.getEndPC(),
//                    tryState.getHandleLabel(),
//                    tryState.getClassInfo());
        } else {
            throw new Error("Fatal error: Try state was reached but state is not set");
        }
        stage3Env.prevState();
        if (tryVisitor.hasUnreachableCodeReached()) {
            stage3Env.changeState(new UnreachableCodeState(), node);
        } else {
            BlockState blockState = stage3Env.hasState(BlockState.class);
            Label jmpOver = tryVisitor.getLabel();
            if (jmpOver != null) {
                blockState.addLabel(jmpOver);
            }
            //Bind jmpOver to NOP instruction
            stage3Env.getExpressionEnvironment().insertCode(new NOP());
        }

    }

    @Override
    public void endImpl(TypeSwitchStatementNode node) throws CompilerException {
        if (typeSwitchStatement.isNextStatementUnreachable()) {
            stage3Env.changeState(new UnreachableCodeState(), node);
        } else {
            BlockState block = stage3Env.hasState(BlockState.class);
            Label l = typeSwitchStatement.getLabel();
            if (l != null) {
                block.addLabel(l);
            }
            stage3Env.getExpressionEnvironment().insertCode(new NOP());
            stage3Env.prevState();
            
        }
    }
}
