/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package stage3.utils;

import stage3.Stage3ExpressionVisitor;
import abstractsyntaxtree.ExpressionNode;
import abstractsyntaxtree.InfixOpNode;
import codegeneration.environment.Stage3Environment;
import datastructs.Pair;
import exceptions.CompilerException;
import java.util.ArrayList;
import java.util.Deque;
import java.util.LinkedList;
import java.util.List;
import teajay.bytecode.ClassFile;
import teajay.bytecode.attributes.code.JVMInstructionList;
import teajay.bytecode.attributes.code.PutField;
import teajay.bytecode.attributes.code.PutStatic;
import teajay.bytecode.attributes.code.Return;
import teajay.bytecode.cpool.FieldDescriptor;
import teajay.bytecode.cpool.FieldRefInfo;
import teajay.bytecode.cpool.MethodDescriptor;
import teajay.bytecode.exceptions.ByteCodeException;
import teajay.bytecode.fields.FieldInfo;
import teajay.bytecode.methods.MethodAccessFlag;
import teajay.bytecode.methods.MethodInfo;
import teajay.generics.parser.Type;

/**
 *
 * @author Michael Burzan
 */
public class StaticConstructorFieldInitBuilder extends FieldInitBuilder {

    private final List<Pair<Type, Pair<String, ExpressionNode>>> variables = new ArrayList<>();

    public StaticConstructorFieldInitBuilder() {
    }
    /**
     * stores the static field declaration
     * @param type of field
     * @param name of field
     * @param node expression 
     */
    @Override
    public void addVariable(Type type, String name, ExpressionNode node) {
        variables.add(new Pair<>(type, new Pair<>(name, node)));
    }
    /**
     * writes stored fields in the static constructor instruction list
     * @param stage3Env the current environment
     * @throws CompilerException 
     */
    @Override
    public void writeToInstructionList(Stage3Environment env) throws CompilerException {
        ClassFile file = env.getClassFile();
        try {
            //exist static constructor. If not exist create the static constructor
            if (!variables.isEmpty()) {
                List<MethodInfo> staticBlocks = file.getMethods("<clinit>");
                MethodInfo staticBlock;
                if (staticBlocks == null) {
                    MethodDescriptor staticBlockDesc = new MethodDescriptor(FieldDescriptor.getVoid());
                    staticBlock = new MethodInfo("<clinit>", staticBlockDesc, file.getThis(), MethodAccessFlag.STATIC);
                    file.addMethod(staticBlock);
                } else {
                    staticBlock = staticBlocks.get(0);
                }
                //Iterate over all stored fields
                JVMInstructionList instructionList = staticBlock.getCode().getInstructionList();
                for (Pair<Type, Pair<String, ExpressionNode>> p : variables) {
                    //visit expression
                    p.value2.value2.accept(new Stage3ExpressionVisitor(env, new LinkedList<InfixOpNode.InfixOps>(), true));
                    FieldInfo field = file.getField(p.value2.value1);
                    FieldRefInfo fieldRef = FieldRefInfo.make(file.getThis(), p.value2.value1, field.getDescriptor());

                    if (!field.isStatic()) {
                        throw new Error("Fatal error: field want be init static is not declared static");
                    }
                    //store value to field
                    instructionList.add(new PutStatic(fieldRef));
                }

            }
        } catch (ByteCodeException ex) {
            ex.printStackTrace(System.err);
            System.exit(-1);
        }
    }
}
