/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package symbol;

import exceptions.CompilerException;
import teajay.runtime.exceptions.ResolveException;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import teajay.bytecode.exceptions.ByteCodeException;
import teajay.util.QIDTools;
import teajay.util.io.ClassFileRef;
import teajay.util.io.ClassFinder;


/**
 *
 * @author Michael Burzan
 */
public class NameResolver {

    private final static String[] javaStdLibDefaultImports = {
        "String", "Boolean", "System", "Object", "Throwable",
        "Exception", "NullPointerException", "IndexOutOfBoundsException",
        "IllegalArgumentException", "ArithmeticException", "Error", "IllegalStateException",
        "RuntimeException", "AssertionError", "Class", "Enum",
        "NumberFormatException", "OutOfMemoryError", "StackOverflowError",
        "Runtime", "Thread", "Runnable", "CharSequence"
    };
    private HashMap<String, String> exisitingNames = new HashMap<>();
    private HashMap<String, String> staticImports = new HashMap<>();

    public NameResolver() {
        ClassFinder finder = ClassFinder.instance();

        for (String s : javaStdLibDefaultImports) {
            exisitingNames.put(s, "java/lang/" + s);
        }
        Map<String, ClassFileRef> res = finder.findAll("teajay/lang");
        for (String s : res.keySet()) {
            exisitingNames.put(s, "teajay/lang/" + s);
        }
    }

    public void addNameFromPackage(String qId) throws CompilerException, ByteCodeException, IOException, ResolveException {
        ClassFileRef ref = ClassFinder.instance().find(qId);
        if (ref == null) {
            throw new CompilerException("Import " + qId + " does not exist");
        }
        this.exisitingNames.put(ref.get().getThis().getSimpleName(), qId);

    }
    
    public void addNamesFromImportNoCheck(String qualifiedImportName) {
        this.exisitingNames.put(QIDTools.simpleName(qualifiedImportName), qualifiedImportName);
    }

    public void addNamesFromImport(String qualifiedImportName) throws CompilerException, ByteCodeException, IOException, ResolveException {
        ClassFinder finder = ClassFinder.instance();
        ClassFileRef findClass = finder.find(qualifiedImportName);
        if (findClass == null) {
            throw new CompilerException("Import " + qualifiedImportName + " does not exist");
        }

        this.exisitingNames.put(findClass.get().getThis().getSimpleName(), qualifiedImportName);
    }

    public void addNamesFromStaticImport(String qualifiedImportName) throws ByteCodeException, IOException, CompilerException, ResolveException {
        ClassFinder finder = ClassFinder.instance();
        ClassFileRef ref = finder.find(qualifiedImportName);
        if (ref == null) {
            throw new CompilerException("Import " + qualifiedImportName + " does not exist");
        }
        this.staticImports.put(ref.get().getThis().getSimpleName(), qualifiedImportName);
        //Collection<FieldInfo> fields = ref.get().getFields();

    }

    public void addNamesFromImportWildcard(String importNameWildcard) throws CompilerException, ByteCodeException, IOException, ResolveException {
        ClassFinder finder = ClassFinder.instance();
        Map<String, ClassFileRef> findAll = finder.findAll(importNameWildcard);

        if (findAll == null) {
            throw new CompilerException("Import " + importNameWildcard + " does not exist");
        }
        for (String s : findAll.keySet()) {
            this.exisitingNames.put(s, importNameWildcard + "/" + s);
        }
    }

    public void addNamesFromStaticImportWildcard(String importNameWildcard) throws CompilerException, ByteCodeException, IOException, ResolveException {
        ClassFinder finder = ClassFinder.instance();
        Map<String, ClassFileRef> findAll = finder.findAll(importNameWildcard);

        if (findAll == null) {
            throw new CompilerException("Import " + importNameWildcard + " does not exist");
        }
        for (String s : findAll.keySet()) {
            this.staticImports.put(s, importNameWildcard + "/" + s);
        }
    }

    public String getQualifiedNameBySimpleName(String simpleName) {
        return exisitingNames.get(simpleName);
    }

    public boolean isQualifiedName(String qID) {
        /*for (String s : exisitingNames.values()) {
            if (s.equals(qID)) {
                return true;
            }
        }*/
        return ClassFinder.instance().find(qID) != null;
        
    }

    public List<ClassFileRef> getStaticImports() {
        Collection<String> values = staticImports.values();
        List<ClassFileRef> list = new ArrayList<>();
        for (String v : values) {
            list.add(ClassFinder.instance().find(v));
        }
        return list;
    }
    
    @Override
    public String toString() {
        return exisitingNames.toString();
    }
}