/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package symbol;

import abstractsyntaxtree.SyntaxNode;
import exceptions.CompilerException;
import teajay.runtime.exceptions.ResolveException;
import java.io.IOException;
import java.util.List;
import teajay.bytecode.exceptions.ByteCodeException;
import teajay.util.io.ClassFileRef;

/**
 *
 * @author Michael Burzan
 */
public class SymbolFacade {

    private NameResolver nameResolver;
    private LocalTable localTable;

    public SymbolFacade() {
        this.nameResolver = new NameResolver();
    }

    
    public String toString() {
        return nameResolver.toString();
    }

    public void addNameFromPackage(String className) throws CompilerException, ByteCodeException, IOException, ResolveException {
        // System.err.println("ClassName:"+className);
        this.nameResolver.addNameFromPackage(className);
    }

    public void addNameFromImport(String importName) throws CompilerException, ByteCodeException, IOException, ResolveException {
        this.nameResolver.addNamesFromImport(importName);
    }

    public void addNameFromImportNoCheck(String importName) throws CompilerException, ByteCodeException, IOException, ResolveException {
        this.nameResolver.addNamesFromImportNoCheck(importName);
    }

    public void addNameFromImportWithWildcard(String importName) throws CompilerException, ByteCodeException, IOException, ResolveException {
        this.nameResolver.addNamesFromImportWildcard(importName);
    }

    public void addNameFromStaticImport(String importName) throws CompilerException, ByteCodeException, IOException, ResolveException {
        this.nameResolver.addNamesFromStaticImport(importName);
    }

    public void addNameFromStaticImportWithWildcard(String importName) throws CompilerException, ByteCodeException, IOException, ResolveException {
        this.addNameFromImportWithWildcard(importName);
    }
    /**
     * 
     * @param id the identifier of a type
     * @return the qualified identifier
     */
    public String getFullQualifiedId(String id) {
        String qID = this.nameResolver.getQualifiedNameBySimpleName(id);
        if (qID == null && this.nameResolver.isQualifiedName(id)) {
            return id;
        }
        return qID;
    }

    public List<ClassFileRef> getStaticImports() {
        return this.nameResolver.getStaticImports();
    }

    public void setLocalTable(LocalTable table) {
        this.localTable = table;
    }

    public LocalTable getLocalTable() {
        return this.localTable;
    }
}
