/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package teajay.lang;

import teajay.util.TJType;
import teajay.util.tjattribute.TeaJayAttribute;

/**
 * Java compatiblity layer for java.lang.Object.hashCode() and equals().
 * This is the base class of all TeaJay types.
 * @author Michael Kußmann <michi@outx.de>
 */
@TJType(type = TeaJayAttribute.Type.Typedef)
public class TJObject extends Object {
    private static Number U_INT_MAX_VAL = Number.$valueOf(0xFFFFFFFF);
    private final TJObject rthis;

    @TJType(accessType = TeaJayAttribute.AccessType.Hidden)
    public TJObject() {
        this.rthis = this;
    }

    public TJObject(TJObject rthis) {
        if (rthis != null) {
            this.rthis = rthis;
        } else {
            this.rthis = this;
        }
    }

    public Number tjHashCode() {
        return Number.$valueOf(System.identityHashCode(rthis));
    }

    @Override
    @TJType(accessType = TeaJayAttribute.AccessType.Hidden)
    public int hashCode() {
        return (int) rthis.tjHashCode().and(U_INT_MAX_VAL).$long_val();
    }

    public Boolean operator_eq(Object obj) {
        return rthis == obj;
    }

    @Override
    @TJType(accessType = TeaJayAttribute.AccessType.Hidden)
    public boolean equals(Object obj) {
        if (obj == null) {
            return false;
        } else if (rthis.operator_eq(obj)) {
            return true;
        } else if (obj instanceof TJObject) {
            return ((TJObject) obj).operator_eq(rthis);
        }
        return false;
    }
    
    @TJType(accessType = TeaJayAttribute.AccessType.None)
    @Override
    public String toString() {
        return rthis.getClass().getName() + "@" + Integer.toHexString(hashCode());
    }
}