/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package teajay.lang.java;

import java.lang.reflect.Array;
import teajay.util.TJType;
import teajay.util.tjattribute.TeaJayAttribute;

/**
 *
 * @author Michael Kußmann <michi@outx.de>
 */
@TJType(accessType = TeaJayAttribute.AccessType.Hidden)
public abstract class PrimitiveArray<T> extends JavaArray<T> {

    @Override
    public String toString() {
        StringBuilder buf = new StringBuilder();
        toString(getJavaArray(), buf);
        return buf.toString();
    }

    private void toString(Object arr, StringBuilder buf) {
        if (arr == null) {
            buf.append("null");
        } else if (!arr.getClass().isArray()) {
            buf.append(arr);
        } else {
            int len = Array.getLength(arr);
            buf.append('[');
            for (int i = 0; i < len; i++) {
                toString(Array.get(arr, i), buf);
                if (i != len - 1) {
                    buf.append(", ");
                }
            }
            buf.append(']');
        }
    }

    private static boolean arrayEquals(Object arr1, Object arr2) {
        int len = Array.getLength(arr1);
        if (len == Array.getLength(arr2)) {
            for (int i = 0; i < len; i++) {
                Object a = Array.get(arr1, i);
                Object b = Array.get(arr2, i);
                if (a == null || b == null) {
                    if (a != b) {
                        return false;
                    }
                } else if (a.getClass().isArray() && b.getClass().isArray()) {
                    if (!arrayEquals(a, b)) {
                        return false;
                    }
                } else if (!a.equals(b)) {
                    return false;
                }
            }
            return true;
        }
        return false;
    }

    @Override
    public Boolean operator_eq(Object obj) {
        if (obj == null) {
            return false;
        }
        if (this.getClass().equals(obj.getClass())) {
            return arrayEquals(((PrimitiveArray) obj).getJavaArray(), this.getJavaArray());
        }
        return false;
    }
}
