/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package teajay.runtime;

import teajay.util.TJType;
import teajay.util.tjattribute.TeaJayAttribute;
import teajay.lang.Number;
import teajay.lang.java.JavaArray;
/**
 * Mappings for java.lang.System making it the TeaJay-Type "System".
 * @author Michael Kußmann <michi@outx.de>
 */
@TJType(accessType = TeaJayAttribute.AccessType.Hidden)
public final class VM {
    private VM() {
    }
    /**
     * @see System#arraycopy(java.lang.Object, int, java.lang.Object, int, int) 
     * @param src
     * @param srcPos
     * @param dest
     * @param destPos
     * @param len 
     */
    public static void arraycopy(JavaArray<?> src, Number srcPos, JavaArray<?> dest, Number destPos, Number len) {
        System.arraycopy(src.getJavaArray(), srcPos.$int_val(), dest.getJavaArray(), destPos.$int_val(), len.$int_val());
    }
    /**
     * @see System#exit(int) 
     * @param exitCode 
     */
    public static void exit(Number exitCode) {
        System.exit(exitCode.$int_val());
    }
    /**
     * @see System#currentTimeMillis() 
     * @return 
     */
    public static Number currentTimeMillis() {
        return Number.$valueOf(System.currentTimeMillis());
    }
    /**
     * @see System#nanoTime() 
     * @return 
     */
    public static Number nanoTime() {
        return Number.$valueOf(System.nanoTime());
    }
    /**
     * @see System#identityHashCode(java.lang.Object) 
     * @param obj
     * @return 
     */
    public static Number identityHashCode(Object obj) {
        return Number.$valueOf(System.identityHashCode(obj));
    }
}
