/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package teajay.util;

import teajay.util.tjattribute.TeaJayAttribute;

/**
 * A uniformly distributed random number generator.
 * @author Michael Kußmann <michi@outx.de>
 */
@TJType(type = TeaJayAttribute.Type.Typedef, accessType = TeaJayAttribute.AccessType.Final)
public abstract class RandomGenerator {
    /**
     * Creates an autoseeded random number generator.
     */
    public RandomGenerator() {}
    /**
     * Creates random number generator with the given seed.
     * Only the first 64-bit of the seed are considered.
     * @param seed a 64-bit unsigned integer
     */
    public RandomGenerator(Number seed) {}
    /**
     * Returns true or false.
     * @return true or false
     */
    public abstract Boolean nextBoolean();
    /**
     * A random floating point number between 0.0 and 1.0.
     * This method can only work with double precision.
     * @return 
     */
    public abstract teajay.lang.Number nextFloat();
    /**
     * a random integer
     * @param maxVal upper limit
     * @return a random inteer between 0 and maxVal
     */
    public abstract teajay.lang.Number nextInteger(teajay.lang.Number maxVal);
    
    /**
     * a random rational number.
     * This is equal to nextFloat().mult(magnitude).mult(nextBoolean() ? Number.ONE : Number.M_ONE)
     * @param magnitude
     * @return a random number
     */
    public abstract teajay.lang.Number nextNumber(teajay.lang.Number magnitude);
    
    /**
     * Sets the seed to seed.
     * @param seed the seed
     */
    public abstract void setSeed(teajay.lang.Number seed);
}
